;	DGDRV.CPY - digitizer driver header code
;
;	(c) Copyright 1986,88 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  1/7/1986
;	modified for driver toolkit 8/15/86
;	modified for revision 7 6/7/88 MR
;	modified for rev 8 .BIN driver format 12/1/88 MR
;
dgdrv	segment	public
	assume	cs:dgdrv,ds:dgdrv
;
dvrlvl	equ	8			;driver interface revision level number
dvrtyp	equ	2			;driver type (2=digitizer driver)
;
	if1
xfunc	=	0			;assume no extended functions
	endif
;
entry	macro				;enter driver procedure
	push	ds			;save old DS
	mov	bp,cs			;use local DS
	mov	ds,bp
	endm
;
exit	macro				;exit driver procedure
	pop	ds			;restore entry DS
	db	0CBH			;RETF
	endm
;
;	------------------------------------------------------------------
;	Required Equates Defined before INCLUDE DGDRV.CPY
;	------------------------------------------------------------------
;
;	DEVREV equ is Specific-Device driver code revision #
;	DPTI equ is # dig points per ten inches
;
;	CFGCTL equ selects by bits: (0005g std mouse, 00FAh std tablet)
;
;	   1h = device is mouse
;	   2h = device is tablet
;	   4h = acceleration sensitivity level
;	   8h = com1 /com2 serial port
;	  10h = # buttons 1..4
;	  20h = tablet pointer area
;	  40h = tablet menu area
;	  80h = tablet real area
;	8000h = call device-specific DEVCFG routine
;	
;	------------------------------------------------------------------
;	.BIN File Header & Entry Offsets Table
;	------------------------------------------------------------------
;
	org	0000h
	hlt				;signature word = xxF4h
	db	dvrtyp			;where xx = dvrtyp
	db	dvrlvl			;driver interface rev # (numeric)
	db	devrev			;device specific code revision #
	dw	xfunc			;extended functions available flag
	dw	offset enddvr+1		;# bytes in driver
	dw	cfgctl			;auto-configure bits
	dw	0FFFFh			;valid communications channels
	dw	offset dgidrn		;start of parameter table
	dw	offset DGRSC		;start of digitizer resource area
	dw	offset DGINIT		;initialize digitizer driver
	dw	offset DGEND		;end use of digitizer driver
	dw	offset DEVCFG		;device-specific config routine
	dw	offset DGREAD
	dw	offset DGMOVE		;move current position (mouse)
	dw	offset DGFIT		;set mouse pointer-area limits
	dw	offset DGCTRL		;device specific control
	dw	offset dgnada		;reserved
	dw	offset dgnada
	dw	offset dgnada
	dw	offset dgnada
;
dgnada:	stc				;flag unavailable
	db	0CBH			;RETF
;
;	------------------------------------------------------------------
;	Digitizer Driver Parameter Data Area
;	------------------------------------------------------------------
;
dgidrn	dw	1			;digitizer driver resource #
dgdpti	dw	dpti			;dots per ten inches real scale
dginum	dw	0			;total menu items available (0=none)
ofstx	dw	0			;pointer-area lower-left offsets
ofsty	dw	0
limitx	dw	2047			;pointer-area upper-left limits
limity	dw	2047
mofstx	dw	0			;menu-area lower-left offsets
mofsty	dw	0
mlimx	dw	0			;menu-area upper-left limits
mlimy	dw	0
tabnc	dw	10			;tablet menu # columns
tabnr	dw	2			;tablet menu # rows
rofstx	dw	0			;real-area lower-left offsets
rofsty	dw	0
rlimx	dw	0			;real-area upper-left limits
rlimy	dw	0
dgaslv	dw	6			;acceleration sensitivity
nbtns	dw	4			;# buttons in use
	if	(cfgctl AND 2) EQ 2
dgflag	dw	81h			;flag bits 1=isdig,80h=needadj
	else
dgflag	dw	0			;flag bits (mouse ready to go)
	endif
port	dw	0			;com# port (0..2)
dgchnl	dw	0			;communications channel
idata	dw	0E7h			;com# port init data
;
;	------------------------------------------------------------------
;	DGMOVE - change current position to SI,DI
;	set JNC if change was made, else JNC
;	digitizers make no change
;	------------------------------------------------------------------
;
DGMOVE:	entry
	if	(cfgctl AND 1) EQ 1
	mov	digx,si
	mov	digy,di
	clc				;change was made
	else
	stc				;no change
	endif
	exit
;
;	------------------------------------------------------------------
;	DGFIT - change pointer-area limits to SI,DI
;	set JNC if change was made, else JNC
;	digitizers make no change
;	------------------------------------------------------------------
;
DGFIT:	entry
	if	(cfgctl AND 1) EQ 1
	mov	limitx,si		;set limits
	mov	limity,di
	shr	si,1			;position in middle
	shr	di,1
	mov	digx,si
	mov	digy,di
	clc				;change was made
	else
	stc				;no change
	endif
	exit
;
