;	DGKURTC.ASM - Kurta Series I, II, & IS/ONE Digitizer Driver
;
;	(c) Copyright 1986,88 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  8/17/86
;	Modified for continuous mode 2/3/88
;	modified for tablet areas 5/28/88 MR
;	fixed help message & wire diag. 8/8/88 SB
;	new .BIN driver format 12/06/88 SB
;	devrev=1: added BUTTONS function 03/10/89 SB
;
	NAME	DGKURTC
;
devrev	equ	1			;driver code revision #
dpti	equ	2000			;dots per ten inches
cfgctl	equ	80EAH			;DEVCFG, serial tablet, all areas
;
	include	DGDRV.CPY
;
;	---------------------------------------------------------------
;	DEVCFG - Determine Number of Buttons
;	(All CONIN/CONOUT must use DOS functions)
;	---------------------------------------------------------------
;
DEVCFG:	entry				;quiz for 4 or more buttons
	mov	xbtns,0			;assume table for <4 buttons
	mov	dx,offset btqz		;select buttons message
	mov	ah,9
	int	21H
	mov	ah,7			;get key
	int	21H
	push	ax			;echo choice
	mov	dl,al
	mov	ah,2
	int	21H
	pop	ax
	cmp	al,13			;CR?
	jz	short btxit
	cmp	al,'Y'			;answer Y?
	jz	btchg
	cmp	al,'y'
	jnz	btxit
btchg:	mov	xbtns,1			;table for >4 buttons
btxit:	mov	dx,offset crlf		;append crlf
	mov	ah,9
	int	21H
	exit
;
crlf	db	13,10,'$'
;
;	------------------------------------------------------------------
;	DGINIT - initialize digitizer driver
;	------------------------------------------------------------------
;
DGINIT:	entry
	mov	dx,port			;init COM port
	mov	ax,00E7H		;9600 baud, 8 data 2 stop no parity
	int	14H
	sub	ax,ax			;return AX 0 = OK
	exit
;
;	------------------------------------------------------------------
;	DGEND - stop using digitizer
;	------------------------------------------------------------------
;
DGEND:	entry
	exit
;
;	------------------------------------------------------------------
;	DGREAD - read digitizer position & buttons
;
;	AX = DGDAV (digitizer data available status)
;		=0 no dig data available
;		=1 button up tracking coordinate
;		=2 LEFT button selected (or only button)
;		=3 RIGHT button selected
;		=4 button down tracking coordinate
;		=5 SI is LEFT-picked tablet menu item # 1..DGINUM
;	SI = DIGX position for AX=1,2,3 or 4
;	DI = DIGY position for AX=1,2,3 or 4
;	------------------------------------------------------------------
;
DGREAD:	entry
	mov	dgdav,0			;clear data available flag
	mov	ax,300H			;setup to test for DAV
	mov	dx,port
	int	14H
	test	ah,1			;DAV?
	jz	dgab			;no, get out of here
;	synchronize on start of data byte
dskip:	mov	dx,port			;get data byte
	mov	ah,2
	int	14H
	test	ah,0AH			;error?
	jnz	dgab			;yes
	test	al,80H			;start sequence byte?
	jz	dgab			;no - wait to sync on start data
	mov	dstat,ax		;save switch status
;	allow menu function to be RIGHT button
	test	al,40H			;menu data?
	jz	dgrdx			;no - go read X data
	test	al,03CH			;any switch down?
	jz	dgab			;no - no data available
	mov	dgdav,3			;menu is RIGHT button
	jmp	dgnd
;	read X data value
dgrdx:	and	ax,3			;x highest bits
	ror	ax,1
	ror	ax,1
	mov	digx,ax
	mov	dx,port			;read X med data
	mov	ah,2
	int	14H
	test	ax,0A80H		;error?
	jnz	dgab			;yes
	and	ax,7FH			;mask to data bits
	mov	cl,7			;shift into position
	shl	ax,cl
	or	digx,ax			;and save it
	mov	dx,port			;read X low data
	mov	ah,2
	int	14H
	test	ax,0A80H		;error?
	jnz	dgab			;yes
	and	ax,7FH			;mask to data bits
	or	digx,ax			;put high and low bits together
	jmp	short dgrdy
dgab:	mov	ax,0			;no valid data
	exit
;	read Y data value
dgrdy:	mov	dx,port			;read Y low data
	mov	ah,2
	int	14H
	test	ax,0A80H		;error?
	jnz	dgab			;yes
	and	ax,3			;get highest bits
	ror	ax,1
	ror	ax,1
	mov	digy,ax
	mov	dx,port			;read Y med data
	mov	ah,2
	int	14H
	test	ax,0A80H		;error?
	jnz	dgab			;yes
	and	ax,7FH			;mask to data bits
	mov	cl,7			;shift into position
	shl	ax,cl
	or	digy,ax			;and save it
	mov	dx,port			;read Y low data
	mov	ah,2
	int	14H
	test	ax,0A80H		;error?
	jnz	dgab			;yes
	and	ax,7FH			;mask to data bits
	or	digy,ax			;put high and low bits together

;	determine data available flag
;	BL= SM8421xx S=Sync, M=menu
	mov	dgdav,1			;assume tracking coordinate
swchk:	mov	bx,dstat		;get switch data
	test	bx,03Ch			;any buttons down?
	jz	dgnd
savst:	shr	bx,1			;BX = byte index 
	shr	bx,1			;
	and	bx,0FH			;allow 16 entries
	test	xbtns,1
	jz	p3
	mov	al,pat4[bx]
	jmp	short try2
p3:	mov	al,pat3[bx]
try2:	sub	ah,ah
	cmp	ax,'L'			;"left" button?
	jnz	try3			;no
	mov	dgdav,2			;yes - LEFT button down
	jmp	short tkchk		;  and given priority
try3:	cmp	ax,'R'			;"right" button?
	jnz	trybtn			;no - tracking pen up
	mov	dgdav,3			;yes - RIGHT button down
	jmp	short tkchk		;  2nd priority
trybtn:	mov	dgdav,5
	mov	si,ax
tkchk:	mov	ax,dgdav		;check if tracking button down
	cmp	ax,prior
	jnz	dgnd
tbdn:	mov	dgdav,4			;yes - tracking button down
	jmp	short rtndta
dgnd:	mov	ax,dgdav		;return data
	cmp	ax,0
	jz	rtndta			;save dgdav for prior if not=0
	mov	prior,ax
	cmp	ax,5
	jz	rtbtn
rtndta:	mov	si,digx			;get dig coordinate
	mov	di,digy
rtbtn:	mov	ax,dgdav		;dig status
	exit
;
;	-------------------- local data items ---------------------------
;
	align	2
dgdav	dw	0			;digitizer data available flag
digx	dw	1000			;current X position
digy	dw	750			;current Y position
dstat	dw	0			;sync byte
prior	dw	1			;for left/right buttons
pribtn	dw	0			;for extra buttons
xbtns	dw	0			;set =1 if 4 or more buttons
;
pat3	db	0,'L','R',2,1,3,4,5
pat4	db	0,1,'L',8,2,5,6,7,'R',9,10,11,12,13,14,15,16
;	-----------------------------------------------------------------
;	Digitizer Driver Resource Data
;
DGRSC	dw	2			;# of resources
	dw	offset dgid		;1=driver ID text
	dw	offset btqz		;2=button quiz
;
dgid	db	'Kurta Series I, II, or IS\ONE Digitizer,',0
btqz	db	13,10,'Does your cursor have four or more buttons? Y[N]: $'
;
	include	DGLAST.CPY

