;	DGMSOFT.ASM - MicroSoft Mouse Digitizer Driver
;
;	(c) Copyright 1986 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  1/7/1986
;	modified for Acc. sensing patch 5/25/86
;	modified for level 6 drivers 5/28/88 MR
;	modified for rev 8 .BIN drivers 12/1/88 MR
;
	NAME	DGMSOFT
;
devrev	equ	0			;driver revision #
dpti	equ	0			;dots per ten inches
cfgctl	equ	0005h			;standard mouse
	include	DGDRV.CPY
;
;	------------------------------------------------------------------
;	DGINIT - initialize digitizer driver
;	------------------------------------------------------------------
;
DGINIT:	entry
	mov	ax,0			;mouse installed?
	int	33H			;(and mouse reset)
	cmp	ax,0			;0=no, -1=yes
	jnz	msOK
	mov	ax,3			;DGDRV not installed
	exit
msOK:	sub	ax,ax			;return AX 0 = OK
	exit
;
;	------------------------------------------------------------------
;	DGEND - stop using digitizer
;	------------------------------------------------------------------
;
DGEND:	entry
	exit
;
;	------------------------------------------------------------------
;	DGREAD - read digitizer position & buttons
;
;	AX = DGDAV (digitizer data available status)
;		=0 no dig data available
;		=1 button up tracking coordinate
;		=2 LEFT button selected (or only button)
;		=3 RIGHT button selected
;		=4 button down tracking coordinate
;		=5 SI is LEFT-picked tablet menu item # 1..DGINUM
;	SI = DIGX position for AX=1,2,3 or 4
;	DI = DIGY position for AX=1,2,3 or 4
;	------------------------------------------------------------------
;
DGREAD:	entry
;	set acceleration sense thresholds
	mov	bx,dgaslv		;lookup desired acceleration
	shl	bx,1			;word index
	mov	ax,aclvl[bx]
	mov	dadjh,ax		;set acceleration triger level
	neg	ax
	mov	dadjl,ax
;	get current mouse status
	mov	ax,11			;read mouse motion counters
	int	33H
	mov	si,cx			;store change in X
	neg	dx			;invert Y so + is up
	mov	di,dx			;store change in Y
	mov	ax,3			;get button status
	int	33H			;to BX
;	set AX (DGDAV) by button status
	mov	dx,priorbu		;prior call button up flag
	test	bx,1			;button 1 down?
	jnz	b1dn
	test	bx,2			;button 2 down?
	jnz	b2dn
	mov	dx,11			;no btn down;priorbu=1
	mov	ax,1			;dgdav=1 (tracking coordinate)
	jmp	short spri		;go set priorbu
b1dn:	test	dx,1			;first button down?
	jz	bdtk			;no - btn down tracking
	mov	ax,2			;dgdav=2 (LEFT button down)
	sub	dx,dx			;priorbu=0
	jmp	short spri		;go set priorbu
b2dn:	test	dx,1			;first button down?
	jz	bdtk			;no - btn down tracking
	mov	ax,3			;dgdav=3 (RIGHT button down)
	sub	dx,dx			;priorbu=0
	jmp	short spri		;go set priorbu
bdtk:	mov	ax,4			;button down tracking
spri:	mov	priorbu,dx		;store new prior button up
;	larger delta for fast motion
	xchg	ax,si			;adjust delta x
	call	deladj
	xchg	ax,si
	xchg	ax,di			;adjust delta y
	call	deladj
	xchg	ax,di
;	update digitizer coordinates
	add	si,digx			;digx+deltax
	add	di,digy			;digy+deltay
;	limit location to valid range
	cmp	si,0			;if digx<0
	jge	xge0
	mov	si,0			;then digx=0
xge0:	cmp	si,limitx		;if digx>limitx
	jle	xlem
	mov	si,limitx		;then digx=limitx
xlem:	cmp	di,0			;if digy<0
	jge	yge0
	mov	di,0			;then digy=0
yge0:	cmp	di,limity		;if digy>limity
	jle	ylem
	mov	di,limity		;then digy=limity
ylem:	mov	digx,si			;save new coordinate
	mov	digy,di
	mov	dgdav,ax		;and button status
	exit
;
deladj	proc	near
	cmp	dgaslv,1		;reduce delta?
	ja	nored
	jz	red2
	sar	ax,1			;dgaslv=0, /4
red2:	sar	ax,1			;dgaslv=1, /2
nored:	cmp	dgaslv,2		;done if = 0,1, or 2
	ja	upadj
	ret
upadj:	cmp	ax,dadjl		;low threshold?
	jle	doup
	cmp	ax,dadjh		;high threshold?
	jge	doup
	ret				;no further adjust
doup:	shl	ax,1			;*2 delta
	cmp	dgaslv,8		;*4 if setting is 8 or 9
	jl	daxt
	shl	ax,1
daxt:	ret
deladj	endp
;
;	------------------------------------------------------------------
;	Local Data Items
;	------------------------------------------------------------------
;
	align	2
aclvl	dw	9999,9999,9999,16,8,5,3,2,3,2
dgdav	dw	0			;digitizer data available flag
digx	dw	1048			;current X position
digy	dw	1048			;current Y position
priorbu	dw	1			;prior button up flag
dadjl	dw	-12			;low delta adjust trigger
dadjh	dw	12			;high delta adjust trigger
;
;	------------------------------------------------------------------
;	Digitizer Driver Resource Data
;	------------------------------------------------------------------
;
DGRSC	dw	1			;# of resources
	dw	offset dgid		;1=device id
;
dgid	db	'Microsoft(R) or compatible mouse',0
;
	include	DGLAST.CPY
