;	DSEGA.ASM - EGA 640x350x16 color display driver
;	256K RAM on EGA card is required.
;
;	(c) Copyright 1986,7,8 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  3/5/1986
;	added color range limiting 4/23/86
;	fixed setccm DS bug in DSSAVE/DSREST 5/1/86
;	DSSAVE/DSREST saves color,style, and mode 5/8/86
;	modified for driver toolkit 8/16/86
;	Added EGAFHL routine 9/4/86
;	modified for variable sized prompt text  9/23/86
;	added pattern fill support 4/15/87 MR
;	level 6 revisions 3/6/88 MR
;	level 7 animate functions added 8/13/88 MR
;	rev 8 .BIN driver format 10/4/88 MR
;
	name	DSEGA
;
devrev	equ	0			;driver code revision
cfgctl	equ	3			;configuration select bits
	include	DSDRV.CPY
;
;	EGA register set macro
;
SetReg	macro	port,sel,value
	mov	dx,port			;select register
	mov	al,sel
	out	dx,al
	inc	dx			;select data
	mov	al,value
	out	dx,al
	endm
;
spccm	dw	sdrw			;current mode & color set routine
;
;	DSINIT - initialize the display driver
;
;	This will only be called once per program execution
;	before any other display driver calls.  It must:
;	1 - test for correct display hardware available
;	2 - allocate any required buffer memory from DOS
;	3 - perform global & local variable initialization
;	4 - initialize the display hardware
;	5 - select graphics display mode
;	6 - clear the screen
;	7 - set AX=error code:
;	    0=ok
;	    1=not enough memory
;	    2=not enough EMM memory
;	    3=device unavailable/unuseable
;	    4=CGEN resource missing
;
DSINIT:	entry
	mov	word ptr DVRSVC,di	;save pointer to driver services
	mov	word ptr DVRSVC+2,ES
	mov	ax,CGENR		;load char gen resource
	mov	bx,0			;into alloc segment
	mov	dx,0			;RSCLD function
	call	dword ptr DVRSVC	;do it
	jnc	short ok1
	mov	ax,4			;missing CGEN resource
	exit
ok1:	mov	ax,ES:[bx]		;get segment from handle
	mov	CGSeg,ax		;save segment
	mov	word ptr CGHdl,bx	;save handle
	mov	ax,ES
	mov	word ptr CGHdl+2,ax
	call	setgrm			;select graphics screen mode
	call	cscreen			;clear the screen
	call	spccm			;set pointers for init color and mode
	sub	ax,ax			;ax=0: device available for use
	exit
;
;	DSEND - end use of the display driver
;
;	This will only be called once per program execution
;	before program termination.  It must:
;	1 - restore any interrupt vectors changed by this driver
;	2 - restore the display to DOS-compatible text mode
;	3 - clear the text screen
;	4 - release any allocated buffer memory
;
DSEND:	entry
	call	settxm			;select text screen mode
	les	bx,CGHdl		;release CGEN resource
	mov	dx,1
	call	dword ptr DVRSVC	;do it
	exit
;
;	DSGRM - set screen to graphics display mode
;
DSGRM:	entry
	call	setgrm			;select graphics display
	call	srest			;restore saved screen
	exit
;
;	DSTXM - set screen to DOS-compatible text mode
;
DSTXM:	entry
	call	ssave			;save graphics display
	call	settxm			;select text display
	exit
;
ANIMF	db	0			;animation flag
;
;	DSAUPD - Animation Update
;
;	Copy drawing buffer to 2nd screen buffer
;	Force display of 2nd screen buffer
;
DSAUPD:	entry
	call	ssave			;copy to save buffer at 8000h
	test	animf,1			;doing animation?
	jz	short stanim
	exit				;yes
stanim:	mov	animf,1			;no - flag animation started
	mov	dx,3DAh			;CRTC status port
wvrs:	in	al,dx			;wait for start of vert retrace
	test	al,8
	jz	short wvrs
wvre:	in	al,dx			;wait for end of vert retrace
	test	al,8
	jnz	short wvre
	cli				;disable interrupts
	sub	dl,6			;CRTC I/O port
	mov	ax,800Ch		;start adrs high
	out	dx,ax
	mov	ax,000Dh		;start adrs low
	out	dx,ax
	sti				;enable interrupts
	exit
;
;	DSAEND - Animation End
;
;	Force display of normal draw screen buffer
;
DSAEND:	entry
	test	animf,1			;doing animation?
	jnz	short eanim
	exit				;no
eanim:	mov	animf,0			;yes - flag animation ended
	mov	dx,3DAh			;CRTC status port
ewvrs:	in	al,dx			;wait for start of vert retrace
	test	al,8
	jz	short ewvrs
ewvre:	in	al,dx			;wait for end of vert retrace
	test	al,8
	jnz	short ewvre
	cli				;disable interrupts
	sub	dl,6			;CRTC I/O port
	mov	ax,000Ch		;start adrs high
	out	dx,ax
	mov	ax,000Dh		;start adrs low
	out	dx,ax
	sti				;enable interrupts
	exit
;
;	DSCLEAR - clear the graphics screen
;
DSCLEAR: entry
	call	cscreen			;clear the screen
	exit
;
;	DSERA - select draw background mode (erase)
;
DSERA:	entry
	mov	spccm,offset sera
	call	sera
	exit
;
;	DSDRAW - select draw specified color mode
;
DSDRAW:	entry
	mov	spccm,offset sdrw
	call	sdrw
	exit
;
;	DSINV - select draw inverted pixel mode
;
DSINV:	entry
	mov	spccm,offset sxor
	call	sxor
	exit
;
;	DSCOLOR - specify current drawing color & background
;
DSCOLOR: entry
	mov	byte ptr bkgnd,cl	;save background color
	mov	byte ptr CurClr,al	;set new current color
	call	spccm			;set pointers
	exit
;
	include	SDMEGA.CPY
	include	EGAPAL.CPY
;
;	DSSAVE - save the graphics screen
;
DSSAVE:	entry
	call	ssave			;save current screen
	exit
;
DSREST:	entry
	call	srest			;restore prior screen
	exit
;
DskTyp	equ	2
	include	EGADSK.CPY
;
;	----------------- local use procedures ----------------------
;
setgrm	proc	near			;set screen to graphics mode
	push	bp
	mov	ax,90H			;select 640x350 color mode
	int	10H
	mov	ax,0500H		;select page 0 to display
	int	10H
	call	SETPAL			;setup our palette
	pop	bp
	ret
setgrm	endp
;
settxm	proc	near			;set screen to text mode
	push	bp
	mov	ax,3			;reset to text mode
	int	10H
	call	DOSPAL			;restore normal palette
	pop	bp
	ret
settxm	endp
;
cscreen	proc	near			;clear the screen
	SetReg	3CEH,8,0FFH		;enable all pixels to be changed
	SetReg	3CEH,1,0H		;disable S/R (direct data write)
	SetReg	3CEH,3,0H		;write normal data
	mov	ax,dwseg		;point to display
	mov	es,ax
	mov	cx,sbsiz		;screen size in words
	sub	ax,ax			;clear data
	mov	di,ax
	cld				;scan upwards
	rep	stosw			;clear screen
	call	spccm			;set current color and mode
	ret
cscreen	endp
;
sxor	proc	near
	SetReg	3CEH,1,0H		;disable Set/Reset Mode
	SetReg	3CEH,3,18H		;write xor data
	ret
sxor	endp
;
sdrw	proc	near
	SetReg	3CEH,1,0FH		;enable Set/Reset Mode
	SetReg	3CEH,0,<byte ptr CurClr>;color to Set/Reset data
	SetReg	3CEH,3,0H		;write normal data
	ret
sdrw	endp
;
sera	proc	near
	SetReg	3CEH,1,0FH		;enable Set/Reset Mode
	SetReg	3CEH,0,<byte ptr bkgnd>	;color to Set/Reset data
	SetReg	3CEH,3,0H		;write normal data
	ret
sera	endp
;
ssave	proc	near
	mov	cx,sbsiz		;buffer size in words
	shl	cx,1			;in bytes
	push	ds
	mov	ax,dwseg		;display memory segment
	mov	ds,ax
	mov	es,ax
	mov	si,0000H		;from zero (active page)
	mov	di,8000H		;to inactive page
	cld				;upscan
	SetReg	3CEH,8,0		;copy all write from prior read
	SetReg	3CEH,1,0H		;disable S/R (direct data write)
	SetReg	3CEH,3,0H		;write normal data
	rep	movsb			;copy all data (32-bit copies)
	pop	ds
	call	spccm			;set current color and mode
	ret
ssave	endp
;
srest	proc	near
	mov	cx,sbsiz		;buffer size in words
	shl	cx,1			;in bytes
	push	ds
	mov	ax,dwseg		;display memory segment
	mov	ds,ax
	mov	es,ax
	mov	si,8000H		;from inactive page
	mov	di,0000H		;to active page
	cld				;upscan
	SetReg	3CEH,8,0		;copy all write from prior read
	SetReg	3CEH,1,0H		;disable S/R (direct data write)
	SetReg	3CEH,3,0H		;write normal data
	rep	movsb			;copy all data (32-bit copies)
	pop	ds
	call	spccm			;set current color and mode
	ret
srest	endp
;
;	----------------- select prompt text size ----------------------
;
SELSPT:	entry
	mov	tccw,1
	mov	nrast,12
	mov	dstcol,80
	mov	dstlin,29
	mov	dstxh,12
	mov	dstxw,8
	mov	cgenr,402h		;cgen resource #
	mov	ax,offset ltbls
	mov	ltblptr,ax
	exit
;
SELLPT:	entry
	mov	tccw,2
	mov	nrast,15
	mov	dstcol,40
	mov	dstlin,23
	mov	dstxh,15
	mov	dstxw,16
	mov	cgenr,401h		;cgen resource #
	mov	ax,offset ltbll
	mov	ltblptr,ax
	exit
;
;	--------------------- local data items -------------------------
;
	align	4
dvrsvc	dd	0			;pointer to driver services
CGHdl	dd	0			;CGEN rsc handle
dwseg	dw	0A000H			;screen drawing segment
sbsiz	dw	36B0H			;#words per screen page
;
curclr	dw	3			;current color #
curpat	dw	0FFFFH			;current line pattern
bkgnd	dw	0			;current background color
;
pmasks	db	80H,40H,20H,10H,8,4,2,1	;pixel select masks
;
;	LTBL(line*2)=offset of text lines 0(top) thru 39(bottom)
;
ltbls	dw 0000H,03C0H,0780H,0B40H,0F00H,12C0H,1680H,1A40H,1E00H,21C0H
	dw 2580H,2940H,2D00H,30C0H,3480H,3840H,3C00H,3FC0H,4380H,4740H
	dw 4B00H,4EC0H,5280H,5640H,5A00H,5DC0H,6180H,6540H,69A0H
;
ltbll	dw 0,1200,2400,3600,4800,6000,7200,8400,9600,10800
	dw 12000,13200,14400,15600,16800,18000,19200,20400,21600,22800
	dw 24000,25200,26800
;
;	---------------- global data items -------------------------
;
dsidrn	dw	1			;device id resource #
dsxmax	dw	639			;# X dots -1
dsymax	dw	349			;# Y dots -1
dsyar	dw	77			;aspect ratio Y/X
dsxar	dw	103
dstcol	dw	80			;# text columns
dstlin	dw	29			;# text lines
dsnclrs	dw	15			;max color # (0-ncolor)
dstxh	dw	12			;text height in pixels
dstxw	dw	8			;text width in pixels
dsflags	dw	4003h			;circles & arcs,animation
dsf2	dw	0			;more flags
slow	db	0			;slow function bits
iconsc	db	2			;icon size code (16x16)
	db	16 dup (0)		;reserved for future use
;
DSRSC	dw	1			;total # of resources
	dw	offset dsid		;1=device id text
;
dsid	db	'256K EGA in 640x350 16-color mode',0
;
	include	DSLAST.CPY
