;	DSHER.ASM - Hercules Graphics Card display driver
;
;	(c) Copyright 1986 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  3/5/1986
;	added color range limiting 4/23/86
;	added DSSAVE/DSREST mode,color,pattern save 5/9/86
;	modified 8/16/86 for driver toolkit compatibility
;	added small text option 9/24/86
;	added background logic to SDCSP/BWFHL 3/17/87 MR
;	level 6 revisions 3/14/88 MR
;	level 7 revisions 8/28/88 SB
;	new .BIN driver format 10/18/88 SB
;
	name	DSHER
;
devrev	equ	0			;driver code revision #
cfgctl	equ	3			;config select bits
;
	include	DSDRV.CPY
;
;	DSINIT - initialize the display driver
;
;	This will only be called once per program execution
;	before any other display driver calls.  It must:
;	1 - test for correct display hardware available
;	2 - allocate any required buffer memory from DOS
;	3 - perform global & local variable initialization
;	4 - initialize the display hardware
;	5 - select graphics display mode
;	6 - clear the screen
;	7 - set AX=error code:
;	    0=ok
;	    1=not enough memory
;	    2=not enough EMM memory
;	    3=device unavailable/unuseable
;	    4=CGEN resource missing
;
DSINIT:	entry
	mov	word ptr DVRSVC,di	;save pointer to driver services
	mov	word ptr DVRSVC+2,ES
	mov	ax,CGENR		;load char gen resource
	mov	bx,0			;into alloc segment
	mov	dx,0			;RSCLD function
	call	dword ptr DVRSVC	;do it
	jnc	short ok1
	mov	ax,4			;missing CGEN resource
	exit
ok1:	mov	ax,ES:[bx]		;get segment from handle
	mov	CGSeg,ax		;save segment
	mov	word ptr CGHdl,bx	;save handle
	mov	ax,ES
	mov	word ptr CGHdl+2,ax
	call	setgrm			;select graphics screen mode
	call	cscreen			;clear the screen
	sub	ax,ax			;ax=0: device available for use
	exit
;
;	DSEND - end use of the display driver
;
;	This will only be called once per program execution
;	before program termination.  It must:
;	1 - restore any interrupt vectors changed by this driver
;	2 - restore the display to DOS-compatible text mode
;	3 - clear the text screen
;	4 - release any allocated buffer memory
;
DSEND:	entry
	call	settxm			;select text screen mode
	les	bx,CGHdl		;release CGEN resource
	mov	dx,1
	call	dword ptr DVRSVC	;do it
	exit
;
;	DSGRM - set screen to graphics display mode
;
DSGRM:	entry
	call	setgrm			;select graphics display
	call	srest			;restore saved screen
	exit
;
;	DSTXM - set screen to DOS-compatible text mode
;
DSTXM:	entry
	call	ssave			;save graphics display
	call	settxm			;select text display
	exit
;
;	DSCLEAR - clear the graphics screen
;
DSCLEAR: entry
	call	cscreen			;clear the screen
	exit
;
;	DSERA - select draw black mode (erase)
;
DSERA:	entry
	mov	CurMode,0		;select erase mode
	exit
;
;	DSDRAW - select draw specified color mode
;
DSDRAW:	entry
	mov	CurMode,1		;select draw mode
	exit
;
;	DSINV - select draw inverted pixel mode
;
DSINV:	entry
	mov	CurMode,2		;select xor mode
	exit
;
;	DSCOLOR - specify current drawing & background color
;
DSCOLOR: entry
	mov	bkgnd,cx		;save background color
	mov	CurClr,ax		;set new current color
	exit
;
;	---------------- Set Dot Macros used by SDSUBS --------------
;
SDGPM	macro				;get pixel modification masks
	mov	bx,si			;X mod 8 =
	and	bx,7			;bit position in byte
	shl	bx,1			;word index
	mov	dx,bmask[bx]		;DH=and mask; DL=or mask
	endm
;
SDCBO	macro				;calculate byte offset to change
	mov	bx,dsymax		;invert y value (0 at bottom)
	sub	bx,di
	shl	bx,1			;word index
	mov	bx,ytbl[bx]		;lookup line offset
	mov	bp,si			;work copy of X
	shr	bp,1			;+ x/8
	shr	bp,1
	shr	bp,1
	add	bx,bp			;ES:[bx] = byte to change
	mov	xscan,si		;save x position
	endm
;
SDCSP	macro				;change selected pixel
	local	isx,ise,sdxt,isdel,isdrw
	cmp	CurMode,1		;xor mode?
	ja	isx
	jb	ise
	cmp	CurClr,0		;draw black?
	jz	isdel
isdrw:	or	ES:[bx],dl		;set dot
	jmp	short sdxt
isx:	xor	ES:[bx],dl		;invert dot
	jmp	short sdxt
ise:	cmp	bkgnd,0			;black background?
	jnz	isdrw
isdel:	and	ES:[bx],dh		;clear dot
sdxt:
	endm
;
SDUPL	macro				;move offset up one line
	local	upl
	sub	bh,20H			;decrement interleave
	test	bh,80H			;detect borrow?
	jz	upl
	and	bh,7FH			;yes - remove borrow
	sub	bx,5AH			;then bump line offset
upl:
	endm
;
SDDNL	macro				;move offset down one line
	local	dnl
	add	bh,20H			;advance interleave
	and	bh,7FH			;remove carry
	test	bh,60H			;if now = 0
	jnz	dnl
	add	bx,5AH			;then bump line offset
dnl:
	endm
;
SDUPC	macro				;move right one pixel
	local	upc
	ror	dh,1			;advance and mask
	ror	dl,1			;advance or color mask
	inc	xscan			;bump column nr
	test	xscan,7			;if new byte
	jnz	upc
	inc	bx			;then bump offset
upc:
	endm
;
SDDNC	macro				;move right one pixel
	local	dnc
	rol	dh,1			;advance and mask
	rol	dl,1			;advance or color mask
	test	xscan,7			;if new byte
	jnz	dnc
	dec	bx			;then bump offset
dnc:	dec	xscan			;bump column nr
	endm
;
;	CCHRO - calculate character offset to BX; DX=line, AX=column
;
CCHRO	macro				;calculate char offset
	call	cchros
	endm
;
cchros	proc	near
	mov	bx,dx			;line number
	shl	bx,1			;word index
	add	bx,ltblptr		;lookup line offset
	mov	bx,[bx]
	add	bx,ax			;+cposn
	cmp	tccw,2			;*2?
	jnz	notx2
	add	bx,ax			;yes
notx2:	ret
cchros	endp
;
;	GCHR - generate character from DS:[SI] cgen table, ES:BX=chr cell
;	BP= inversion mask 0 or -1; CX = NRAST value for loop; DX=tccw
;
GCHR	macro				;generate character
	local	gc1,gc2,gcxt
	cmp	dx,2			;word or byte?
	jz	short gc2
gc1:	lodsb				;get raster line
	xor	ax,bp			;invert if needed
	mov	ES:[bx],al		;set it into display
	sddnl				;move down to next raster
	loop	short gc1		;loop for entire character
	jmp	short gcxt
gc2:	lodsw				;get raster line
	xor	ax,bp			;invert if needed
	xchg	al,ah			;byte swap
	mov	ES:[bx],ax		;set it into display
	sddnl				;move down to next raster
	loop	short gc2		;loop for entire character
gcxt:
	endm
;
;	INVCR - invert character raster line ES:[BX], CX characters
;
INVCR	macro
	local	inv1,inv2,invc
	cmp	tccw,2			;word or byte?
	jz	inv2
inv1:	not	byte ptr ES:[bx]	;invert character raster
	inc	bx
	loop	inv1
	jmp	short invc
inv2:	not	word ptr ES:[bx]	;invert character raster
	add	bx,2			;next character
	loop	inv2			;for all char posns
invc:
	endm
;
;	CLRCR - clear character raster line ES:[BX],CX characters
;
CLRCR	macro
	local	clr1,clr2,clrc
	sub	ax,ax			;clear to value
	cmp	tccw,2			;word or byte?
	jz	clr2
clr1:	mov	ES:[bx],al		;clear chr raster
	inc	bx
	loop	clr1
	jmp	short clrc
clr2:	mov	ES:[bx],ax		;clear chr raster
	add	bx,2			;bump chr cell
	loop	clr2			;for all char positions
clrc:
	endm
;
tccw	dw	1			;text char cell width in bytes
nrast	dw	12			;raster lines per character
CGENR	dw	402h			;character generator resource #
LTBLPTR	dw	offset ltbls		;line offset table
;
LSScl	equ	2			;line style scale
;
;	SDFHL - fast horizontal line
;
;	ES:[bx] = leftmost byte offset
;	BP = X1 scanner, if needed
;	CX = pixel count
;
sdfhl	macro
	call	BWFHL
	endm
;
FILCSP	macro				;change selected pixels (AL) for FILL
	local	inv,drw,era,del,ok
	cmp	CurMode,1		;DSERA?
	ja	inv
	jb	era
	cmp	CurClr,0		;color black?
	jz	del
drw:	or	byte ptr ES:[bx],al	;change actual pixels
	jmp	short ok
inv:	xor	byte ptr ES:[bx],al	;invert actual pixels
	jmp	short ok
era:	cmp	bkgnd,0			;black background?
	jnz	drw
del:	not	al			;erase actual pixels
	and	byte ptr ES:[bx],al
ok:
	endm
;
BWFHL	proc	near
	mov	si,scanyl		;select current lines' pattern
	and	si,07H
	mov	ah,FilPat[si]
	mov	si,bp			;x column position
FHL:	and	si,7			;mod 8 = left byte pixel mask index
	sub	cx,8			;remove left byte pixels from count
	add	cx,si
	jge	ntsngl			;more than one byte changes?
	mov	al,lbpmt[si]		;set left byte pixel mask
	and	al,ah			;(pattern mask)
	lea	si,rbpmt+8		;less unchanged right pixels
	add	si,cx	
	and	al,[si]
	FILCSP
	ret
ntsngl:	mov	di,cx			;right byte pixel mask index
	and	di,7
	shr	cx,1			;remove right byte pixels from count
	shr	cx,1
	shr	cx,1
	mov	al,lbpmt[si]		;set left byte pixel mask
	and	al,ah			;(pattern mask)
	FILCSP
	inc	bx			;bump byte to change offset
	jcxz	chgrb
chgmb:	mov	al,ah			;pixel mask for middle bytes
	FILCSP
	inc	bx			;bump byte to change offset
	loop	chgmb			;loop for all middle bytes
chgrb:	mov	al,rbpmt[di]		;set right byte pixel mask
	and	al,ah			;(pattern mask)
	FILCSP
	ret
bwfhl	endp
;
lbpmt	db	0FFH,07FH,03FH,01FH,00FH,007H,003H,001H
rbpmt	db	000H,080H,0C0H,0E0H,0F0H,0F8H,0FCH,0FEH
;
LSScl	equ	2			;line style scale
	include SDSUBS4.CPY
	include	VTEXT.CPY
;
;	DSSAVE - save the graphics screen
;
DSSAVE:	entry
	call	ssave
	exit
;
;	DSREST - restore the saved screen
;
DSREST:	entry
	call	srest
	exit
;
;	----------------- local use procedures ----------------------
;
setgrm	proc	near			;set screen to graphics mode
	mov	dx,03BFH		;set configuration = FULL
	mov	al,03H
	out	dx,al
	mov	al,2			;change to graphics mode
	lea	si,gtable
	mov	bx,0
	mov	cx,4000H
	call	setmd
	ret
setgrm	endp
;
settxm	proc	near			;set screen to text mode
	mov	al,20H			;select text mode
	lea	si,ttable
	mov	bx,720H
	mov	cx,2000H
	call	setmd
	ret
settxm	endp
;
setmd	proc	near
	cld				;upscan
	push	ax
	push	bx
	push	cx
	mov	dx,03B8H		;screen off for mode change
	out	dx,al
	mov	dx,03B4H		;init 6845
	mov	cx,12			;12 parameters
	xor	ah,ah			;start reg 0
setpd:	mov	al,ah			;send reg #
	out	dx,al
	inc	dx			;data value
	lodsb
	out	dx,al
	inc	ah			;bump reg #
	dec	dx			;next register
	loop	setpd
	pop	cx			;clear the buffer
	mov	es,dwseg
	xor	di,di
	pop	ax			;clear to value
	rep	stosw
	mov	dx,03B8H		;screen on, page 0
	pop	ax
	add	al,8
	out	dx,al
	ret
setmd	endp
;
cscreen	proc	near			;clear the screen
	mov	ax,dwseg		;point to display
	mov	es,ax
	mov	cx,sbsiz		;screen size in words
	sub	ax,ax			;clear data
	mov	di,ax
	cld				;scan upwards
	rep	stosw			;clear screen
	ret
cscreen	endp
;
spccm	proc	near			;set ptrs for current color/mode
	ret
spccm	endp
;
ssave	proc	near
	push	ds
	mov	cx,sbsiz		;save word count
	mov	si,0			;start of video
	mov	di,0
	mov	es,svseg		;ES=save page
	mov	ds,dwseg		;DS=video
	cld				;upscan
	rep	movsw
	pop	ds
	ret
ssave	endp
;
srest	proc	near
	push	ds
	mov	cx,sbsiz		;save buffer # words
	mov	si,0			;start of video
	mov	di,0
	mov	es,dwseg		;ES=display screen
	mov	ds,svseg		;DS=save page
	cld				;upscan
	rep	movsw
	pop	ds
	ret
srest	endp
;
;	----------------- select prompt text size ----------------------
;
SELSPT:	entry
	mov	tccw,1
	mov	nrast,12
	mov	dstcol,90
	mov	dstlin,29
	mov	dstxh,12
	mov	dstxw,8
	mov	cgenr,402h		;cgen resource #
	mov	ax,offset ltbls
	mov	ltblptr,ax
	exit
;
SELLPT:	entry
	mov	tccw,2
	mov	nrast,16
	mov	dstcol,45
	mov	dstlin,21
	mov	dstxh,16
	mov	dstxw,16
	mov	cgenr,401h		;cgen resource #
	mov	ax,offset ltbll
	mov	ltblptr,ax
	exit
;
;	----------------- local data items -------------------------
;
	align	4
dvrsvc	dd	0			;pointer to driver services
CGHdl	dd	0			;CGEN rsc handle
dwseg	dw	0B000H			;screen drawing segment
svseg	dw	0B800H			;save screen segment
sbsiz	dw	4000H			;#words per screen page
;
curclr	dw	1			;current color #
curmode	dw	1			;current drawing mode
curpat	dw	0FFFFH			;current line pattern
xscan	dw	0			;x scan position
bkgnd	dw	0			;current background color
;
;	bit selection and/or masks
;
bmask	dw	07F80H,0BF40H,0DF20H,0EF10H
	dw	0F708H,0FB04H,0FD02H,0FE01H
;
;	mode specification tables for setgrm, settxm
;
gtable	db	35H,2DH,2EH,07H,5BH,02H,57H,57H,02H,03H,00H,00H
ttable	db	61H,50H,52H,0FH,19H,06H,19H,19H,02H,0DH,0BH,0CH
;
;	---------- set dot macros support data ---------------------
;
;	YTBL4.CPY - 4 line interleave, 90 byte/line Y offset table
;
;	entry(Y)=2000H*(Y mod 4)+90*Y/4
;
ytbl DW 00000H,02000H,04000H,06000H,0005AH,0205AH,0405AH,0605AH
 DW 000B4H,020B4H,040B4H,060B4H,0010EH,0210EH,0410EH,0610EH
 DW 00168H,02168H,04168H,06168H,001C2H,021C2H,041C2H,061C2H
 DW 0021CH,0221CH,0421CH,0621CH,00276H,02276H,04276H,06276H
 DW 002D0H,022D0H,042D0H,062D0H,0032AH,0232AH,0432AH,0632AH
 DW 00384H,02384H,04384H,06384H,003DEH,023DEH,043DEH,063DEH
 DW 00438H,02438H,04438H,06438H,00492H,02492H,04492H,06492H
 DW 004ECH,024ECH,044ECH,064ECH,00546H,02546H,04546H,06546H
 DW 005A0H,025A0H,045A0H,065A0H,005FAH,025FAH,045FAH,065FAH
 DW 00654H,02654H,04654H,06654H,006AEH,026AEH,046AEH,066AEH
 DW 00708H,02708H,04708H,06708H,00762H,02762H,04762H,06762H
 DW 007BCH,027BCH,047BCH,067BCH,00816H,02816H,04816H,06816H
 DW 00870H,02870H,04870H,06870H,008CAH,028CAH,048CAH,068CAH
 DW 00924H,02924H,04924H,06924H,0097EH,0297EH,0497EH,0697EH
 DW 009D8H,029D8H,049D8H,069D8H,00A32H,02A32H,04A32H,06A32H
 DW 00A8CH,02A8CH,04A8CH,06A8CH,00AE6H,02AE6H,04AE6H,06AE6H
 DW 00B40H,02B40H,04B40H,06B40H,00B9AH,02B9AH,04B9AH,06B9AH
 DW 00BF4H,02BF4H,04BF4H,06BF4H,00C4EH,02C4EH,04C4EH,06C4EH
 DW 00CA8H,02CA8H,04CA8H,06CA8H,00D02H,02D02H,04D02H,06D02H
 DW 00D5CH,02D5CH,04D5CH,06D5CH,00DB6H,02DB6H,04DB6H,06DB6H
 DW 00E10H,02E10H,04E10H,06E10H,00E6AH,02E6AH,04E6AH,06E6AH
 DW 00EC4H,02EC4H,04EC4H,06EC4H,00F1EH,02F1EH,04F1EH,06F1EH
 DW 00F78H,02F78H,04F78H,06F78H,00FD2H,02FD2H,04FD2H,06FD2H
 DW 0102CH,0302CH,0502CH,0702CH,01086H,03086H,05086H,07086H
 DW 010E0H,030E0H,050E0H,070E0H,0113AH,0313AH,0513AH,0713AH
 DW 01194H,03194H,05194H,07194H,011EEH,031EEH,051EEH,071EEH
 DW 01248H,03248H,05248H,07248H,012A2H,032A2H,052A2H,072A2H
 DW 012FCH,032FCH,052FCH,072FCH,01356H,03356H,05356H,07356H
 DW 013B0H,033B0H,053B0H,073B0H,0140AH,0340AH,0540AH,0740AH
 DW 01464H,03464H,05464H,07464H,014BEH,034BEH,054BEH,074BEH
 DW 01518H,03518H,05518H,07518H,01572H,03572H,05572H,07572H
 DW 015CCH,035CCH,055CCH,075CCH,01626H,03626H,05626H,07626H
 DW 01680H,03680H,05680H,07680H,016DAH,036DAH,056DAH,076DAH
 DW 01734H,03734H,05734H,07734H,0178EH,0378EH,0578EH,0778EH
 DW 017E8H,037E8H,057E8H,077E8H,01842H,03842H,05842H,07842H
 DW 0189CH,0389CH,0589CH,0789CH,018F6H,038F6H,058F6H,078F6H
 DW 01950H,03950H,05950H,07950H,019AAH,039AAH,059AAH,079AAH
 DW 01A04H,03A04H,05A04H,07A04H,01A5EH,03A5EH,05A5EH,07A5EH
 DW 01AB8H,03AB8H,05AB8H,07AB8H,01B12H,03B12H,05B12H,07B12H
 DW 01B6CH,03B6CH,05B6CH,07B6CH,01BC6H,03BC6H,05BC6H,07BC6H
 DW 01C20H,03C20H,05C20H,07C20H,01C7AH,03C7AH,05C7AH,07C7AH
 DW 01CD4H,03CD4H,05CD4H,07CD4H,01D2EH,03D2EH,05D2EH,07D2EH
 DW 01D88H,03D88H,05D88H,07D88H,01DE2H,03DE2H,05DE2H,07DE2H
 DW 01E3CH,03E3CH,05E3CH,07E3CH,01E96H,03E96H,05E96H,07E96H
;
;	LTBL(line*2)=offset of text lines 0(top) thru 28(bottom)
;
ltbll	dw	0000H,0168H,02D0H,0438H,05A0H,0708H,0870H,09D8H
	dw	0B40H,0CA8H,0E10H,0F78H,10E0H,1248H,13B0H,1518H
	dw	1680H,17E8H,1950H,1AB8H,1D2EH
;
ltbls	dw	0000H,010EH,021CH,032AH,0438H,0546H,0654H,0762H
	dw	0870H,097EH,0A8CH,0B9AH,0CA8H,0DB6H,0EC4H,0FD2H
	dw	10E0H,11EEH,12FCH,140AH,1518H,1626H,1734H,1842H
	dw	1950H,1A5EH,1B6CH,1C7AH,1D88H,1E96H
;
;	---------------- global data items -------------------------
;
dsidrn	dw	1			;device id resource #
dsxmax	dw	719			;# X dots -1
dsymax	dw	347			;# Y dots -1
dsyar	dw	144			;aspect ratio Y/X
dsxar	dw	215
dstcol	dw	90			;# text columns
dstlin	dw	29			;# text lines
dsnclrs	dw	1			;max color # (0-ncolor)
dstxh	dw	12			;char height in pixels
dstxw	dw	8			;char width in pixels
dsflags	dw	3			;DSCIR/DSARC available
dsf2	dw	0			;more flags
slow	db	0			;slow function bits
iconsc	db	2			;16x16 icons
	db	16 dup (0)		;reserved
;
DSRSC	dw	1			;total # of resources
	dw	offset dsid		;1=device id text
;
dsid	db	'720x348 Hercules graphics board',0
;
	include DSLAST.CPY
