;	DSPGA.ASM - IBM PGA 640x480x16 color display driver
;
;	(c) Copyright 1986 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  4/22/1986
;	mod DSSAVE/DSREST to save mode,color,pattern 5/9/86
;	mode to allow colors in overlay (SAVE/REST) mode 5/9/86
;	modified 8/16/86 for driver toolkit compatibility
;	level 7 revisions 8/26/88 SB
;	new .BIN driver format 10/18/88 SB
;
	name	DSPGA
;
devrev	equ	0			;driver code rev #
cfgctl	equ	0			;config select bits
;
	include	DSDRV.CPY
;
; ----------------- PGA Macros ------------------------------
;
Send	macro	count,table
	mov	cx,count
	mov	bx,offset table
	call	PGASend
	endm
;
SendBV	macro	cmd,value
	mov	bx,offset cmdbfr
	mov	byte ptr [bx],cmd
	mov	cl,byte ptr value
	mov	1[bx],cl
	mov	cx,2
	call	PGASend
	endm
;
SendWV	macro	cmd,value
	mov	bx,offset cmdbfr
	mov	byte ptr [bx],cmd
	mov	cx,value
	mov	1[bx],cx
	mov	cx,3
	call	PGASend
	endm
;
SendXY	macro	cmd,X,Y
	mov	bx,offset cmdbfr
	mov	byte ptr [bx],cmd
	mov	cx,X
	mov	1[bx],cx
	mov	cx,Y
	mov	5[bx],cx
	mov	cx,9
	call	PGASend
	endm
;
; ------------ Display Driver Routines ----------------------
;
;	DSINIT - initialize the display driver
;
;	This will only be called once per program execution
;	before any other display driver calls.  It must:
;	1 - test for correct display hardware available
;	2 - allocate any required buffer memory from DOS
;	3 - perform global & local variable initialization
;	4 - initialize the display hardware
;	5 - select graphics display mode
;	6 - clear the screen
;	7 - set AX=error code:
;	    0=ok
;	    1=not enough memory
;	    2=not enough EMM memory
;	    3=device unavailable/unuseable
;	    4=CGEN resource missing
;
DSINIT:	entry
	call	PGAInit			;initialize hardware
	call	delay
	send	icxsiz,initcx		;select hex com mode
	call	delay
	send	initsz,initdt		;send initialization string
	mov	cx,256			;initialize color LUT
	mov	bx,offset lutdt
lblup:	push	cx			;save loop counter
	push	bx	
	mov	cx,lutsz		;send entry string
	call	PGASend
	pop	bx			;advance to next entry
	add	bx,lutsz
	pop	cx			;loop for all colors
	loop	lblup
	sub	ax,ax			;ax=0: device available for use
	exit
;
;	DSEND - end use of the display driver
;
;	This will only be called once per program execution
;	before program termination.  It must:
;	1 - restore any interrupt vectors changed by this driver
;	2 - restore the display to DOS-compatible text mode
;	3 - clear the text screen
;	4 - release any allocated buffer memory
;
DSEND:	entry
	send	dtxsz,dtxdt		;display text screen
	exit
;
;	DSGRM - set screen to graphics display mode
;
DSGRM:	entry
	send	dgrsz,dgrdt
	exit
;
;	DSTXM - set screen to DOS-compatible text mode
;
DSTXM:	entry
	send	dtxsz,dtxdt	;display text screen
	exit
;
;	DSCLEAR - clear the graphics screen
;
DSCLEAR: entry
	send	clrsz,clrdt	;clear screen to black
	exit
;
;	DSERA - select draw black mode (erase)
;
DSERA:	entry
	mov	CurMode,0	;select erase mode
	call	SetCCM		;set current mode and color
	exit
;
;	DSDRAW - select draw specified color mode
;
DSDRAW:	entry
	mov	CurMode,1	;select draw mode
	call	SetCCM		;set current mode and color
	exit
;
;	DSINV - select draw inverted pixel mode
;
DSINV:	entry
	mov	CurMode,2	;select xor mode
	call	SetCCM		;set current mode and color
	exit
;
;	DSCOLOR - specify current drawing color
;
;	AX = foreground color
;	CX = background color for patterns (if transparent unavailable)
;
DSCOLOR: entry
	mov	BkgClr,cx	;save for erase
	cmp	ax,0		;allow actual color #0
	jnz	ckclr
	mov	ax,0E0H		;set black for ovly & regular
	jmp	short stclr
ckclr:	and	ax,dsnclrs	;limit to maximum color number
	cmp	al,0		;remap color 0 to maximum color
	jnz	clrok
	mov	ax,dsnclrs
clrok:	mov	bl,al		;dup color bits 0-3 into 4-7
	shl	bl,1
	shl	bl,1
	shl	bl,1
	shl	bl,1
	or	al,bl
stclr:	mov	CurClr,ax	;set new current color
	call	SetCCM		;set current mode and color
	exit
;
;	DSPAT - set current line pattern from AX
;
;	AX = bit pattern, bit 15 is leftmost, 0 rightmost
;
DSPAT:	entry
	cmp	ax,-1		;solid?
	jz	revok
	mov	cx,16		;reverse bit order
revp:	rol	ax,1
	rcr	dx,1
	loop	revp
	mov	ax,dx
revok:	call	slpat		;make it current line style
	exit
;
slpat	proc	near
	cmp	ax,CurPat	;unchanged?
	jz	patxt
	mov	CurPat,ax	;was a change
	SendWV	0EAH,CurPat	;use new line pattern
patxt:	ret
slpat	endp
;
;	DSPOINT - draw a single point
;
;	SI = X coordinate
;	DI = Y coordinate
;
DSPOINT: entry
	mov	x1,si			;save new location
	mov	y1,di
	sendXY	10H,x1,y1		;move to position
	send	pcsz,pcdt		;draw point cmd
	exit
;
;	DSLINE - draw a line of current color and pattern
;
;	SI = X1, DI = Y1
;	DX = X2, AX = Y2
;
DSLINE:	entry
	mov	x1,si			;save location
	mov	y1,di
	mov	x2,dx			;save location
	mov	y2,ax
	sendXY	10H,x1,y1		;move cursor
	sendXY	28H,x2,y2		;draw line
	exit
;
;	DSREC - draw a rectangle of current color and pattern
;
;	SI = X1, DI = Y1
;	DX = X2, AX = Y2
;
DSREC:	entry
	mov	x1,si			;save location
	mov	y1,di
	mov	x2,dx			;save location
	mov	y2,ax
	sendXY	10H,x1,y1		;move cursor
	SendBV	0E9H,0			;fill off
	sendXY	34H,x2,y2		;draw rectangle
	SendBV	0E9H,1			;fill on
	exit
;
;	DSCIR - draw circle on screen (must entirely fit)
;
;	AX = center X
;	DX = center Y
;	CX = radius (horizontal)
;
DSCIR:	entry
	mov	x1,ax			;move to center
	mov	y1,dx
	mov	RV,cx			;(save radius)
	SendXY	10H,x1,y1
	SendBV	0E9H,0			;fill off
	mov	bx,offset cmdbfr	;circle command
	mov	byte ptr [bx],38H
	mov	cx,RV			;radius
	mov	1[bx],cx
	mov	cx,5
	call	PGASend			;send it
	SendBV	0E9H,1			;fill on
	exit
;
;	DSARC - draw arc on screen (circle must entirely fit)
;
;	AX = center X
;	DX = center Y
;	CX = radius (horizontal)
;	SI = starting angle (0-1023)
;	DI = ending angle (0-1023)
;
DSARC:	entry
	mov	x1,ax			;move to center
	mov	y1,dx
	mov	RV,cx			;(save radius)
	mov	ax,si			;range start angle
	mul	k360
	div	k1024
	mov	X2,ax
	mov	ax,di			;range end angle
	mul	k360
	div	k1024
	mov	Y2,ax
	SendXY	10H,x1,y1
	SendBV	0E9H,0			;fill off
	mov	bx,offset cmdbfr	;arc command
	mov	byte ptr [bx],3CH
	mov	cx,RV			;radius
	mov	1[bx],cx
	mov	ax,X2			;start angle
	mov	5[bx],ax
	mov	ax,Y2			;end angle
	mov	7[bx],ax
	mov	cx,9
	call	PGASend			;send it
	mov	word ptr cmdbfr+7,0	;clear end angle from X2 frac word
	SendBV	0E9H,1			;fill on
	exit
;
IDFHL	macro
	local	revxp
	mov	x1,si			;save location
	mov	y1,di
	mov	x2,dx			;save location
	mov	y2,ax
	and	di,07H			;select pattern for current line
	mov	dl,FilPat[di]
	mov	cx,si			;sync for horizontal start
	and	cx,07H
	rol	dl,cl
	mov	cx,8			;reverse & expand to 16-bits
revxp:	rol	dl,1
	rcr	al,1
	loop	revxp
	mov	ah,al
	call	slpat			;and select it
	sendXY	10H,x1,y1		;move cursor
	sendXY	28H,x2,y2		;draw line
	endm
;
;	DSFREC - draw a rectangle filled with the current color and mode
;
;	SI = X1, DI = Y1
;	DX = X2, AX = Y2
;
DSFREC:	entry
	cmp	CurFSN,0		;currently solid fill?
	jnz	nosfr
	mov	x1,si			;save location
	mov	y1,di
	mov	x2,dx			;save location
	mov	y2,ax
	sendXY	10H,x1,y1		;move cursor
	sendXY	34H,x2,y2		;draw rectangle
	exit
nosfr:	cmp	di,ax			;di lowest?
	jbe	fordok
	xchg	di,ax			;no - swap them
fordok:	push	si			;save values
	push	di
	push	dx
	push	ax
	mov	ax,di			;horizontal line drawn
	IDFHL				;draw the fill line
	pop	ax			;recover values
	pop	dx
	pop	di
	pop	si
	inc	di			;bump to next line
	cmp	di,ax			;done?
	jbe	fordok			;no - loop for all lines
	exit
;
	include	IDPOLY.CPY
;
;	DSTEXT - display prompt text
;	DSITXT - display inverted prompt text
;
;	ES:DI = text string address
;	        (string ended by a 0 byte)
;	DX    = desired text line number (0 top)
;	AX    = desired text column number (0 left)
;	single chr string only may contain backspace
;
DSTEXT:	entry
	mov	txclr,0FFH
	mov	txbkc,0E0H
	jmp	short dtcom
DSITXT:	entry
	mov	txclr,0E0H
	mov	txbkc,0FFH
dtcom:	cmp	byte ptr ES:[DI],8	;backspace?
	jnz	istext
	mov	cx,1			;allow 1chr stg = backspace
	dec	ax			;clear column to left
	jmp	backsp			;yes
istext:	mov	Y1,dx			;save line #
	mov	X1,ax			;save column #
	sub	cx,cx			;count string length
	mov	bx,di
txclup:	cmp	byte ptr ES:[bx],0	;at eos?
	jz	cxok
	inc	cx			;bump count
	inc	bx			;and scan
	jmp	short txclup
cxok:	call	tclr			;clear text background
	SendBV	0EBH,0			;draw mode
	SendBV	6,txclr			;in text color
	mov	ax,dstlin		;y2=(dstlin-1-line)*dstxh+txvo
	dec	ax
	sub	ax,y1
	mul	dstxh
	add	ax,txvo
	mov	Y2,ax
	mov	ax,X1			;x1=column*dstxw+1
	mul	dstxw
	inc	ax
	mov	X2,ax
	SendXY	10H,X2,Y2		;position to start of text
	mov	txcsiz,3		;init text length
	mov	si,offset txbfr		;where to store it
txcpy:	mov	al,ES:[DI]		;get character
	cmp	al,0			;EOS?
	jz	iseos
	cmp	al,txdel		;display delimiter chr?
	jnz	normc
	inc	di			;yes - bump source past it
	mov	bx,offset deltx		;copy delimiter text
	mov	cx,deltxl		;length
dclup:	mov	al,[bx]			;get chr
	mov	[si],al			;store it
	inc	si			;bump pointers
	inc	bx
	inc	txcsiz			;and count
	loop	dclup			;move all delimiter replacement
	jmp	short txcpy		;continue copying text
normc:	mov	[SI],al			;store character
	inc	si			;bump pointers
	inc	di
	inc	txcsiz			;count character
	jmp	short txcpy		;continue copying text
iseos:	mov	al,txdel		;append text delimiter
	mov	[SI],al
	send	txcsiz,txcmd		;send text command
	call	SetCCM			;restore current color and mode
	exit
;
;	DSTINV - invert text cells
;
;	DX = desired line number (0 top)
;	AX = desired column number (0 left)
;	CX = number of char. positions
;
DSTINV:	entry
	mov	Y1,dx			;save line #
	mov	X1,ax			;save column #
	mov	NChrs,cx		;save character count
	mov	ax,dstlin		;y2=(dstlin-1-line)*dstxh
	dec	ax
	sub	ax,y1
	mul	dstxh
	mov	Y2,ax
	mov	ax,X1			;x1=column*dstxw
	mul	dstxw
	mov	X2,ax
	SendXY	10H,X2,Y2		;position to start of text
	mov	ax,NChrs		;X2=X2+tcount*dstxw-1
	mul	dstxw
	dec	ax
	add	X2,ax
	mov	ax,Y2			;Y2=Y2+dstxh-1
	add	ax,dstxh
	dec	ax
	mov	Y2,ax
	SendBV	0EBH,1			;xor mode
	SendBV	0E8H,imask		;invert plane select
	SendXY	34H,X2,Y2		;fill the rectangle
	SendBV	0E8H,nmask		;normal plane mask
	call	SetCCM			;restore normal mode
	exit
;
;	DSTCLR - clear text cells
;
;	DX = desired line number (0 top)
;	AX = desired column number (0 left)
;	CX = number of char. positions
;
DSTCLR:	entry
backsp:	mov	Y1,dx			;save line #
	mov	X1,ax			;save column #
	mov	txbkc,0E0H		;clear to color
	call	tclr
	exit
;
tclr	proc	near
	mov	NChrs,cx		;save character count
	mov	ax,dstlin		;y2=(dstlin-1-line)*dstxh
	dec	ax
	sub	ax,y1
	mul	dstxh
	mov	Y2,ax
	mov	ax,X1			;x1=column*dstxw
	mul	dstxw
	mov	X2,ax
	SendXY	10H,X2,Y2		;position to start of text
	mov	ax,NChrs		;X2=X2+tcount*dstxw-1
	mul	dstxw
	dec	ax
	add	X2,ax
	mov	ax,Y2			;Y2=Y2+dstxh-1
	add	ax,dstxh
	dec	ax
	mov	Y2,ax
	SendBV	0EBH,0			;draw mode
	SendBV	6,txbkc			;in text background color
	SendXY	34H,X2,Y2		;fill the rectangle
	call	SetCCM			;restore normal mode
	ret
tclr	endp
;
;	DSSAVE - save the graphics screen
;		 (enter overlay write mode)
;
DSSAVE:	entry
	sendBV	0E8H,0F0H		;select overlay planes
	mov	imask,010H		;invert text mask
	mov	nmask,0F0H		;normal mask
	exit
;
;	DSREST - restore the graphics screen
;		 (end overlay write mode, and clear overlay)
;
DSREST:	entry
	sendBV	0E8H,0F0H		;select overlay planes
	SendBV	07H,0			;and clear them
	SendBV	0E8H,0FH		;select normal planes
	mov	imask,0FH		;invert text mask
	mov	nmask,0FH		;normal mask
	exit
;
;	----------------- local use procedures ----------------------
;
SetCCM	proc	near			;set Current color and Mode
	cmp	CurMode,1
	jz	isdrw
	jg	isxor
isera:	SendBV	0EBH,0			;draw mode
	SendBV	6,BkgClr		;in background color
	ret
isdrw:	SendBV	0EBH,0			;draw mode
	SendBV	6,CurClr		;current color
	ret
isxor:	SendBV	0EBH,1			;xor mode
	ret
setccm	endp
;
;	PGA primitive routines
;
PGAInit	proc	near			;init PGA board
	push	es
	mov	ax,PGASeg
	mov	es,ax
	mov	di,RstOffset		;reset "port"
	mov	al,01H
	mov	ES:[di],al		;trigger reset
rstwt:	mov	al,ES:[di]		;wait for reset complete
	cmp	al,0
	jnz	rstwt
	pop	es
	ret
PGAInit	endp
;
;	PGASend - send CX bytes from [BX] to PGA board
;
PGASend	proc	near
	push	es			;use PGA segment
	push	di
	push	si
	mov	ax,PGASeg
	mov	es,ax
	mov	si,WPOffset		;write pointer offset
wpblup:	mov	di,RPOffset		;read pointer offset
	mov	al,ES:[si]		;get write pointer
	mov	dl,[bx]			;get source data byte
	inc	bx			;bump source scanner
wntrdy:	mov	ah,ES:[di]		;get read pointer value
	sub	ah,al			;room in buffer?
	cmp	ah,1
	jz	wntrdy			;no - wait
	xor	ah,ah			;store data
	mov	di,ax
	mov	ES:[di],dl
	inc	al			;bump write pointer
	mov	ES:[si],al
	loop	wpblup			;loop for all bytes
	pop	si
	pop	di
	pop	es
	ret
PGASend	endp
;
delay	proc	near
	mov	cx,-1			;general delay for init
dlylp:	loop	dlylp
	ret
delay	endp
;
;	----------------- local data items -------------------------
;
CurClr	dw	1		;current color #
CurMode	dw	1		;current drawing mode
CurPat	dw	0FFFFH		;current line pattern
BkgClr	dw	0		;background color
;
X1	dw	0		;working coordinates
Y1	dw	0
X2	dw	0
Y2	dw	0
k360	dw	360		;constant
k1024	dw	1024		;constant
RV	dw	0
NChrs	dw	0		;character count save
imask	dw	0FH		;inversion bit plane mask
nmask	dw	0FH		;normal bit plane mask
;
PGASeg	dw	0C600H		;PGA segment address
WPOffset dw	300H		;offset to write pointer
RPOffset dw	301H		;offset to read pointer
RstOffset dw	307H		;offset to reset byte
;
CmdBfr	db	0		;SendBV,SendWV,SendXY buffer
	dw	0		;X parm value
	dw	0		;(always zero)
	dw	0		;Y parm value
	dw	0		;(always zero)
;
txclr	db	0FFH		;text color code
txbkc	db	0E0H		;text background color
txvo	dw	5		;text cell vertical offset
txcsiz	dw	3		;text command length
txcmd	db	80H		;text command
txdel	db	22H		;text delimiter
txbfr	db	256 dup (22H)	;text command data buffer
;
deltxl	dw	7		;delimiter text equivalent length
deltx	db	22H,80H,27H,22H,27H,80H,22H ;delimiter text equivalent
;
;
;	initialization commands
;
icxsiz	dw	3			;mode size
initcx	db	'C','X',' '		;communications hex
;
initsz	dw	31			;initialization string
initdt	db	4			;reset flags
	db	0D0H,0			;graphics display visible
	db	0B3H			;window 0 639 0 479
	db	0,0,0,0,07FH,02H,0,0
	db	0,0,0,0,0DFH,01H,0,0
	db	81H			;double height text
	db	10H,0,0,0
	db	0E9H,1			;primitive fill on
	db	0FH,0			;clear all bit planes
	db	0E8H,0FH		;select normal bit planes
;
	include	DSPGA.CPY		;LUT data table
;
;	general commands
;
dtxsz	dw	2			;display text screen
dtxdt	db	0D0H,1
;
dgrsz	dw	2			;display graphics screen
dgrdt	db	0D0H,0
;
clrsz	dw	2			;clear screen to black
clrdt	db	0FH,0
;
pcsz	dw	1			;draw point command
pcdt	db	8
;
;	---------------- global data items -------------------------
;
dsidrn	dw	1			;device id resource #
dsxmax	dw	639		;# X dots -1
dsymax	dw	479		;# Y dots -1
dsyar	dw	183		;aspect ratio Y/X
dsxar	dw	240
dstcol	dw	40		;# text columns
dstlin	dw	20		;# text lines
dsnclrs	dw	15		;max color # (0-ncolor)
dstxh	dw	24		;text height in pixels
dstxw	dw	16		;text width in pixels
dsflags	dw	3		;DSARC/DSCIR available
dsf2	dw	0		;more flags
slow	db	0		;slow function
iconsc	db	2		;16x16 icons
	db	16 dup (0)	;reserved
;
DSRSC	dw	1			;total # of resources
	dw	offset dsid		;1=device id text
;
dsid	db	'IBM 640x480 PGA graphics board.',0
;
	include DSLAST.CPY
