;	DSVGA.ASM - VGA mode 12 (640x480x16 color) Display Driver
;	using VGA write mode 3 logic
;
;	(c) Copyright 1986,7,8 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  7/1/88
;	New .BIN driver format 9/18/88 MR
;	added DSBLN 3/15/89
;
	name	DSVGA
;
devrev	equ	0			;driver code revision #
cfgctl	equ	3			;configuration select bits
;
	include	DSDRV.CPY
;
;	VGA graphics register set macro
;
SGREGD	macro	sel,value
	mov	dx,03CEh		;select register
	mov	al,sel
	mov	ah,value
	out	dx,ax
	endm
;
SGREG	macro	sel,value
	mov	dx,03CEh		;select register
	mov	ax,sel+value*256
	out	dx,ax
	endm
;
spccm	dw	sdrw			;current mode & color set routine
;
;	DSINIT - initialize the display driver
;
;	This will only be called once per program execution
;	before any other display driver calls.  It must:
;	1 - test for correct display hardware available
;	2 - allocate any required buffer memory from DOS
;	3 - perform global & local variable initialization
;	4 - initialize the display hardware
;	5 - select graphics display mode
;	6 - clear the screen
;	7 - set AX=error code:
;	    0=ok
;	    1=not enough memory
;	    2=not enough EMM memory
;	    3=device unavailable/unuseable
;	    4=CGEN resource missing
;
DSINIT:	entry
	mov	word ptr DVRSVC,di	;save pointer to driver services
	mov	word ptr DVRSVC+2,ES
	mov	ax,CGENR		;load char gen resource
	mov	bx,0			;into alloc segment
	mov	dx,0			;RSCLD function
	call	dword ptr DVRSVC	;do it
	jnc	short ok1
	mov	ax,4			;missing CGEN resource
	exit
ok1:	mov	ax,ES:[bx]		;get segment from handle
	mov	CGSeg,ax		;save segment
	mov	word ptr CGHdl,bx	;save handle
	mov	ax,ES
	mov	word ptr CGHdl+2,ax
	call	asbfr			;allocate screen save buffer
	jnc	short ok2
	mov	ax,1			;not enough memory
	exit
ok2:	call	setgrm			;select graphics screen mode
	call	cscreen			;clear the screen
	call	spccm			;set pointers for init color and mode
	sub	ax,ax			;ax=0: device available for use
	exit
;
;	DSEND - end use of the display driver
;
;	This will only be called once per program execution
;	before program termination.  It must:
;	1 - restore any interrupt vectors changed by this driver
;	2 - restore the display to DOS-compatible text mode
;	3 - clear the text screen
;	4 - release any allocated buffer memory
;
DSEND:	entry
	call	settxm			;select text screen mode
	call	rsbfr			;release the screen save buffer
	les	bx,CGHdl		;release CGEN resource
	mov	dx,1
	call	dword ptr DVRSVC	;do it
	exit
;
;	DSGRM - set screen to graphics display mode
;
DSGRM:	entry
	call	setgrm			;select graphics display
	call	srest			;restore saved screen
	exit
;
;	DSTXM - set screen to DOS-compatible text mode
;
DSTXM:	entry
	call	ssave			;save graphics display
	call	settxm			;select text display
	exit
;
;	DSCLEAR - clear the graphics screen
;
DSCLEAR: entry
	call	cscreen			;clear the screen
	exit
;
;	DSERA - select draw background mode (erase)
;
DSERA:	entry
	mov	spccm,offset sera
	call	sera
	exit
;
;	DSDRAW - select draw specified color mode
;
DSDRAW:	entry
	mov	spccm,offset sdrw
	call	sdrw
	exit
;
;	DSINV - select draw inverted pixel mode
;
DSINV:	entry
	mov	spccm,offset sxor
	call	sxor
	exit
;
;	DSCOLOR - specify current drawing color & background
;
DSCOLOR: entry
	mov	byte ptr bkgnd,cl	;save background color
	mov	byte ptr CurClr,al	;set new current color
	call	spccm			;set pointers
	exit
;
;	---------------- Set Dot Macros used by SDSUBS --------------
;
SDGPM	macro				;get pixel modification masks
	mov	bx,si			;x mod 8
	and	bx,7
	mov	AH,pmasks[bx]		;mask for selected pixel
	endm
;
SDCBO	macro				;calculate byte offset to change
	mov	bx,dsymax		;invert y value (0 at bottom)
	sub	bx,di
	shl	bx,1			;y*80=y*16+y*64 (16 clocks total)
	shl	bx,1
	shl	bx,1
	shl	bx,1
	mov	bp,bx
	shl	bx,1
	shl	bx,1
	add	bx,bp
	mov	bp,si			;work copy of X
	shr	bp,1			;+ x/8
	shr	bp,1
	shr	bp,1
	add	bx,bp			;ES:[bx] = byte to change
	endm
;
SDCSP	macro				;change selected pixel
	mov	al,ES:[bx]		;read data into latches
	mov	ES:[bx],ah		;set pixel mask
	endm
;
SDUPL	macro				;move offset up one line
	sub	bx,80			;bump line offset
	endm
;
SDDNL	macro				;move offset down one line
	add	bx,80			;bump line offset
	endm
;
SDUPC	macro
	ror	ah,1			;shift pixel mask right
	adc	bx,0			;if wrap, bump byte offset
	endm
;
SDDNC	macro
	rol	ah,1			;shift pixel mask left
	sbb	bx,0			;if wrap, decrement byte offset
	endm
;
;	CCHRO - calculate character offset to BX; DX=line, AX=column
;
CCHRO	macro				;calculate char offset
	call	cchros
	endm
;
cchros	proc	near
	mov	bx,dx			;line number
	shl	bx,1			;word index
	add	bx,ltblptr		;lookup line offset
	mov	bx,[bx]
	add	bx,ax			;+cposn
	cmp	tccw,2			;*2?
	jnz	notx2
	add	bx,ax			;yes
notx2:	SGREG	3,0			;function = latch copy data
	SGREG	5,0			;write mode 0
	ret
cchros	endp
;
;	GCHR - generate character from DS:[SI] cgen table, ES:BX=chr cell
;	BP= inversion mask 0 or -1; CX = NRAST value for loop; DX=tccw
;
GCHR	macro				;generate character
	local	gc1,gc2,gcxt
	cmp	dx,2			;word or byte?
	jz	short gc2
gc1:	lodsb				;get raster line
	xor	ax,bp			;invert if needed
	mov	ES:[bx],al		;set it into display
	sddnl				;move down to next raster
	loop	short gc1		;loop for entire character
	jmp	short gcxt
gc2:	lodsw				;get raster line
	xor	ax,bp			;invert if needed
	xchg	al,ah			;byte swap
	mov	ES:[bx],ax		;set it into display
	sddnl				;move down to next raster
	loop	short gc2		;loop for entire character
gcxt:
	endm
;
;	INVCR - invert character raster line ES:[BX], CX characters
;
INVCR	macro
	call	invcrs
	endm
;
invcrs	proc	near
invlup:	cmp	tccw,2			;word or byte?
	jz	inv2
	not	byte ptr ES:[bx]	;invert character raster
	jmp	short invc
inv2:	not	word ptr ES:[bx]	;invert character raster
invc:	add	bx,tccw			;next character
	loop	invlup			;for all char posns
	ret
invcrs	endp
;
;	CLRCR - clear character raster line ES:[BX],CX characters
;
CLRCR	macro
	call	clrcrs
	endm
;
clrcrs	proc	near
	sub	ax,ax			;clear to value
clrlup:	cmp	tccw,2			;word or byte?
	jz	clr2
	mov	ES:[bx],al		;clear chr raster
	jmp	short clrc
clr2:	mov	ES:[bx],ax		;clear chr raster
clrc:	add	bx,tccw			;bump chr cell
	loop	clrlup			;for all char positions
	ret
clrcrs	endp
;
;	-------------------------------------------------------------------
;
tccw	dw	1			;text character cell width in bytes
nrast	dw	12			;raster lines per character
;
CGENR	dw	402h			;character generator resource #
LTBLPTR	dw	offset ltbls		;line offset table
;
sdfhl	macro
	call	VGAFHL
	endm
;
;	SDFHL entry: SCANYL = Y value of horiz line
;	BP=lowX, CX=#pixels, ES=DwSeg, DX=pixel masks (if set by SDGPM)
;
VGAFHL	proc	near
	mov	si,scanyl		;select current lines' pattern
	and	si,07H
	mov	ah,FilPat[si]
	mov	si,bp			;x column position
	and	si,7			;mod 8 = left byte pixel mask index
	sub	cx,8			;remove left byte pixels from count
	add	cx,si
	jge	ntsngl			;more than one byte changes?
	mov	al,lbpmt[si]		;set left byte pixel mask
	and	al,ah			;(pattern mask)
	lea	si,rbpmt+8		;less unchanged right pixels
	add	si,cx	
	and	al,[si]
	mov	dl,ES:[bx]		;read latch data
	mov	ES:[bx],al		;set selected pixels
	ret
ntsngl:	mov	di,cx			;right byte pixel mask index
	and	di,7
	shr	cx,1			;remove right byte pixels from count
	shr	cx,1
	shr	cx,1
	mov	al,lbpmt[si]		;set left byte pixel mask
	and	al,ah			;(pattern mask)
	mov	dl,ES:[bx]		;read latch data
	mov	ES:[bx],al		;set selected pixels
	inc	bx			;bump byte to change offset
	jcxz	chgrb
	mov	al,ah			;set middle bytes pixel mask
chgmb:	mov	dl,ES:[bx]		;read latch data
	mov	ES:[bx],al		;set selected pixels
	inc	bx			;bump byte to change offset
	loop	chgmb			;loop for all middle bytes
chgrb:	mov	al,rbpmt[di]		;set right byte pixel mask
	and	al,ah			;(pattern mask)
	mov	dl,ES:[bx]		;read latch data
	mov	ES:[bx],al		;set selected pixels
	ret
vgafhl	endp
;
lbpmt	db	0FFH,07FH,03FH,01FH,00FH,007H,003H,001H
rbpmt	db	000H,080H,0C0H,0E0H,0F0H,0F8H,0FCH,0FEH
;
LSScl	equ	2			;line style scale
	include SDSUBS4.CPY
	include	VTEXT.CPY
	include	DSBLN.CPY
	include	VGAPAL.CPY
;
ANIMF	db	0			;animation flag
;
;	DSABGN - Setup screen mode for animation
;
DSABGN:	entry
	mov	ax,90H			;select 640x350 color mode
	int	10H
	call	SETPAL			;setup our palette
	SGREG	8,0FFH			;enable all pixels to be changed
	SGREG	1,0H			;disable S/R (direct write in wm0)
	mov	dsxmax,639		;shrink screen area
	mov	dsymax,349
	mov	ax,dsyar
	mov	yarsav,ax
	mov	ax,dsxar
	mov	xarsav,ax
	mov	dsyar,77
	mov	dsxar,103
	mov	sbsiz,4000h
	exit
;
;	DSAUPD - Animation Update
;
;	Copy drawing buffer to 2nd screen buffer
;	Force display of 2nd screen buffer
;
DSAUPD:	entry
;
;	copy low 32k to hi 32k screen memory on card
;
	mov	cx,4000h		;buffer size in words
	shl	cx,1			;in bytes
	push	ds
	mov	ax,dwseg		;display memory segment
	mov	ds,ax
	mov	es,ax
	mov	si,0000H		;from zero (active page)
	mov	di,8000H		;to inactive page
	cld				;upscan
	SGREG	8,0			;copy all write from prior read
	SGREG	3,0H			;write normal data
	rep	movsb			;copy all data (32-bit copies)
	pop	ds
	SGREG	8,0FFH			;enable all pixels to be changed
	call	spccm			;set current color and mode
;
;	switch to display high buffer area
;
	test	animf,1			;doing animation?
	jz	short stanim
	exit				;yes
stanim:	mov	animf,1			;no - flag animation started
	mov	dx,3DAh			;CRTC status port
wvrs:	in	al,dx			;wait for start of vert retrace
	test	al,8
	jz	short wvrs
wvre:	in	al,dx			;wait for end of vert retrace
	test	al,8
	jnz	short wvre
	cli				;disable interrupts
	sub	dl,6			;CRTC I/O port
	mov	ax,800Ch		;start adrs high
	out	dx,ax
	mov	ax,000Dh		;start adrs low
	out	dx,ax
	sti				;enable interrupts
	exit
;
;	DSAEND - Animation End
;
;	Force display of normal draw screen buffer
;
DSAEND:	entry
	test	animf,1			;doing animation?
	jnz	short eanim
	exit				;no
eanim:	mov	ax,092H			;select 640x480 color mode
	int	10H
	call	SETPAL			;setup our palette
	SGREG	8,0FFH			;enable all pixels to be changed
	SGREG	1,0H			;disable S/R (direct write in wm0)
	mov	dsxmax,639		;restore screen area
	mov	dsymax,479
	mov	ax,yarsav
	mov	dsyar,ax
	mov	ax,xarsav
	mov	dsxar,ax
	mov	sbsiz,4B00h
	mov	animf,0			;yes - flag animation ended
	mov	dx,3DAh			;CRTC status port
ewvrs:	in	al,dx			;wait for start of vert retrace
	test	al,8
	jz	short ewvrs
ewvre:	in	al,dx			;wait for end of vert retrace
	test	al,8
	jnz	short ewvre
	cli				;disable interrupts
	sub	dl,6			;CRTC I/O port
	mov	ax,000Ch		;start adrs high
	out	dx,ax
	mov	ax,000Dh		;start adrs low
	out	dx,ax
	sti				;enable interrupts
	exit
;
;	DSSAVE - save the graphics screen
;
DSSAVE:	entry
	call	ssave			;save current screen
	exit
;
DSREST:	entry
	call	srest			;restore prior screen
	exit
;
DskTyp	equ	1			;disk save type id
;
;	EGADSK.CPY - EGA/VGA Disk File Screen SAVE/RESTORE Routines
;
;	Parameter is DSKTYP equ save data format type id
;	BX = open file handle for read/write (do not close)
;
;	0 = screen save unimplimented
;	1 = 640 x 480 VGA or EGA+
;	2 = 640 x 350 EGA
;
	if	DskTyp EQ 1
DskSiz	equ	9600h
	endif
	if	DskTyp EQ 2
DskSiz	equ	6D60h
	endif
;
DPSAVE	macro	arg
	mov	al,arg			;enable selected plane read
	out	dx,al
	push	dx			;save port
	mov	ah,40h			;write to BX handle
	sub	dx,dx			;DS:0000h:dsksiz-1
	mov	cx,DskSiz		;bytes per plane
	int	21H
	pop	dx			;restore port
	endm
;
dsktw	dw	dsktyp
dsktr	dw	dsktyp
;
;	DSDSKS - Save screen image to file handle BX
;
DSDSKS:	entry
	SGREG	5,0			;write mode 0
	mov	ah,40h			;write to BX handle
	lea	dx,dsktw		;DS:dsktw
	mov	cx,2			;2 bytes
	int	21H
	push	ds
	mov	ax,dwseg		;display memory segment
	mov	ds,ax
	cld				;upscan
	mov	dx,03CEH		;select read map register adrs
	mov	al,4
	out	dx,al
	inc	dx			;select read map data port
	dpsave	0
	dpsave	1
	dpsave	2
	dpsave	3
	pop	ds			;restore working segments
	call	spccm			;set current color and mode
	clc				;worked ok
	exit
;
DPREST	macro	arg
	mov	al,arg			;enable selected write plane
	out	dx,al
	push	dx			;save port
	mov	ah,3Fh			;read from BX handle
	mov	dx,bp			;DS:0000h:dsksiz-1
	mov	cx,di			;bytes per plane to read
	int	21H
	mov	ax,4201h		;seek from present posn
	sub	cx,cx			;CX:DX bytes
	mov	dx,si
	int	21h
	pop	dx			;restore port
	endm
;
;	DSDSKR - Restore screen image from file handle BX
;
DSDSKR:	entry
	mov	dsktr,0			;clear read type
	mov	ah,3Fh			;read from BX handle
	lea	dx,dsktr		;DS:dsktr
	mov	cx,2			;2 bytes
	int	21H
	mov	ax,dsktr		;is it compatible?
	cmp	ax,0
	jz	dfnotc
	cmp	ax,2
	ja	dfnotc
	mov	di,DskSiz		;set read plane count
	mov	si,0			;set skip after read count
	mov	bp,0			;set read start offset
	cmp	ax,DskTW		;this drivers' own format?
	jz	dfisc
	cmp	ax,2			;no - read ega?
	jz	rdega
	mov	si,28A0h		;read VGA file on EGA
	jmp	short dfisc
rdega:	mov	di,6D60h		;read EGA file on VGA
	mov	bp,1450h		;center on screen
	jmp	short dfisc
dfnotc:	stc				;no - not restored
	mov	ax,0			;no read type
	exit
dfisc:	SGREG	5,0			;write mode 0
	SGREG	3,0H			;write normal data
	push	ds
	mov	ax,dwseg		;set video segment
	mov	ds,ax
	cld				;upscan
	mov	dx,03CEH		;select pixel mask register adrs
	mov	al,8
	out	dx,al
	inc	dx			;enable all pixels of a byte
	mov	al,0FFH
	out	dx,al
	mov	dx,03C4H		;select plane write mask reg adrs
	mov	al,2
	out	dx,al
	inc	dx			;select plane write mask data port
	dprest	1
	dprest	2
	dprest	4
	dprest	8
	mov	dx,03C4H		;select plane write mask reg adrs
	mov	al,2
	out	dx,al
	inc	dx			;select plane write mask data port
	mov	al,0FFH			;enable all planes
	out	dx,al
	pop	ds			;restore working segments
	call	spccm			;set current color and mode
	mov	ax,DskTR		;return loaded type code
	clc				;worked ok
	exit
;
;	The VGA board uses 9600H bytes per screen. To save the entire
;	screen, it is necessary to save some of the screen in high VGA
;	RAM (27136 byte/planes) and some in system memory (11264 bytes
;	* 4 bit-planes (45056 bytes system ram total).
;
;	ASBFR - allocate save buffer
;
ASBFR	proc	near
	mov	EMMSeg,0		;assume no EMM available
	mov	ax,4			;ask for 4 pages
	mov	dx,8			;DVRSVC EMMA
	call	dword ptr DVRSVC
	jc	short noemma		;available?
	mov	EmmHdl,bx		;yes!
	mov	EmmSeg,ax
	clc				;only mem we need
	ret
noemma:	mov	ax,2816			;# paras needed (11264*4/16)
	mov	bx,2			;no release until end!
	mov	dx,2			;allocate RAM block
	call	dword ptr DVRSVC
	jc	asbxt			;error?
	mov	ax,ES:[bx]		;ok - get segment selector
	mov	SavSeg,AX		;save it
	mov	word ptr SHdl,bx	;save RAM block handle
	mov	ax,ES
	mov	word ptr SHdl+2,ax
	clc				;no error
asbxt:	ret				;carry set if error
asbfr	endp
;
;	RSBFR - release save buffer
;
RSBFR	proc	near
	cmp	EMMSeg,0		;buffer in EMM?
	jz	short noemmr
	mov	bx,EMMHdl		;yes - release it
	mov	dx,9			;DVRSVC EMMR
	call	dword ptr DVRSVC
	ret
noemmr:	les	bx,SHdl			;RAM block handle
	mov	dx,3			;release RAM block
	call	dword ptr DVRSVC
	ret				;carry set if error
rsbfr	endp
;
;	Save Entire Screen
;
ssave	proc	near
	SGREG	5,0			;write mode 0
	call	xsave			;save highest 1600H pixels in sysram
	mov	cx,27136		;buffer size in bytes
	push	ds
	mov	ax,dwseg		;display memory segment
	mov	ds,ax
	mov	es,ax
	mov	si,0000H		;from zero (active page)
	mov	di,9600H		;to inactive page
	cld				;upscan
	SGREG	8,0			;copy all write from prior read
	SGREG	3,0H			;write normal data
	rep	movsb			;copy all data (32-bit copies)
	pop	ds
	SGREG	8,0FFH			;enable all pixels to be changed
	call	spccm			;set current color and mode
	ret
ssave	endp
;
srest	proc	near
	SGREG	5,0			;write mode 0
	mov	cx,27136		;buffer size in bytes
	push	ds
	mov	ax,dwseg		;display memory segment
	mov	ds,ax
	mov	es,ax
	mov	si,9600H		;from inactive page
	mov	di,0000H		;to active page
	cld				;upscan
	SGREG	8,0			;copy all write from prior read
	SGREG	3,0H			;write normal data
	rep	movsb			;copy all data (32-bit copies)
	pop	ds
	call	xrest			;restore highest 1600H pxls from sysram
	SGREG	8,0FFH			;enable all pixels to be changed
	call	spccm			;set current color and mode
	ret
srest	endp
;
;	EMM or System RAM save/restore routines
;
PSAVE	macro	arg,epage
	mov	al,arg			;enable selected plane read
	out	dx,al
	mov	ax,epage
	call	pgsave
	endm
;
PGSAVE	proc	near
	push	dx
	push	ES
	push	DS
	cmp	EMMSeg,0		;save to EMM?
	jz	short nesav
	mov	bx,EMMHdl		;yes - access page # AX
	mov	dx,10			;DVRSVC EMMU
	call	dword ptr DVRSVC
	mov	ES,EMMSeg
	jmp	short savc
nesav:	mov	ES,SavSeg		;setup copy segments
savc:	mov	DS,DwSeg
	mov	si,6A00H		;DS:6A00H-95FFH is display plane memory
	sub	di,di			;ES:0-2BFFH is save plane buffer
	mov	cx,1600H		;page size in words
	rep	movsw			;copy page
	pop	DS
	pop	ES
	pop	dx
	add	SavSeg,2C0H		;advance to next plane
	ret
pgsave	endp
;
xsave	proc	near
	cld				;upscan
	mov	dx,03CEH		;select read map register adrs
	mov	al,4
	out	dx,al
	inc	dx			;select read map data port
	push	SavSeg			;save start segment
	psave	0,0
	psave	1,1
	psave	2,2
	psave	3,3
	pop	SavSeg			;reset to start
	ret
xsave	endp
;
PREST	macro	arg,epage
	mov	al,arg			;enable selected write plane
	out	dx,al
	mov	ax,epage
	call	pgrest
	endm
;
PGREST	proc	near
	push	dx
	push	ES
	push	DS
	cmp	EMMSeg,0		;restore from EMM?
	jz	short neres
	mov	bx,EMMHdl		;yes - access page # AX
	mov	dx,10			;DVRSVC EMMU
	call	dword ptr DVRSVC
	mov	ES,DwSeg
	mov	DS,EMMSeg
	jmp	short resc
neres:	mov	ES,DwSeg		;setup copy segments
	mov	DS,SavSeg
resc:	mov	di,6A00H		;ES:6A00H-95FFH is display plane
	sub	si,si			;DS:0-2BFFH is save plane buffer
	mov	cx,1600H		;page size in words
	rep	movsw			;copy page
	pop	DS
	pop	ES
	pop	dx
	add	SavSeg,2C0H		;advance to next plane
	ret
pgrest	endp
;
xrest	proc	near
	cld				;upscan
	mov	dx,03CEH		;select pixel mask register adrs
	mov	al,8
	out	dx,al
	inc	dx			;enable all pixels of a byte
	mov	al,0FFH
	out	dx,al
	mov	dx,03C4H		;select plane write mask reg adrs
	mov	al,2
	out	dx,al
	inc	dx			;select plane write mask data port
	push	SavSeg			;save start segment
	prest	1,0
	prest	2,1
	prest	4,2
	prest	8,3
	pop	SavSeg			;reset to start
	mov	dx,03C4H		;select plane write mask reg adrs
	mov	al,2
	out	dx,al
	inc	dx			;select plane write mask data port
	mov	al,0FFH			;enable all planes
	out	dx,al
	ret
xrest	endp
;
;	----------------- local use procedures ----------------------
;
setgrm	proc	near			;set screen to graphics mode
	mov	ax,092H			;select 640x480 color mode
	int	10H
	mov	ax,0500H		;select page 0 to display
	int	10H
	call	SETPAL			;setup our palette
	SGREG	8,0FFH			;enable all pixels to be changed
	SGREG	1,0H			;disable S/R (direct write in wm0)
	ret
setgrm	endp
;
settxm	proc	near			;set screen to text mode
	mov	ax,3			;reset to text mode
	int	10H
	call	DOSPAL			;restore normal palette
	ret
settxm	endp
;
cscreen	proc	near			;clear the screen
	SGREG	5,0			;write mode 0
	SGREG	3,0H			;write normal data
	mov	ax,dwseg		;point to display
	mov	es,ax
	mov	cx,sbsiz		;screen size in words
	sub	ax,ax			;clear data
	mov	di,ax
	cld				;scan upwards
	rep	stosw			;clear screen
	call	spccm			;set current color and mode
	ret
cscreen	endp
;
sxor	proc	near
	SGREG	0,00Fh			;S/R = all bits of pixel xor
	SGREG	5,3			;write mode 3
	SGREG	3,18H			;function = xor S/R data
	ret
sxor	endp
;
sdrw	proc	near
	SGREGD	0,<byte ptr CurClr>	;S/R = color to write
	SGREG	3,0			;function = copy S/R data
	SGREG	5,3			;write mode 3
	ret
sdrw	endp
;
sera	proc	near
	SGREGD	0,<byte ptr bkgnd>	;S/R = color to write
	SGREG	3,0			;function = copy S/R data
	SGREG	5,3			;write mode 3
	ret
sera	endp
;
;	----------------- select prompt text size ----------------------
;
SELSPT:	entry				;select small prompt text
	mov	tccw,1
	mov	nrast,12
	mov	dstcol,80
	mov	dstlin,40
	mov	dstxh,12
	mov	dstxw,8
	mov	cgenr,402h		;cgen resource #
	mov	ax,offset ltbls
	mov	ltblptr,ax
	exit
;
SELLPT:	entry				;select small prompt text
	mov	tccw,2
	mov	nrast,16
	mov	dstcol,40
	mov	dstlin,30
	mov	dstxh,16
	mov	dstxw,16
	mov	cgenr,401h		;cgen resource #
	mov	ax,offset ltbll
	mov	ltblptr,ax
	exit
;
DEVCFG:	entry				;device specific configuration
	exit
;
;	--------------------- local data items -------------------------
;
	align	4
dvrsvc	dd	0			;pointer to driver services
SHdl	dd	0			;save buffer RAM block handle
CGHdl	dd	0			;CGEN rsc handle
EMMHdl	dw	0			;EMM screen save bfr handle
EMMSeg	dw	0			;EMM screen save bfr segment
dwseg	dw	0A000H			;screen drawing segment
sbsiz	dw	4B00H			;#words per screen page
SavSeg	dw	0			;allocated save segment
yarsav	dw	0			;animate aspect ratio save
xarsav	dw	0
;
curclr	dw	3			;current color #
curpat	dw	0FFFFH			;current line pattern
bkgnd	dw	0			;current background color
;
pmasks	db	80H,40H,20H,10H,8,4,2,1	;pixel select masks
;
;	LTBL(line*2)=offset of text lines 0(top) thru 39(bottom)
;
	align	2
ltbls	dw 0000H,03C0H,0780H,0B40H,0F00H,12C0H,1680H,1A40H,1E00H,21C0H
	dw 2580H,2940H,2D00H,30C0H,3480H,3840H,3C00H,3FC0H,4380H,4740H
	dw 4B00H,4EC0H,5280H,5640H,5A00H,5DC0H,6180H,6540H,6900H,6CC0H
	dw 7080H,7440H,7800H,7BC0H,7F80H,8340H,8700H,8AC0H,8E80H,9240H
;
ltbll	dw 0000H,0500H,0A00H,0F00H,1400H,1900H,1E00H,2300H,2800H,2D00H
	dw 3200H,3700H,3C00H,4100H,4600H,4B00H,5000H,5500H,5A00H,5F00H
	dw 6400H,6900H,6E00H,7300H,7800H,7D00H,8200H,8700H,8C00H,9100H
;
;	---------------- driver parameter data -------------------------
;
dsidrn	dw	1			;display driver id resource #
dsxmax	dw	639			;# X dots -1
dsymax	dw	479			;# Y dots -1
dsyar	dw	77			;aspect ratio Y/X
dsxar	dw	98
dstcol	dw	80			;# text columns
dstlin	dw	40			;# text lines
dsnclrs	dw	15			;max color # (0-ncolor)
dstxh	dw	12			;text height in pixels
dstxw	dw	8			;text width in pixels
dsflags	dw	3			;DSCIR/DSARC available
dsf2	dw	0			;more flag bits
slow	db	0			;slow function bits
iconsc	db	2			;16x16 icons
	db	16 dup (0)		;reserved for future use
;
;	--------------- driver resource data -------------------------
;
DSRSC	dw	1			;# of resource items available
	dw	offset dsid		;1=driver id text
;
dsid	db	'VGA in 640x480 16-color mode',0  ;64-chr max
;
	include	DSLAST.CPY
