;	DSVGA800.ASM - Extended VGA 800x600x16 color Display Driver
;	using VGA write mode 3 logic
;
;	(c) Copyright 1986,7,8 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  7/2/88
;	800x600 generic version 3/17/89 MR
;	devrev=1:added many more brand names to the list 9/09/90 SB
;		 added Oak and Cirrus 12/29/90 SB
;	devrev=2:added SLIDES and ANIMATION 1/27/91 SB
;
	name	DSVGA800
devrev	equ	2
cfgctl	equ	7			;DEVCFG, Aspect Ratio, text size
;
	include	DSDRV.CPY
;
;	VGA graphics register set macro
;
SGREGD	macro	sel,value
	mov	dx,03CEh		;select register
	mov	al,sel
	mov	ah,value
	out	dx,ax
	endm
;
SGREG	macro	sel,value
	mov	dx,03CEh		;select register
	mov	ax,sel+value*256
	out	dx,ax
	endm
;
spccm	dw	sdrw			;current mode & color set routine
;
;	---------------------------------------------------------------
;	DEVCFG - Select Specific VGA Card
;	(All CONIN/CONOUT must use DOS functions)
;	---------------------------------------------------------------
;
DEVCFG:	entry
ctagn:	mov	bx,offset DSRSC+2+2	;adrs of resource #2
	mov	dx,[bx]			;msg address in DX
	mov	ah,9			;display message
	int	21H
	mov	ah,7			;get key
	int	21H
	push	ax			;echo choice
	mov	dl,al
	mov	ah,2
	int	21H
	pop	ax
	cmp	al,13			;CR?
	jz	short dfltct
	cmp	al,3			;CTRL-C?
	jz	short noadj
	sub	al,'1'			;range 0..7, currently
	jl	ctagn
	cmp	al,7
	ja	ctagn
	jmp	short okctp
dfltct:	mov	al,byte ptr cardtp	;CR default is current
okctp:	push	ax
	mov	byte ptr cardtp,al	;save selection
	add	al,'1'			;store selection in message
	mov	bx,offset DSRSC+3+3	;ctp address
	mov	bx,[bx]
	mov	[bx],al
noadj:	mov	dx,offset ccrlf		;crlf to end dialog
	mov	ah,9
	int	21H
	pop	bx
	sub	bh,bh
	shl	bx,1
	mov	ax,xartbl[bx]		;setup aspect ratios	
	mov	dsxar,ax
	mov	ax,yartbl[bx]		;for different chipsets
	mov	dsyar,ax
	exit
;
ccrlf	db	13,10,'$'
;
;
;	DSINIT - initialize the display driver
;
;	This will only be called once per program execution
;	before any other display driver calls.  It must:
;	1 - test for correct display hardware available
;	2 - allocate any required buffer memory from DOS
;	3 - perform global & local variable initialization
;	4 - initialize the display hardware
;	5 - select graphics display mode
;	6 - clear the screen
;	7 - set AX=error code:
;	    0=ok
;	    1=not enough memory
;	    2=not enough EMM memory
;	    3=device unavailable/unuseable
;	    4=CGEN resource missing
;
DSINIT:	entry
	mov	word ptr DVRSVC,di	;save pointer to driver services
	mov	word ptr DVRSVC+2,ES
	call	ASBFR			;init emm save screen
	jnc	short ok1
	mov	ax,2			;no EMM 
	exit
ok1:	mov	ax,CGENR		;load char gen resource
	mov	bx,0			;into alloc segment
	mov	dx,0			;RSCLD function
	call	dword ptr DVRSVC	;do it
	jnc	short ok2
	mov	ax,4			;missing CGEN resource
	exit
ok2:	mov	ax,ES:[bx]		;get segment from handle
	mov	CGSeg,ax		;save segment
	mov	word ptr CGHdl,bx	;save handle
	mov	ax,ES
	mov	word ptr CGHdl+2,ax
	call	setgrm			;select graphics screen mode
	call	cscreen			;clear the screen
	call	spccm			;set pointers for init color and mode
	sub	ax,ax			;ax=0: device available for use
	exit
;
;	DSEND - end use of the display driver
;
;	This will only be called once per program execution
;	before program termination.  It must:
;	1 - restore any interrupt vectors changed by this driver
;	2 - restore the display to DOS-compatible text mode
;	3 - clear the text screen
;	4 - release any allocated buffer memory
;
DSEND:	entry
	call	settxm			;select text screen mode
	les	bx,CGHdl		;release CGEN resource
	mov	dx,1
	call	dword ptr DVRSVC	;do it
	call	RSBFR			;release emm save screen
	exit
;
;	DSGRM - set screen to graphics display mode
;
DSGRM:	entry
	call	setgrm			;select graphics display
	call	srest			;restore saved screen
	exit
;
;	DSTXM - set screen to DOS-compatible text mode
;
DSTXM:	entry
	call	ssave			;save graphics display
	call	settxm			;select text display
	exit
;
;	DSCLEAR - clear the graphics screen
;
DSCLEAR: entry
	call	cscreen			;clear the screen
	exit
;
;	DSERA - select draw background mode (erase)
;
DSERA:	entry
	mov	spccm,offset sera
	call	sera
	exit
;
;	DSDRAW - select draw specified color mode
;
DSDRAW:	entry
	mov	spccm,offset sdrw
	call	sdrw
	exit
;
;	DSINV - select draw inverted pixel mode
;
DSINV:	entry
	mov	spccm,offset sxor
	call	sxor
	exit
;
;	DSCOLOR - specify current drawing color & background
;
DSCOLOR: entry
	mov	byte ptr bkgnd,cl	;save background color
	mov	byte ptr CurClr,al	;set new current color
	call	spccm			;set pointers
	exit
;
;	---------------- Set Dot Macros used by SDSUBS --------------
;
SDGPM	macro				;get pixel modification masks
	mov	bx,si			;x mod 8
	and	bx,7
	mov	AH,pmasks[bx]		;mask for selected pixel
	endm
;
SDCBO	macro				;calculate byte offset to change
	push	dx
	push	ax
	mov	ax,dsymax		;invert y value (0 at bottom)
	sub	ax,di
	mul	pitch
	mov	bx,ax
	pop	ax
	pop	dx
	mov	bp,si			;work copy of X
	shr	bp,1			;+ x/8
	shr	bp,1
	shr	bp,1
	add	bx,bp			;ES:[bx] = byte to change
	endm
;
SDCSP	macro				;change selected pixel
	mov	al,ES:[bx]		;read data into latches
	mov	ES:[bx],ah		;set pixel mask
	endm
;
SDUPL	macro				;move offset up one line
	sub	bx,pitch		;bump line offset
	endm
;
SDDNL	macro				;move offset down one line
;					;CS override needed during GCHR macro
	add	bx,CS:pitch		;bump line offset
	endm
;
SDUPC	macro
	ror	ah,1			;shift pixel mask right
	adc	bx,0			;if wrap, bump byte offset
	endm
;
SDDNC	macro
	rol	ah,1			;shift pixel mask left
	sbb	bx,0			;if wrap, bump byte offset
	endm
;
;	CCHRO - calculate character offset to BX; DX=line, AX=column
;
CCHRO	macro				;calculate char offset
	call	cchros
	endm
;
cchros	proc	near
	mov	bx,dx			;line number
	shl	bx,1			;word index
	add	bx,ltblptr		;lookup line offset
	mov	bx,[bx]
	add	bx,ax			;+cposn
	cmp	tccw,2			;*2?
	jnz	notx2
	add	bx,ax			;yes
notx2:	SGREG	3,0			;function = latch copy data
	SGREG	5,0			;write mode 0
	ret
cchros	endp
;
;	GCHR - generate character from DS:[SI] cgen table, ES:BX=chr cell
;	BP= inversion mask 0 or -1; CX = NRAST value for loop; DX=tccw
;
GCHR	macro				;generate character
	local	gc1,gc2,gcxt
	cmp	dx,2			;word or byte?
	jz	short gc2
	sub	cx,4			;change to 12 rasters CGEN
	mov	ax,bp			;get fill data
	mov	ES:[bx],al		;start with two blank lines
	sddnl
	mov	ES:[bx],al
	sddnl
gc1:	lodsb				;get raster line
	xor	ax,bp			;invert if needed
	mov	ES:[bx],al		;set it into display
	sddnl				;move down to next raster
	loop	short gc1		;loop for entire character
	mov	ax,bp			;get fill data
	mov	ES:[bx],al		;end with two blank lines
	sddnl
	mov	ES:[bx],al
	sddnl
	jmp	short gcxt
gc2:	lodsw				;get raster line
	xor	ax,bp			;invert if needed
	xchg	al,ah			;byte swap
	mov	ES:[bx],ax		;set it into display
	sddnl				;move down to next raster
	loop	short gc2		;loop for entire character
gcxt:
	endm
;
;	INVCR - invert character raster line ES:[BX], CX characters
;
INVCR	macro
	call	invcrs
	endm
;
invcrs	proc	near
invlup:	cmp	tccw,2			;word or byte?
	jz	inv2
	not	byte ptr ES:[bx]	;invert character raster
	jmp	short invc
inv2:	not	word ptr ES:[bx]	;invert character raster
invc:	add	bx,tccw			;next character
	loop	invlup			;for all char posns
	ret
invcrs	endp
;
;	CLRCR - clear character raster line ES:[BX],CX characters
;
CLRCR	macro
	call	clrcrs
	endm
;
clrcrs	proc	near
	sub	ax,ax			;clear to value
clrlup:	cmp	tccw,2			;word or byte?
	jz	clr2
	mov	ES:[bx],al		;clear chr raster
	jmp	short clrc
clr2:	mov	ES:[bx],ax		;clear chr raster
clrc:	add	bx,tccw			;bump chr cell
	loop	clrlup			;for all char positions
	ret
clrcrs	endp
;
;	-------------------------------------------------------------------
;
tccw	dw	1			;text character cell width in bytes
nrast	dw	16			;raster lines per character
;
CGENR	dw	402h			;character generator resource #
LTBLPTR	dw	offset ltbll		;line offset table
;
sdfhl	macro
	call	VGAFHL
	endm
;
;	SDFHL entry: SCANYL = Y value of horiz line
;	BP=lowX, CX=#pixels, ES=DwSeg, DX=pixel masks (if set by SDGPM)
;
VGAFHL	proc	near
	mov	si,scanyl		;select current lines' pattern
	and	si,07H
	mov	ah,FilPat[si]
	mov	si,bp			;x column position
	and	si,7			;mod 8 = left byte pixel mask index
	sub	cx,8			;remove left byte pixels from count
	add	cx,si
	jge	ntsngl			;more than one byte changes?
	mov	al,lbpmt[si]		;set left byte pixel mask
	and	al,ah			;(pattern mask)
	lea	si,rbpmt+8		;less unchanged right pixels
	add	si,cx	
	and	al,[si]
	mov	dl,ES:[bx]		;read latch data
	mov	ES:[bx],al		;set selected pixels
	ret
ntsngl:	mov	di,cx			;right byte pixel mask index
	and	di,7
	shr	cx,1			;remove right byte pixels from count
	shr	cx,1
	shr	cx,1
	mov	al,lbpmt[si]		;set left byte pixel mask
	and	al,ah			;(pattern mask)
	mov	dl,ES:[bx]		;read latch data
	mov	ES:[bx],al		;set selected pixels
	inc	bx			;bump byte to change offset
	jcxz	chgrb
	mov	al,ah			;set middle bytes pixel mask
chgmb:	mov	dl,ES:[bx]		;read latch data
	mov	ES:[bx],al		;set selected pixels
	inc	bx			;bump byte to change offset
	loop	chgmb			;loop for all middle bytes
chgrb:	mov	al,rbpmt[di]		;set right byte pixel mask
	and	al,ah			;(pattern mask)
	mov	dl,ES:[bx]		;read latch data
	mov	ES:[bx],al		;set selected pixels
	ret
vgafhl	endp
;
lbpmt	db	0FFH,07FH,03FH,01FH,00FH,007H,003H,001H
rbpmt	db	000H,080H,0C0H,0E0H,0F0H,0F8H,0FCH,0FEH
;
LSScl	equ	2			;line style scale
	include SDSUBS4.CPY
	include	DSBLN.CPY
	include	VTEXT.CPY
	include VGAPAL.CPY
;
ANIMF	db	0			;animation flag
;
;	DSABGN - Setup screen mode for animation
;
DSABGN:	entry
	mov	ax,10H			;select 640x350 color mode
	int	10H
	call	SETPAL			;setup our palette
	SGREG	8,0FFH			;enable all pixels to be changed
	SGREG	1,0H			;disable S/R (direct write in wm0)
	mov	dsxmax,639		;shrink screen area
	mov	dsymax,349
	mov	ax,dsyar
	mov	yarsav,ax
	mov	ax,dsxar
	mov	xarsav,ax
	mov	dsyar,77
	mov	dsxar,103
	mov	sbsiz,4000h
	mov	pitch,80		;lower the line pitch
	exit
;
;	DSAUPD - Animation Update
;
;	Copy drawing buffer to 2nd screen buffer
;	Force display of 2nd screen buffer
;
DSAUPD:	entry
;
;	copy low 32k to hi 32k screen memory on card
;
	mov	cx,sbsiz		;buffer size in words
	shl	cx,1			;in bytes
	push	ds
	mov	ax,dwseg		;display memory segment
	mov	ds,ax
	mov	es,ax
	mov	si,0000H		;from zero (active page)
	mov	di,8000H		;to inactive page
	cld				;upscan
	SGREG	8,0			;copy all write from prior read
	SGREG	3,0			;write normal data
	rep	movsb			;copy all data (32-bit copies)
	pop	ds
	SGREG	8,0FFH			;enable all pixels to be changed
	call	spccm			;set current color and mode
;
;	switch to display high buffer area
;
	test	animf,1			;doing animation?
	jz	short stanim
	exit				;yes
stanim:	mov	animf,1			;no - flag animation started
	mov	dx,3DAh			;CRTC status port
wvrs:	in	al,dx			;wait for start of vert retrace
	test	al,8
	jz	short wvrs
wvre:	in	al,dx			;wait for end of vert retrace
	test	al,8
	jnz	short wvre
	cli				;disable interrupts
	sub	dl,6			;CRTC I/O port
	mov	ax,800Ch		;start adrs high
	out	dx,ax
	mov	ax,000Dh		;start adrs low
	out	dx,ax
	sti				;enable interrupts
	exit
;
;	DSAEND - Animation End
;
;	Force display of normal draw screen buffer
;
DSAEND:	entry
	test	animf,1			;doing animation?
	jnz	short eanim
	exit				;no
eanim:	call	setgrm			;select 800x600 color mode
	mov	dsxmax,799		;restore screen area
	mov	dsymax,599
	mov	ax,yarsav
	mov	dsyar,ax
	mov	ax,xarsav
	mov	dsxar,ax
	mov	sbsiz,7530H
	mov	animf,0			;yes - flag animation ended
	mov	dx,3DAh			;CRTC status port
ewvrs:	in	al,dx			;wait for start of vert retrace
	test	al,8
	jz	short ewvrs
ewvre:	in	al,dx			;wait for end of vert retrace
	test	al,8
	jnz	short ewvre
	cli				;disable interrupts
	sub	dl,6			;CRTC I/O port
	mov	ax,000Ch		;start adrs high
	out	dx,ax
	mov	ax,000Dh		;start adrs low
	out	dx,ax
	sti				;enable interrupts
	mov	pitch,100		;restore normal line pitch
	exit
;
;	DSSAVE - save the graphics screen
;
DSSAVE:	entry
	call	ssave			;save current screen
	exit
;
DSREST:	entry
	call	srest			;restore prior screen
	exit
;
;	SVGA 800x600 Disk File Screen SAVE/RESTORE Routines
;
;	Parameter is DSKTYP equ save data format type id
;	BX = open file handle for read/write (do not close)
;
;	0 = screen save unimplimented
;	1 = 640 x 480 VGA or EGA+
;	2 = 640 x 350 EGA
;	3 = other
;	4 = other
;	5 = 800 x 600 SVGA
;	6 =1024 x 768 SVGA
;
DskTyp	equ	5			;disk save type id
DskSiz	equ	0EA60H			;disk save bytes per plane=60000
;
DPSAVE	macro	arg
	mov	al,arg			;enable selected plane read
	out	dx,al
	push	dx			;save port
	mov	ah,40h			;write to BX handle
	sub	dx,dx			;DS:0000h:dsksiz-1
	mov	cx,DskSiz		;bytes per plane
	int	21H
	pop	dx			;restore port
	endm
;
dsktw	dw	dsktyp
dsktr	dw	dsktyp
;
;	DSDSKS - Save screen image to file handle BX
;
DSDSKS:	entry
	SGREG	5,0			;write mode 0
	mov	ah,40h			;write to BX handle
	lea	dx,dsktw		;DS:dsktw
	mov	cx,2			;2 bytes
	int	21H
	push	ds
	mov	ax,dwseg		;display memory segment
	mov	ds,ax
	cld				;upscan
	mov	dx,03CEH		;select read map register adrs
	mov	al,4
	out	dx,al
	inc	dx			;select read map data port
	dpsave	0
	dpsave	1
	dpsave	2
	dpsave	3
	pop	ds			;restore working segments
	call	spccm			;set current color and mode
	clc				;worked ok
	exit
;
DPREST	macro	arg
	mov	al,arg			;enable selected write plane
	out	dx,al
	push	dx			;save port
	mov	ah,3Fh			;read from BX handle
	mov	dx,bp			;DS:0000h:dsksiz-1
	mov	cx,di			;bytes per plane to read
	int	21H
	mov	ax,4201h		;seek from present posn
	sub	cx,cx			;CX:DX bytes
	mov	dx,si
	int	21h
	pop	dx			;restore port
	endm
;
;	DSDSKR - Restore screen image from file handle BX
;
DSDSKR:	entry
	mov	dsktr,0			;clear read type
	mov	ah,3Fh			;read from BX handle
	lea	dx,dsktr		;DS:dsktr
	mov	cx,2			;2 bytes
	int	21H
	mov	ax,dsktr		;is it compatible?
	cmp	ax,5
	jnz	dfnotc
	mov	di,DskSiz		;set read plane count
	mov	si,0			;set skip after read count
	mov	bp,0			;set read start offset
	cmp	ax,dsktw		;this drivers' own format?
	jz	dfisc
dfnotc:	stc				;no - not restored
	mov	ax,0			;no read type
	exit
dfisc:	SGREG	5,0			;write mode 0
	SGREG	3,0H			;write normal data
	push	ds
	mov	ax,dwseg		;set video segment
	mov	ds,ax
	cld				;upscan
	mov	dx,03CEH		;select pixel mask register adrs
	mov	al,8
	out	dx,al
	inc	dx			;enable all pixels of a byte
	mov	al,0FFH
	out	dx,al
	mov	dx,03C4H		;select plane write mask reg adrs
	mov	al,2
	out	dx,al
	inc	dx			;select plane write mask data port
	dprest	1
	dprest	2
	dprest	4
	dprest	8
	mov	dx,03C4H		;select plane write mask reg adrs
	mov	al,2
	out	dx,al
	inc	dx			;select plane write mask data port
	mov	al,0FFH			;enable all planes
	out	dx,al
	pop	ds			;restore working segments
	call	spccm			;set current color and mode
	mov	ax,dsktr		;return loaded type code
	clc				;worked ok
	exit
;
;	The SVGA board uses EA60H (60000) bytes per plane. To save
;	the screen, it is necessary to save the image in EMM RAM
;	(60000 bytes/plane) * 4 bit-planes (240000 bytes EMM RAM total).
;
;	ASBFR - allocate save buffer - set JC if error
;
ASBFR	proc	near
	mov	ax,16			;# pages needed = 14.64
	mov	dx,8			;allocate EMM RAM block
	call	dword ptr DVRSVC
	jc	asbxt			;error?
	mov	word ptr SHdl,bx	;save block handle
	mov	SavSeg,ax		;and access segment
	clc				;flag aloc. ok
asbxt:	ret				;carry set if error
asbfr	endp
;
;	RSBFR - release save buffer
;
RSBFR	proc	near
	mov	bx,word ptr SHdl	;EMM RAM block handle
	mov	dx,9			;release EMM RAM block
	call	dword ptr DVRSVC
	ret				;carry set if error
rsbfr	endp
;
;	Save Entire Screen
;
PSAVE	macro	arg
	local	ploop
	mov	dx,3CFh			;select read map data port
	mov	al,arg			;enable selected plane read
	out	dx,al
	mov	cx,4			;loop for # of 16k pages
	sub	si,si			;display plane memory start
ploop:	push	cx			;save loop counter
	push	ES			;save context
	push	DS
	push	si
	push	bp
	push	bx
	mov	ax,bp			;access block
	mov	dx,10
	call	dword ptr CS:DVRSVC
	pop	bx			;restore context
	pop	bp
	pop	si
	pop	DS
	pop	ES
	inc	bp			;bump page #
	sub	di,di			;save plane buffer
	mov	cx,2000h		;8 kwords
	cld				;upscan
	rep	movsw			;copy page
	pop	cx			;loop for 4-16k blocks
	loop	short ploop
	endm
;
ssave	proc	near
	push	es			;save working segments
	push	ds
	mov	bx,word ptr SHdl	;EMM page handle
	mov	ES,SavSeg		;save buffer segment
	mov	DS,dwseg		;display memory segment
	mov	dx,03CEH		;select read map register adrs
	mov	al,4
	out	dx,al
	sub	bp,bp			;EMM page start
	psave	0
	psave	1
	psave	2
	psave	3
	pop	ds			;exit dsgsav
	pop	es
	call	spccm			;set current color and mode
	ret
ssave	endp
;
PREST	macro	arg
	local	ploop
	mov	dx,03C4H		;select plane write mask reg adrs
	mov	al,2
	out	dx,al
	inc	dx			;select plane write mask data port
	mov	al,arg			;enable selected plane read
	out	dx,al
	mov	cx,4			;loop for # of 16k pages
	sub	di,di			;display plane memory start
ploop:	push	cx			;save loop counter
	push	ES			;save context
	push	DS
	push	di
	push	bp
	push	bx
	mov	ax,bp			;access block
	mov	dx,10
	call	dword ptr CS:DVRSVC
	pop	bx			;restore context
	pop	bp
	pop	di
	pop	DS
	pop	ES
	inc	bp			;bump page #
	sub	si,si			;restore plane buffer
	mov	cx,2000h
	cld				;upscan
	rep	movsw			;copy page
	pop	cx			;loop for 4-16k blocks
	loop	short ploop
	endm
;
srest	proc	near
	SGREG	5,0			;write mode 0
	mov	bx,word ptr SHdl	;EMM page handle
	push	es			;save working segments
	push	ds
	mov	ES,dwseg		;set video segment
	mov	DS,SavSeg		;save buffer segment
	cld				;upscan
	mov	dx,03CEH		;select pixel mask register adrs
	mov	al,8
	out	dx,al
	inc	dx			;enable all pixels of a byte
	mov	al,0FFH
	out	dx,al
	sub	di,di			;display plane memory start
	sub	bp,bp			;EMM page start
	prest	1
	prest	2
	prest	4
	prest	8
	mov	dx,03C4H		;select plane write mask reg adrs
	mov	al,2
	out	dx,al
	inc	dx			;select plane write mask data port
	mov	al,0FFH			;enable all planes
	out	dx,al
	pop	ds			;exit dsgsav
	pop	es
	call	spccm			;set current color and mode
	ret
srest	endp
;
;	----------------- local use procedures ----------------------
;
setgrm	proc	near			;set screen to graphics mode
	mov	bx,cardtp		;select desired init routine
	shl	bl,1
	jmp	itbl[bx]
itbl	dw	offset init1		;Video7 VRAM/FastWrite
	dw	offset init2		;STB VGA Extra/EM
	dw	offset init3		;Paradise/AST VGA
	dw	offset init4		;ATI Wonder
	dw	offset init5		;Trident
	dw	offset init6		;VESA
	dw	offset init7		;Oak
	dw	offset init8		;Cirrus
;
init1:	mov	ax,06F05H		;select 800x600x16 color mode
	mov	bl,62h
	int	10H			;for Video 7 VRAM, FastWrite
	jmp	short grmfin
;
init2:	mov	ax,00A9H		;select 800x600x16 color mode
	int	10H			;for STB VGA Extra/EM
	jmp	short grmfin
;
init3:	mov	ax,0058H		;select 800x600x16 color mode
	int	10H			;for AST/Paradise VGA Plus
	jmp	short grmfin
;
init4:	mov	ax,00D4H		;select 800x600x16 color mode
	int	10H			;for ATI Wonder
	jmp	short grmfin
;
init5:	mov	ax,005BH		;select 800x600x16 color mode
	int	10H			;for TRIDENT TVGA
	jmp	short grmfin
;
init6:	mov	ax,006AH		;select 800x600x16 color mode
	int	10H			;for "VESA"
	jmp	short grmfin
;
init7:	mov	ax,0052H		;select 800x600x16 color mode
	int	10H			;for OAK 
	jmp	short grmfin
;
init8:	mov	ax,0064H		;select 800x600x16 color mode
	int	10H			;for Cirrus
;
grmfin:	call	SETPAL			;setup our palette
	SGREG	8,0FFH			;enable all pixels to be changed
	SGREG	1,0H			;disable S/R (direct write in wm0)
	ret
setgrm	endp
;
settxm	proc	near			;set screen to text mode
	mov	ax,3			;reset to text mode
	int	10H
	call	DOSPAL			;restore normal palette
	ret
settxm	endp
;
cscreen	proc	near			;clear the screen
	SGREG	5,0			;write mode 0
	SGREG	3,0H			;write normal data
	mov	ax,dwseg		;point to display
	mov	es,ax
	mov	cx,8000h		;screen size in words
	sub	ax,ax			;clear data
	mov	di,ax
	cld				;scan upwards
	rep	stosw			;clear screen
	call	spccm			;set current color and mode
	ret
cscreen	endp
;
sxor	proc	near
	SGREG	0,00Fh			;S/R = all bits of pixel xor
	SGREG	5,3			;write mode 3
	SGREG	3,18H			;function = xor S/R data
	ret
sxor	endp
;
sdrw	proc	near
	SGREGD	0,<byte ptr CurClr>	;S/R = color to write
	SGREG	3,0			;function = copy S/R data
	SGREG	5,3			;write mode 3
	ret
sdrw	endp
;
sera	proc	near
	SGREGD	0,<byte ptr bkgnd>	;S/R = color to write
	SGREG	3,0			;function = copy S/R data
	SGREG	5,3			;write mode 3
	ret
sera	endp
;
;	----------------- select prompt text size ----------------------
;
SELSPT:	entry				;select small prompt text
	mov	tccw,1
	mov	dstcol,100
	mov	dstxw,8
	mov	cgenr,402h		;cgen resource #
	exit
;
SELLPT:	entry				;select large prompt text
	mov	tccw,2
	mov	dstcol,50
	mov	dstxw,16
	mov	cgenr,401h		;cgen resource #
	exit
;
;	--------------------- local data items -------------------------
;
	align	4
dvrsvc	dd	0			;pointer to driver services
SHdl	dd	0			;save buffer RAM block handle
CGHdl	dd	0			;CGEN rsc handle
EMMHdl	dw	0			;EMM screen save bfr handle
EMMSeg	dw	0			;EMM screen save bfr segment
dwseg	dw	0A000H			;screen drawing segment
sbsiz	dw	7530H			;#words per screen plane
SavSeg	dw	0			;allocated save segment
pitch	dw	100			;SDCBO/SDUPL/SDDNL line pitch
;
curclr	dw	3			;current color #
curpat	dw	0FFFFH			;current line pattern
bkgnd	dw	0			;current background color
;
pmasks	db	80H,40H,20H,10H,8,4,2,1	;pixel select masks
;
;	LTBL(line*2)=offset of text lines 0(top) thru 36(bottom)
;
ltbll	dw	0,1600,3200,4800,6400,8000,9600,11200,12800,14400
	dw	16000,17600,19200,20800,22400,24000,25600,27200
	dw	28800,30400,32000,33600,35200,36800,38400,40000
	dw	41600,43200,44800,46400,48000,49600,51200,52800
	dw	54400,56000,58400
;
xartbl	dw	99,98,98,98,99,98,98,98
yartbl	dw	86,84,94,79,85,91,81,81
yarsav	dw	0
xarsav	dw	0
;
;	---------------- global data items -------------------------
;
cardtp	dw	0			;card id #
;
dsidrn	dw	1			;device id resource #
dsxmax	dw	799			;# X dots -1
dsymax	dw	599			;# Y dots -1
dsyar	dw	77			;aspect ratio Y/X
dsxar	dw	98
dstcol	dw	100			;# text columns
dstlin	dw	37			;# text lines
dsnclrs	dw	15			;max color # (0-ncolor)
dstxh	dw	16			;text height in pixels
dstxw	dw	8			;text width in pixels
dsflags	dw	3			;DSCIR/DSARC available
dsf2	dw	0			;more flags
slow	db	0			;slow function
iconsc	db	2			;16x16 icons icons
	db	16 dup (0)		;reserved for future use
;
DSRSC	dw	3			;total # of resources
	dw	offset dsid		;1=device id text
	dw	offset csel		;2=card type selection
	dw	offset ctp		;3=card type tens character
;
dsid	db	'Extended VGA in 800x600 16-color mode.',0
csel	db	13,10,'Select VGA type:',13,10,13,10
	db	'  1: Headland Video 7 VRAM/FastWrite',13,10
	db	'  2: STB PowerGraph/PowerView/VGA Plus/Extra/EM',13,10
	db	'     Orchid ProDesigner VGA/Plus',13,10
	db	'     Genoa 5300/5400',13,10
	db	'     Sigma VGA/H, Willow',13,10
	db	'  3: AST VGA Plus',13,10
	db	'     Paradise VGA Plus/Pro/1024/Plus-16',13,10
	db	'  4: ATI VGA Wonder',13,10
	db	'  5: Trident TVGA 8816/8916',13,10
	db	'     Imtec, Logix, Maxon',13,10
	db	'  6: Ahead VGA Wizard Deluxe',13,10
	db	'     Zymos Super RVGA II',13,10
	db	'     Genoa Super VGA 6300/6400',13,10
	db	'     "VESA"-compatables',13,10
	db	'  7: Oak OTI-037/067',13,10
	db	'  8: Cirrus Logic VGA (CRT only)',13,10,13,10
	db	'Enter choice 1..8 ['
ctp	db	'1]:$'
;
	include DSLAST.CPY
