;	EGAPAL.CPY - EGA driver color palette functions
;
;	Copyright 1990 Evolution Computing
;	All rights reserved.
;	Written by Mike Riddle 9-2-90
;
;	----------------------------------------------------------------
;	The palette data is 48 bytes: 16 3-byte sets of Red, Green,
;	then Blue values for each of the 16 user colors 0..15.
;	Each value is stored in a "standard" form: 0..255 with 255
;	representing the brightest level.
;	----------------------------------------------------------------
;
PAL	macro	pname
pname	db	000H, 000H, 000H	;black
	db	000H, 0F8H, 000H	;lime green
	db	0E8H, 000H, 038H	;red
	db	000H, 000H, 0F8H	;blue
	db	0F8H, 0E8H, 000H	;YELLOW
	db	000H, 0C8H, 0D8H	;CYAN
	db	0F8H, 000H, 0F8H	;MAGENTA
	db	0A8H, 000H, 0E8H	;PURPLE
	db	0F8H, 048H, 000H	;ORANGE
	db	068H, 088H, 0F8H	;LIGHT BLUE
	db	0F8H, 098H, 058H	;SAND
	db	0A8H, 048H, 000H	;BROWN
	db	088H, 088H, 038H	;OLIVE GREEN
	db	000H, 068H, 038H	;FOREST GREEN
	db	080H, 080H, 080H	;GREY
	db	0FFH, 0FFH, 0FFH	;WHITE
	endm
;
	PAL	PalDta			;user-changeable
	PAL	StdDta			;Standard restore copy
;
;	----------------------------------------------------------------
;
;	EGA palette values are:  00rgbRGB bits
;	where lc is dim, uc is bright
;
;	dk grn 010 med grn 002 brt grn 012 (green phosphor is dayglo)
;	dk red 020 med red 004 brt red 024 (red phosphor is orangish)
;	dk blu 008 med blu 001 brt blu 009
;
EGAPal	db	00H,2H,4H,1H,36H,1BH,2DH,0DH
	db	34H,31H,26H,14H,30H,10H,38H,07H,00H
;
bootpal	db	0,1,2,3,4,5,6,7,38H,39H,3AH,3BH,3CH,3DH,3EH,3FH,0
;
;	----------------------------------------------------------------
;	DSRPAL - READ 48-byte color palette from driver
;	ES:DI -> palette data destination
;	Carry bit Set = standard set, clear = user set
;	----------------------------------------------------------------
;
DSRPAL:	ENTRY
	mov	si,offset PalDta	;copy changeable palette data
	jnc	short rdpal
	mov	si,offset StdDta	;copy standard palette data
rdpal:	cld
	mov	cx,24			;24 words
	rep	movsw			;do it
	EXIT
;
;	----------------------------------------------------------------
;	DSWPAL - WRITE 48-byte color palette to driver
;	ES:DI -> palette data source
;	----------------------------------------------------------------
;
DSWPAL:	ENTRY
	mov	si,di			;setup source
	mov	di,offset PalDta	;our data is destination
	push	DS			;save for our DS restore
	push	DS			;swap DS & ES
	push	ES
	pop	DS
	pop	ES
	cld				;upscan
	mov	cx,24			;24 words
	rep	movsw			;copy data
	pop	DS			;restore our DS
	call	SetPal
	EXIT
;
;	----------------------------------------------------------------
;	SETPAL - Setup local palette data to display
;	----------------------------------------------------------------
;
;	SCBITS:
;	get 0..255 color byte from PalDta
;	change AL bits 21xx.xxxx
;	to bits 0001.0002
;	and OR with DL
;
SCBITS	macro
	lodsb				;get next 0..255 color value
	shl	dl,1			;advance color assembly R->G->B
	mov	ah,al			;save copy of bit 80h
	shr	al,1			;AL=0x1x.xxxx
	shr	al,1			;AL=00x1.xxxx
	shr	al,1			;AL=000x.1xxx
	and	al,08h			;AL=0000.1000
	shr	al,1			;AL=0000.0100
	rcl	ah,1			;bit 2-> carry
	rcl	al,1			;AL=0000.1002
	or	dl,al			;assemble color bits
	endm
;
SETPAL	proc	near
	mov	ax,DS			;set ES=DS
	mov	ES,ax
	mov	si,offset PalDta	;source = std form
	mov	di,offset EGAPal	;dest = ROM BIOS EGA form
	cld				;upscan
	mov	cx,16			;for 16 colors
NxClr:	mov	dl,0			;zero EGA color byte
	SCBITS				;assemble RED data
	SCBITS				;assemble GREEN data
	SCBITS				;assemble BLUE data
	mov	al,dl			;store EGA palette value
	stosb
	loop	short NxClr		;repeat for all 16 colors
;
;	send EGA palette to ROM BIOS
;
	mov	ax,1002H		;initialize palette
	mov	dx,offset EGAPal
	push	DS
	push	ds			;ES:dx is palette data
	pop	es
	int	10H
	pop	DS			;restore our DS
	ret
setpal	endp
;
;	----------------------------------------------------------------
;	DOSPAL - Restore normal DOS palette data to display
;	----------------------------------------------------------------
;
DOSPAL	proc	near
	mov	ax,1002H		;initialize palette
	mov	dx,offset bootpal
	push	ds			;ES:dx is palette data
	pop	es
	int	10H
	ret
dospal	endp
;
