;	PLPOLY.CPY - Polygon fill routine for pen plotters
;
;	(c) Copyright 1986 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  11/21/86
;	Expanded to Multi-poly form 1/4/87 Mike Riddle
;	cloned from SDPOLY.CPY 1/11/87 by Mike Riddle
;	revised SETFLP & DSYNC for more accuracy, lower-limit check 3/16/88 MR
;	Saved ES around DRAWLINE call. 11/27/88 MR
;
;	Requires DRWLINE and SETLP near procedures be provided.
;	See PLSDL.CPY for an example.
;
;	DSPOLY - draw screen filled polygons from list
;
;	ES:DI points to integer list of polys:
;	  Nr-of-Polys
;	    plist1
;	    plist2
;	    ...
;	    plistN
;	  Where each PLIST is:
;	    Count, X1,Y1, X2,Y2, ... Xn,Yn, X1,Y1
;	  Note that the first point MUST be repeated for closure.
;
DSPOLY:	entry
	mov	ax,ES			;save parmlist segment
	mov	PlSeg,ax
	mov	cx,ES:[di]		;get nr of polys
	mov	PlyCnt,cx
	add	di,2
	mov	PlBase,di		;first poly base offset
	push	CurPat			;save current line style
	push	PScale
;	first hatch fill scan
	mov	ax,FilS1		;fill style
	cmp	ax,0			;no line?
	jz	nof1
	mov	bx,FilPL1		;pattern length
	mov	FilST,ax
	mov	FilPL,bx
	mov	ax,FilSP1		;line spacing
	mov	FilSP,ax
	mov	ax,FilO1		;line space offset
	mov	FilLO,ax
	mov	ax,FilA1		;setup scan of poly
	mov	FilAng,ax		;save actual angle
	mov	FilRA2,ax		;unrotate angle1
	mov	FilRA1,1024		;initial rotate by -angle1
	sub	FilRA1,ax
	call	poly1			;do first scan of poly
;	second hatch fill scan
nof1:	mov	ax,FilS2		;fill style
	cmp	ax,0			;no line?
	jz	nof2
	mov	bx,FilPL2		;pattern length
	mov	FilST,ax
	mov	FilPL,bx
	mov	ax,FilSP2		;line spacing
	mov	FilSP,ax
	mov	ax,FilO2		;line space offset
	mov	FilLO,ax
	mov	ax,FilA2		;setup scan of poly
	mov	FilAng,ax		;save actual angle
	mov	FilRA2,ax		;unrotate angle2
	mov	FilRA1,1024		;initial rotate by -angle2
	sub	FilRA1,ax
	mov	ax,FilA1		;correct for already rotated by -angle1
	add	FilRA1,ax
	call	poly1			;do second scan of poly
;	third hatch fill scan
nof2:	mov	ax,FilS3		;fill style
	cmp	ax,0			;no line?
	jz	nof3
	mov	bx,FilPL3		;pattern length
	mov	FilST,ax
	mov	FilPL,bx
	mov	ax,FilSP3		;line spacing
	mov	FilSP,ax
	mov	ax,FilO3		;line space offset
	mov	FilLO,ax
	mov	ax,FilA3		;setup scan of poly
	mov	FilAng,ax		;save actual angle
	mov	FilRA2,ax		;unrotate angle2
	mov	FilRA1,1024		;initial rotate by -angle2
	sub	FilRA1,ax
	mov	ax,FilA2		;correct for already rotated by -angle2
	add	FilRA1,ax
	call	poly1			;do third scan of poly
;	forth hatch fill scan
nof3:	mov	ax,FilS4		;fill style
	cmp	ax,0			;no line?
	jz	nof4
	mov	bx,FilPL4		;pattern length
	mov	FilST,ax
	mov	FilPL,bx
	mov	ax,FilSP4		;line spacing
	mov	FilSP,ax
	mov	ax,FilO4		;line space offset
	mov	FilLO,ax
	mov	ax,FilA4		;setup scan of poly
	mov	FilAng,ax		;save actual angle
	mov	FilRA2,ax		;unrotate angle2
	mov	FilRA1,1024		;initial rotate by -angle2
	sub	FilRA1,ax
	mov	ax,FilA3		;correct for already rotated by -angle3
	add	FilRA1,ax
	call	poly1			;do third scan of poly
nof4:	mov	byte ptr synctl,0	;normal lstyle sync
	pop	PScale			;restore spec line style
	pop	ax
	call	setlp
	exit				;exit DSPOLY
;
poly1	proc	near
	call	SETFLP			;init fill pattern controls
;
;	scale FILSP by PLPWID for actual use
;
	mov	ax,FilSP		;filsp=filsp*plpwid
	mul	plpwid
	mov	FilSP,ax
	mov	FilSPf,0
	mov	ax,FilLO		;fillo=fillo*plpwid
	mul	plpwid
	mov	FilLO,ax
	mov	FilLOf,0
;
;	scale FILSP if line angle <>0 or 90 degrees
;
	cmp	FilAng,128		;45 degrees?
	jz	scalsp
	cmp	FilAng,384
	jnz	noscal
scalsp:	mov	ax,FilSP		;FILSP*sin(45)
	mov	bx,0B505H		;(sin(45)=0B505H/10000H)
	mul	bx
	mov	FilSP,dx
	mov	FilSPf,ax
	mov	ax,FilLO		;FILLO*sin(45)
	mul	bx
	mov	FilLO,dx
	mov	FilLOf,ax
noscal:
;
;	rotate all points in list by angle1
;
	mov	bx,PLBase		;first poly base offset
	mov	cx,PlyCnt		;# of polys to rotate
rp1:	push	cx			;save poly counter
	mov	cx,ES:[bx]		;get node count
	inc	cx			;include closure repeat
;	rotate all points in this one poly
	add	bx,2			;point to X1
rp2:	push	cx			;save node counter
	mov	si,ES:[bx]		;get x,y to SI,DI
	mov	di,ES:2[bx]
	push	bx			;save scanner
	mov	bp,FilRA1
	call	ROTPT
	pop	bx			;restore scanner
	mov	ES:[bx],si		;save rotated point
	mov	ES:2[bx],di
	add	bx,4			;bump to next point
	pop	cx			;loop for all nodes
	loop	rp2
	pop	cx			;loop for all polys
	loop	rp1
;
;	scan list and determine low and high Y values
;
	mov	ax,07FFFH		;AX = scan Y low
	mov	dx,08000H		;DX = scan Y high
	mov	di,PLBase		;first poly base offset
	mov	cx,PlyCnt		;# of polys to scan
NPHiLo:	push	cx			;save poly counter
	mov	cx,ES:[di]		;get node count
;	check this one poly for low/high Y values
	add	di,4			;point to Y1
yhilo:	mov	si,ES:[di]		;get next Y value
	cmp	si,ax			;Yn lower than YLO?
	jge	yhilo2
	mov	ax,si
yhilo2:	cmp	si,dx			;Yn higher than YHI?
	jle	nxhilo
	mov	dx,si
nxhilo:	add	di,4			;try next Y value
	loop	yhilo
;	loop for any more polys for low/high Y test
	add	di,2			;at node 1 closing point
	pop	cx			;countdown # of polys
	loop	NPHiLo			;loop if more
	mov	scanyl,ax		;save Y scan limits
	mov	scanyh,dx
;	sync start to spacing grid multiple on paper, not object
	mov	dx,ScanYL		;init scan start
	sub	ax,ax
	mov	ScanYf,ax
	cmp	dx,0			;negative?
	jge	algnlp
	neg	dx			;yes - make positive
algnlp:	sub	ax,FilSPf		;decrement until pass 0
	sbb	dx,FilSP
	jnc	algnlp
	test	ScanYL,8000H		;orig value negative?
	jnz	wasng
	add	ax,FilSPf		;no - add back bump
	adc	dx,FilSP
	sub	ScanYf,ax		;and subtract corrector
	sbb	ScanYL,dx
	jmp	short adlno		;go add offset
wasng:	add	ScanYf,ax		;was neg - add corrector
	adc	ScanYL,dx
adlno:	mov	dx,ScanYL		;add line offset
	mov	ax,ScanYf
	add	ax,FilLOf
	adc	dx,FilLO
	mov	ScanYL,dx		;save starting Y value
	mov	ScanYf,ax
;
;	for each Y value, scan for X intersection values
;	from all polys and then draw horizontal lines.
;
	jmp	short psygo		;start first scan
PSYLup:	mov	dx,ScanYL		;bump by fill spacing
	mov	ax,ScanYf
	add	ax,FilSPf
	adc	dx,FilSP
	mov	ScanYL,dx
	mov	ScanYf,ax
psygo:	cmp	dx,ScanYH		;past the limit?
	jl	dopsy
	ret				;exit poly1
dopsy:	lea	di,XIC			;x intersection values
	mov	NXIC,0			;no intersections yet
	mov	si,PlBase		;offset of 1st poly count
	mov	cx,PlyCnt		;# of polys in list
npxic:	push	cx			;save poly counter
	mov	cx,ES:[si]		;get count
	mov	NodeC,cx		;# of nodes = # lines
	add	si,2			;offset of X1
	call	BldXIC			;build table for this poly
	add	si,4			;skip closing point data
	pop	cx			;# polys left
	loop	npxic			;loop for all polys
	call	DrawXIC			;draw this scan line
	jmp	short PSYLup		;loop for next scan line
poly1	endp
;
;	scan next poly for X intersection values
;
BldXIC	proc	near
	mov	firsts,si		;offset of first line seg
	mov	bx,NodeC		;offset of last line seg
	shl	bx,1
	shl	bx,1
	sub	bx,4
	add	bx,si
	mov	lasts,bx
nxpsy:	mov	ax,ES:2[si]		;test SCANYL within line segment
	mov	dx,ES:6[si]
	cmp	ax,dx			;order low,high
	jle	ordy12
	xchg	ax,dx
ordy12:	cmp	ax,SCANYL		;lowest Y <= scanyl?
	jg	psyadv
	cmp	dx,SCANYL		;highest Y >= scanyl?
	jge	syok
psyadv:	add	si,4			;test next node line
	dec	cx
	jnz	nxpsy
	ret				;finished with this poly
syok:	mov	bx,ES:2[si]		;bx=Y1-Y2
	sub	bx,ES:6[si]
	jz	psyadv			;ignore horizontal line segments
	mov	ydiv,bx			;save Y1-Y2
	mov	ax,ES:[si]		;ax=X1-X2
	sub	ax,ES:4[si]
	jnz	psynv			;vertical?
isvert:	mov	ax,ES:[si]		;X1 is x intersection
savxis:	mov	[di],ax			;save intersection
	mov	ax,ES:2[si]		;Y1 = ScanYL?
	cmp	ax,ScanYL
	jnz	doais			;no - do add the intersection
	mov	bx,si			;point to prior line seg
bkuppp:	sub	bx,4
	cmp	bx,firsts		;need to wrap around?
	jae	prildr
	mov	bx,lasts		;yes - point to last line seg.
prildr:	mov	ax,ES:2[bx]		;ax = prior line direction
	sub	ax,ES:6[bx]
	jz	bkuppp			;if prior horizontal, backup
	mov	dx,ES:2[si]		;this line direction
	sub	dx,ES:6[si]
	xor	dx,ax			;same direction?
	jge	psyadv			;yes - don't add intersection
doais:	add	di,2
	inc	nxic			;and count it
	jmp	short psyadv		;go test next node
psynv:	mov	bx,ScanYL		;bx=YS-Y1
	sub	bx,ES:2[si]
	imul	bx			;ax=(Ys-Y1)*(X1-X2)
	idiv	ydiv			;ax=((Ys-Y1)*(X1-X2))/(Y1-Y2)
	add	ax,ES:[si]		;+X1 = x intersection value
	jmp	short savxis		;go save it
BldXIC	endp
;
;	For the current Y value, the table of X intersections,
;	XIC is complete. Draw the horizontal line segments.
;
DrawXIC	proc	near
	mov	ax,NXIC			;# intersections/2 = # line segs
	shr	ax,1
	mov	NLSEGS,ax
nxpdrw:	dec	NLSEGS			;countdown?
	jge	dopdrw
	ret				;end of this scan line
;	find the two lowest unused X values in XIC
dopdrw:	lea	bx,XIC			;find lowest X
	mov	cx,NXIC
	mov	dx,7FFFH
pdlow1:	cmp	dx,[bx]			;lowest yet?
	jl	nths1
	mov	dx,[bx]			;yes
	mov	si,bx
nths1:	add	bx,2			;advance to next point
	loop	pdlow1
	mov	PSLX,dx			;save new low X
	mov	word ptr [si],07FFFH	;and mark it used
	lea	bx,XIC			;find lowest X
	mov	cx,NXIC
	mov	dx,7FFFH
pdlow2:	cmp	dx,[bx]			;lowest yet?
	jl	nths2
	mov	dx,[bx]			;yes
	mov	si,bx
nths2:	add	bx,2			;advance to next point
	loop	pdlow2
	mov	word ptr [si],07FFFH	;and mark it used
;	set up and draw the fill line from PSLX to DX at SCANYL.
	mov	si,dx			;from point
	mov	di,SCANYL
	mov	bp,FilRA2		;rotate back
	call	ROTPT
	push	si			;save it
	push	di
	mov	si,PSLX			;to point
	mov	di,SCANYL
	call	ROTPT			;rotate back
	pop	ax			;recover from point
	pop	dx
;	now line to draw is from SI,DI to DX,AX
	mov	bx,si			;abs(x1-curx)+abs(y1-cury)
	sub	bx,curx
	jge	ok1
	neg	bx
ok1:	mov	cx,di
	sub	cx,cury
	jge	ok2
	neg	cx
ok2:	add	bx,cx
	mov	bp,bx			;bp==dist p1..curp
	mov	bx,dx			;abs(x2-curx)+abs(y2-cury)
	sub	bx,curx
	jge	ok3
	neg	bx
ok3:	mov	cx,ax
	sub	cx,cury
	jge	ok4
	neg	cx
ok4:	add	bx,cx			;bx==dist p2..curp
	cmp	bx,bp			;which is closer?
	jge	noswap
	xchg	si,dx			;swap move/draw order
	xchg	di,ax
noswap:	push	ES
	call	drwline			;draw the hatch line
	pop	ES
	jmp	NxPDrw			;go draw next segment
DrawXIC	endp
;
synctl	db	0			;0=norm,1=horiz,2=vert sync
PlyCnt	dw	0			;# polys in mpoly list
PlBase	dw	0			;first poly in list starting offset
FilAng	dw	0			;fill line bearing angle
FilRA1	dw	0			;fill scan rotation angle
FilRA2	dw	0			;fill scan unrotate angle
FilSP	dw	0			;fill scan spacing
FilSPf	dw	0			;& its fraction
FilLO	dw	0			;fill scan line offset
FilLOf	dw	0			;& its fraction
FilST	dw	0FFFFH			;fill line pattern
FilPL	dw	1			;fill pattern length
NodeC	dw	0			;node count of current poly
PlSeg	dw	0			;poly list segment
scanyl	dw	0			;scan Y lowest value
scanyf	dw	0			;& its fraction
scanyh	dw	0			;scan Y highest value
ydiv	dw	0			;Y1-Y2 for intersection divisor
firsts	dw	0			;offset of first line seg
lasts	dw	0			;offset of last line seg
PSLX	dw	0			;lowest x for segment scan
NLSEGS	dw	0			;# line segs to draw (horiz)
NXIC	dw	0			;# of scan line intersections
XIC	dw	128 dup (0)		;X intersection values table
;
;	Rotate point SI,DI by angle BP (0..1023) about center 0,0
;
;	ax,bx,cx,dx changed. SI,DI=new point, BP angle unchanged.
;
ROTPT	proc	near
	mov	bx,bp			;by angle
	mov	ax,si			;X*cos
	call	ICOS			;to dx
	push	dx
	mov	bx,bp			;by angle
	mov	ax,di			;Y*sin
	call	ISIN
	pop	ax			;newx=x*cos-y*sin
	sub	ax,dx
	push	ax			;save new x
	mov	bx,bp			;by angle
	mov	ax,si			;X*sin
	call	ISIN
	push	dx
	mov	bx,bp			;by angle
	mov	ax,di			;Y*cos
	call	ICOS
	pop	ax			;newy=x*sin+y*cos
	add	ax,dx
	mov	di,ax
	pop	si			;recover new x
	ret
rotpt	endp
;
;	SETFLP - Set fill line pattern AX of length BX penwidths
;
setflp	proc	near
	mov	ax,FilST		;flag line style for doline
	mov	CurPat,ax
	mov	ax,FilPL		;calc pattern multiple
	mul	plpwid			;(dsync will /16)
	cmp	ax,16			;minimum value is one*16
	jg	not0
	mov	ax,16
not0:	mov	FilPL,ax		;# dwg dots per pattern dot
	mov	synctl,1		;assume horiz sync
	cmp	FilAng,128		;unless 45deg<ang<135deg
	jbe	synok
	cmp	FilAng,384
	jae	synok
	mov	synctl,2		;then use vert sync
synok:	ret
setflp	endp
;
;	DSYNC - SDOT SI,DI line style sync to AX bit 15
;
DSYNC	proc	near
	cmp	synctl,1		;type of sync
	jz	synch
	ja	syncv
	ret
synch:	mov	ax,newx			;horizontal
	jz	syncom
syncv:	mov	ax,newy			;vertical
syncom:	mov	dx,ax			;filpl is 16x too big
	mov	cl,12
	shr	dx,cl
	mov	cl,4
	shl	ax,cl
	mov	cx,FilPL		;add 1/2 for rounding
	shr	cx,1
	add	ax,cx
	adc	dx,0	
	div	FilPL			;scale to pattern size
	mov	cl,al			;set bit index
	and	cl,15
	mov	ax,FilST		;the basic pattern
	rol	ax,cl			;positioned
	ret
dsync	endp
;
