;	-----------------------------------------------------------------
;	PLSDL.CPY - Basic Plot Line through moveto,drawto with SDOT model
;	-----------------------------------------------------------------
;
;	(c) Copyright 1986 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  5/25/86
;	cloned from SDSUBS.CPY
;
;	This module is the primitive code to draw patterned lines
;	by treating them as an SDOT mapping, but moving and drawing
;	based on the endpoints generated.  It lets pen plotters
;	draw patterned lines, and can provide basic support for
;	poly fill routines if needed.
;
;	use of this module requires:
;
;	Local MOVETO(si,di) and DRAWTO(si,di) near procedures, and
;	PSCALE DW n, where n is number of pixels per pattern bit.
;
savea	macro
	push	ax
	push	bx
	push	cx
	push	dx
	push	si
	push	di
	push	bp
	endm
;
resta	macro
	pop	bp
	pop	di
	pop	si
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	endm
;
sdorg	macro
	call	lorg
	endm
;
lorg	proc	near	;line origin
	savea
	mov	newx,si
	mov	newy,di
	mov	oldx,si
	mov	oldy,di
	call	MOVETO
	resta
	ret
lorg	endp
;
sdot	macro
	call	ldot
	endm
;
ldot	proc	near		;line dot
	savea
	mov	ax,upatrn
	call	DSYNC		;give fill sync a shot
	mov	bl,2		;bl = 2 pen down, = 1 pen up
	test	al,1		;pen down?
	jnz	pdn
	mov	bl,1		;no - pen up
pdn:	cmp	pstat,0		;start of line?
	jnz	nstrt
	mov	pstat,bl	;yes - save init pen status
	jmp	short ldxt	;and exit
nstrt:	cmp	pstat,bl	;same pen position?
	jz	pbump		;yes - bump pattern for next dot
	mov	pstat,bl	;mark new pen status
	cmp	bl,1		;draw or move (old was opposite)
	jz	drw
	mov	si,oldx		;move to old position
	mov	di,oldy
	call	MOVETO
	jmp	short pbump
drw:	mov	si,oldx		;draw to old position
	mov	di,oldy
	call	DRAWTO
pbump:	dec	pcount		;pattern scale countdown
	jnz	ldxt
	mov	ax,pscale	;start again on count
	mov	pcount,ax
	ror	upatrn,1	;rotate the pattern bits
ldxt:	resta
	ret
ldot	endp
;
sdend	macro
	call	lend
	endm
;
lend	proc	near		;line end
	cmp	pstat,2		;need to draw final line?
	jnz	lendx
	mov	si,oldx		;DOLINE loops position one dot past
	mov	di,oldy		;the end of the line (SDOT/SDUP?/LOOP)
	call	DRAWTO		;always end with draw
lendx:	ret
lend	endp
;
sdupc	macro
	push	newx
	pop	oldx
	inc	newx
	endm
;
sddnc	macro
	push	newx
	pop	oldx
	dec	newx
	endm
;
sdupl	macro
	push	newy
	pop	oldy
	inc	newy
	endm
;
sddnl	macro
	push	newy
	pop	oldy
	dec	newy
	endm
;
drwline	proc	near
	cmp	CurPat,0FFFFH	;unpatterned line?
	jnz	doline
	push	dx		;yes - just draw it
	push	ax
	call	MOVETO		;move to si,di
	pop	di		;draw to dx,ax
	pop	si
	call	DRAWTO
	ret
drwline	endp
;
;	--------------------------------------------------------------
;	DOLINE - Draw line from P1 to P2
;	Direction is important for non-solid line styles
;	SI,DI = x1,y1  DX,AX = x2,y2
;	UPatrn = current line pattern register
;	--------------------------------------------------------------
;
DOLINE	proc	near
	mov	pstat,0			;flag start of line
;
;	Calculate VPC, HPC, and CX = # pixels to draw
;	Set BX for desired scan routine:
;	  bit  2 = X2<=X1
;	  bit  4 = X2<X1
;	  bit  8 = Y2<=Y1
;	  bit 16 = Y2<Y1
;	  bit 32 = VPC>HPC (scan by Y)
;	  bit 64 = solid line style (optimize)
;
	xor	bx,bx			;bx = 0
	sub	dx,si			;DX = x2-x1
	jz	short x2ex1		;bx = bx + 2 if x2 <= x1
	jg	short x2gx1		;bx = bx + 0 if x2 > x1
	or	bx,4			;bx = bx + 4 if x2 < x1
	neg	dx			;DX = |dx|
x2ex1:	or	bx,2
x2gx1:	sub	ax,di			;AX = y2-y1
	jz	short y2ey1		;bx = bx + 8 if y2 <= y1
	jg	short y2gy1		;bx = bx + 0 if y2 > y1
	add	bx,16			;bx = bx + 16 if y2 < y1
	neg	ax			;AX = |ax|
y2ey1:	add	bx,8
y2gy1:	inc	ax			;AX = vert pixel count
	inc	dx			;DX = horizontal pixel count
	mov	cx,dx			;assume scan by X
	cmp	ax,dx			;vpc>hpc?
	jle	short scnbyx
	or	bx,32			;yes - scan by Y
	mov	cx,ax			;CX = # pixels counter
scnbyx:	cmp	Upatrn,-1		;solid line?
	jnz	short ntsldl
	or	bx,64			;yes - optimize
ntsldl:	push	bx			;push routine index
	push	dx			;push HPC
	push	ax			;push VPC
	SDORG				;start line at SI,DI
	pop	si			;SI = bump by
	pop	di			;DI = bump limit
	pop	bp			;go to draw routine
	jmp	DS:ldtbl[bp]
;
	align	2
ldtbl	dw	lxuyu			; 0 = scan x up, bump y up
	dw	0			; 2 = invalid
	dw	0			; 4 = invalid
	dw	lxdyu			; 6 = scan x down, bump y up
	dw	lhxu			; 8 = horiz line, scan x up
	dw	lsp			;10 = single point
	dw	0			;12 = invalid
	dw	lhxd			;14 = horiz line, scan x down
	dw	0			;16 = invalid
	dw	0			;18 = invalid
	dw	0			;20 = invalid
	dw	0			;22 = invalid
	dw	lxuyd			;24 = scan x up, bump y down
	dw	0			;26 = invalid
	dw	0			;28 = invalid
	dw	lxdyd			;30 = scan x down, bump y down
	dw	lyuxu			;32 = scan y up, bump x up
	dw	lvyu			;34 = vert line, scan y up
	dw	0			;36 = invalid
	dw	lyuxd			;38 = scan y up, bump x down
	dw	0			;40 = invalid
	dw	lsp			;42 = single point
	dw	0			;44 = invalid
	dw	0			;46 = invalid
	dw	0			;48 = invalid
	dw	0			;50 = invalid
	dw	0			;52 = invalid
	dw	0			;54 = invalid
	dw	lydxu			;56 = scan y down, bump x up
	dw	lvyd			;58 = vert line, scan y down
	dw	0			;60 = invalid
	dw	lydxd			;62 = scan y down, bump x down
; ----------------- solid line optimized versions --------------------
	dw	lsxuyu			; 0 = scan x up, bump y up
	dw	0			; 2 = invalid
	dw	0			; 4 = invalid
	dw	lsxdyu			; 6 = scan x down, bump y up
	dw	lshxu			; 8 = horiz line, scan x up
	dw	lsp			;10 = single point
	dw	0			;12 = invalid
	dw	lshxd			;14 = horiz line, scan x down
	dw	0			;16 = invalid
	dw	0			;18 = invalid
	dw	0			;20 = invalid
	dw	0			;22 = invalid
	dw	lsxuyd			;24 = scan x up, bump y down
	dw	0			;26 = invalid
	dw	0			;28 = invalid
	dw	lsxdyd			;30 = scan x down, bump y down
	dw	lsyuxu			;32 = scan y up, bump x up
	dw	lsvyu			;34 = vert line, scan y up
	dw	0			;36 = invalid
	dw	lsyuxd			;38 = scan y up, bump x down
	dw	0			;40 = invalid
	dw	lsp			;42 = single point
	dw	0			;44 = invalid
	dw	0			;46 = invalid
	dw	0			;48 = invalid
	dw	0			;50 = invalid
	dw	0			;52 = invalid
	dw	0			;54 = invalid
	dw	lsydxu			;56 = scan y down, bump x up
	dw	lsvyd			;58 = vert line, scan y down
	dw	0			;60 = invalid
	dw	lsydxd			;62 = scan y down, bump x down
;
LSP:	SDOT				;single point - change it
	SDEND				;end of line
	ret				;done
;
LHXU:	SDOT				;horizontal non-solid line
	SDUPC				;scan x up
	loop	short lhxu		;scan for all pixels
	SDEND				;end of line
	ret				;done
;
LHXD:	SDOT				;horizontal non-solid line
	SDDNC				;scan x down
	loop	short lhxd		;scan for all pixels
	SDEND				;end of line
	ret				;done
;
LVYU:	SDOT				;vertical non-solid line
	SDUPL				;scan y up
	loop	short lvyu		;scan for all pixels
	SDEND				;end of line
	ret				;done
;
LVYD:	SDOT				;vertical non-solid line
	SDDNL				;scan y down
	loop	short lvyd		;scan for all pixels
	SDEND				;end of line
	ret				;done
;
LSHXU:	SDOT				;horizontal solid line
	SDUPC				;scan x up
	loop	short lshxu		;scan for all pixels
	SDEND				;end of line
	ret				;done
;
LSHXD:	SDOT				;horizontal solid line
	SDDNC				;scan x down
	loop	short lshxd		;scan for all pixels
	SDEND				;end of line
	ret				;done
;
LSVYU:	SDOT				;vertical solid line
	SDUPL				;scan y up
	loop	short lsvyu		;scan for all pixels
	SDEND				;end of line
	ret				;done
;
LSVYD:	SDOT				;vertical solid line
	SDDNL				;scan y down
	loop	short lsvyd		;scan for all pixels
	SDEND				;end of line
	ret				;done
;
nsolid	equ	0
isolid	equ	1
byx	equ	0
byy	equ	1
;
DDL	macro	DSET,SBY,SCNBY,BMPBY
	local	slup,nobmp
	if	SBY EQ BYY
	xchg	si,di			;SI = bump by, DI = bump limit
	endif
	xor	bp,bp			;BP = bump counter
slup:
	if	DSET EQ ISolid
	SDOT				;set solid line pixel
	else
	SDOT				;set patterned line pixel
	endif
	SCNBY				;scan (SDUPC/SDDNC/SDUPL/SDDNL)
	add	bp,si			;bump = bump + bumpby
	cmp	bp,di			;bump >= bump limit?
	jb	short nobmp
	sub	bp,di			;yes: bump = bump - bump limit
	BMPBY				;bump (SDUPL/SDDNL/SDUPC/SDDNC)
nobmp:	loop	short slup		;loop for all pixels
	SDEND				;end of line
	ret				;done
	endm
;
lxuyu:	DDL	NSolid,byx,SDUPC,SDUPL	;scan x up, bump y up
lxdyu:	DDL	NSolid,byx,SDDNC,SDUPL	;scan x down, bump y up
lxuyd:	DDL	NSolid,byx,SDUPC,SDDNL	;scan x up, bump y down
lxdyd:	DDL	NSolid,byx,SDDNC,SDDNL	;scan x down, bump y down
;
lyuxu:	DDL	NSolid,byy,SDUPL,SDUPC	;scan y up, bump x up
lydxu:	DDL	NSolid,byy,SDDNL,SDUPC	;scan y down, bump x up
lyuxd:	DDL	NSolid,byy,SDUPL,SDDNC	;scan y up, bump x down
lydxd:	DDL	NSolid,byy,SDDNL,SDDNC	;scan y down, bump x down
;
lsxuyu:	DDL	ISolid,byx,SDUPC,SDUPL	;scan x up, bump y up
lsxdyu:	DDL	ISolid,byx,SDDNC,SDUPL	;scan x down, bump y up
lsxuyd:	DDL	ISolid,byx,SDUPC,SDDNL	;scan x up, bump y down
lsxdyd:	DDL	ISolid,byx,SDDNC,SDDNL	;scan x down, bump y down
;
lsyuxu:	DDL	ISolid,byy,SDUPL,SDUPC	;scan y up, bump x up
lsydxu:	DDL	ISolid,byy,SDDNL,SDUPC	;scan y down, bump x up
lsyuxd:	DDL	ISolid,byy,SDUPL,SDDNC	;scan y up, bump x down
lsydxd:	DDL	ISolid,byy,SDDNL,SDDNC	;scan y down, bump x down
;
doline	endp
;
scanx	dw	0		;frec scanner
upatrn	dw	0		;line pattern currently in use
x1	dw	0
y1	dw	0
x2	dw	0
y2	dw	0
;
newx	dw	0		;plot pattern segments
newy	dw	0
oldx	dw	0
oldy	dw	0
;
pcount	dw	0		;current pscale coutn
pstat	db	0		;current pen status
;
