;	SDCIR.CPY - SDOT Circle and Arc routines
;
;	(c) Copyright 1986 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  11/21/86
;	changed to avoid use of PSCALE 7/2/88 MR
;
circx	dw	0		;circle center x
circy	dw	0		;circle center y
cirrd	dw	0		;circle radius
strta	dw	0		;starting angle 0-1023
strtax	dw	0		;starting angle endpoint deltas
strtay	dw	0
strtq	dw	0		;starting angle quadrant 0..3
enda	dw	0		;ending angle 0-1023
endax	dw	0		;ending angle endpoint deltas
enday	dw	0
endq	dw	0		;ending angle quadrant 0..3
;
cirnx	dw	0		;circle new point to plot
cirny	dw	0
oldcx	dw	0		;prior point
oldcy	dw	0
cdelta	dw	0		;circle delta
;
GenQ	dw	0		;generate quadrant number 0..3
qxft	dw	0,-1,-1,0	;quadrant sign flags tables
qyft	dw	0,0,-1,-1
qxf	dw	0		;quadrant sign flags
qyf	dw	0
LimTst	db	0		;end limit test 0:x<=0 1:x<=endax 2:y>=enday
;
sqm	dw	1		;squish multiplier
sqd	dw	1		;squish divisor
sqh	dw	0		;1/2 sqd for round-off
sqflag	db	0		;sqm/sqd calculated flag
;
;	--------------------------------------------------------------
;	SDINIT - Calculate square pixel correction factors
;	--------------------------------------------------------------
;
SDINIT	proc	near		;calc square pixel correction to Y
	test	sqflag,1	;already calc?
	jnz	sdixt
	mov	ax,dsxmax
	inc	ax
	mov	sqd,ax
	shr	ax,1
	mov	sqh,ax
	mov	ax,dsymax
	inc	ax
	mul	dsxar
	div	dsyar
	mov	sqm,ax
	mov	sqflag,1	;flag now calculated
sdixt:	ret
sdinit	endp
;
;	--------------------------------------------------------------
;	SDOT - Draw a dot procedure
;	--------------------------------------------------------------
;
sdot	proc	near		;put SI,DI dot on screen
	sdgpm
	sdcbo
	pbump
	ret
sdot	endp
;
;	--------------------------------------------------------------
;	DSCIR - draw circle on screen (must entirely fit)
;
;	AX = center X
;	DX = center Y
;	CX = radius (horizontal)
;
;	Circle is drawn using a modified Bresenham's algorithm
;	from "Procedural Elements for Computer Graphics"
;	by David F. Rogers, McGraw-Hill 1985, pg 42-51
;	reversed to start at +radius,0 instead of 0,+radius.
;	--------------------------------------------------------------
;
DSCIR:	ENTRY
	mov	circx,ax	;save parameters
	mov	circy,dx
	mov	cirrd,cx
	mov	ES,dwseg	;use display buffer segment
	call	SDINIT		;calc squish factor
	mov	oldcx,32767	;prior not same point
	mov	LimTst,0	;all quadrants full quadrants
	mov	si,cirrd	;generate full 1st quadrant
	mov	di,0
	mov	bx,0		;quadrant 0
	call	GEN1CQ
	mov	si,cirrd	;generate full 2nd quadrant
	mov	di,0
	mov	bx,1		;quadrant 1
	call	GEN1CQ
	mov	si,cirrd	;generate full 3rd quadrant
	mov	di,0
	mov	bx,2		;quadrant 2
	call	GEN1CQ
	mov	si,cirrd	;generate full 4th quadrant
	mov	di,0
	mov	bx,3		;quadrant 3
	call	GEN1CQ
	EXIT
;
;	--------------------------------------------------------------
;	DSARC - draw arc on screen (circle must entirely fit)
;
;	AX = center X
;	DX = center Y
;	CX = radius (horizontal)
;	SI = starting angle (0-1023)
;	DI = ending angle (0-1023)
;	--------------------------------------------------------------
;
DSARC:	ENTRY
	mov	strta,si	;save parameters
	mov	enda,di
	mov	circx,ax
	mov	circy,dx
	mov	cirrd,cx
	mov	ES,dwseg	;use display buffer segment
	call	SDINIT		;calc squish factor
	mov	oldcx,32767	;prior not same point
;
;	calc starting angle endpoint deltas and quadrant
;	Save deltas for first quadrant
;
	mov	bx,strta	;radius*COS(strta)
	mov	ax,cirrd
	and	bx,00FFh	;range 0..255 (quadrant 0)
	call	ICOS
	mov	strtax,dx
	mov	bx,strta	;radius*SIN(strta)
	mov	ax,cirrd
	and	bx,00FFh	;range 0..255 (quadrant 0)
	call	ISIN
	mov	strtay,dx
	mov	ax,strta	;quadrant = angle /256
	mov	al,ah
	sub	ah,ah
	mov	strtq,ax
;
;	calc ending angle endpoint deltas and quadrant
;	Save deltas for first quadrant
;
	mov	bx,enda		;radius*COS(enda)
	and	bx,00FFh	;range 0..255 (quadrant 0)
	mov	ax,cirrd
	call	ICOS
	mov	endax,dx
	mov	bx,enda		;radius*SIN(enda)
	and	bx,00FFh	;range 0..255 (quadrant 0)
	mov	ax,cirrd
	call	ISIN
	mov	enday,dx
	mov	ax,enda		;quadrant = angle /256
	mov	al,ah
	sub	ah,ah
	mov	endq,ax
	mov	ax,enda		;end angle >= start angle?
	cmp	ax,strta
	jge	short eaok	;yes
	add	endq,4		;no - arc wraps past 0 degrees
eaok:
;
;	Draw arc quadrants
;
	mov	bx,StrtQ	;arc starting point in quadrant
	mov	si,strtax	;set start point
	mov	di,strtay
anxq:	mov	LimTst,0	;arc ends in this quadrant?
	cmp	bx,EndQ
	jnz	short anetq
	mov	ax,enda		;end test 1 or 2?
	rol	al,1
	and	al,1
	inc	al		;ax = 1 (0..127) or 2 (128..255)
	mov	LimTst,al
anetq:	push	bx		;save quadrant #
	call	GEN1CQ
	pop	bx
	cmp	bx,EndQ		;arc ended this quadrant?
	jnz	short aneq2
	EXIT
aneq2:	inc	bx		;start next quadrant
	mov	si,cirrd	;start drawing at start of quad
	mov	di,0
	jmp	short anxq	;loop and process this quadrant
;
;	--------------------------------------------------------------
;	GEN1CQ - Generate 1 Circle Quadrant
;
;	Generate a quadrant of the circle starting at SI,DI
;	in 0,0 centered unsquished coordinates, ending at the end
;	of the quadrant.
;
;	BX = 0..3 quadrant # on entry
;	SI = X starting position
;	DI = Y starting position
;
;	Quad	QXF	QYF	Angles		exchange X & Y
;	0	0	0	0..255		no
;	1	0	-1	256..511	yes
;	2	-1	-1	512..767	no
;	3	0	-1	768..1023	yes
;	--------------------------------------------------------------
;
GEN1CQ	proc	near
	and	bx,3		;range 0..3
	mov	GenQ,bx		;save quadrant #
	mov	CirNX,si	;and starting position
	mov	CirNY,di
	shl	bx,1		;lookup QXF and QYF values
	mov	ax,QXFT[bx]
	mov	QXF,ax
	mov	ax,QYFT[bx]
	mov	QYF,ax
	mov	ax,di		;initial delta = y+y-x-x+2
	add	ax,di
	sub	ax,si
	sub	ax,si
	add	ax,2
	mov	cdelta,ax
cirlp:	mov	di,cirny	;get new point to si,di
	mov	si,cirnx
	test	GenQ,1		;exchange for q=1 or 3
	jz	short noqx
	xchg	si,di
noqx:	mov	ax,di		;squish Y for square pixels
	mul	sqm
	add	ax,sqh		;half-adjust for rounding
	adc	dx,0
	div	sqd
	mov	di,ax		;set si,di = deltas from center
	xor	si,QXF		;invert signs for desired quadrant
	sub	si,QXF
	xor	di,QYF
	sub	di,QYF
	add	si,circx	;translate circle center
	add	di,circy
	cmp	si,oldcx	;same point as last dot?
	jnz	cirns
	cmp	di,oldcy
	jz	short nxtct	;yes - don't plot
cirns:	mov	oldcx,si	;save for next prior test
	mov	oldcy,di
	call	sdot		;not same - do plot it
nxtct:	cmp	LimTst,1	;type of limit test desired...
	je	ltest1
	jg	ltest2
ltest0:	cmp	cirnx,0		;cirnx<=0?
	jg	nxtcp
	ret			;yes - quadrant complete
ltest1:	mov	bp,cirny	;cirny>=enday?
	cmp	bp,enday
	jl	nxtcp
	ret			;yes - quadrant complete
ltest2:	mov	bp,cirnx	;cirnx<=endax?
	cmp	bp,endax
	jg	nxtcp
	ret			;yes - quadrant complete
nxtcp:	cmp	cdelta,0	;select move horiz, vert, or diag
	jg	hord		;must be horiz or diagonal
	jl	vord		;must be vertical or diagonal
cmd:	inc	cirny		;move diagonally to new pixel
	dec	cirnx
	mov	ax,cdelta	;new delta=delta+y+y-x-x+2
	add	ax,cirny
	add	ax,cirny
	sub	ax,cirnx
	sub	ax,cirnx
	add	ax,2
	mov	cdelta,ax
	jmp	cirlp		;loop for next pixel
vord:	mov	ax,cdelta	;test=delta+delta+x+x-1
	add	ax,ax
	add	ax,cirnx
	add	ax,cirnx
	dec	ax
	jg	cmd		;if test>0 then move diagonally
cmv:	inc	cirny		;move vertically
	mov	ax,cdelta	;new delta=delta+y+y+1
	add	ax,cirny
	add	ax,cirny
	inc	ax
	mov	cdelta,ax
	jmp	cirlp		;loop for next pixel
hord:	mov	ax,cdelta	;test=delta+delta-y-y-1
	add	ax,ax
	sub	ax,cirny
	sub	ax,cirny
	dec	ax
	jle	cmd		;if test<=0 then move diagonally
cmh:	dec	cirnx		;move horizontally
	mov	ax,cdelta	;new delta=delta-x-x+1
	sub	ax,cirnx
	sub	ax,cirnx
	inc	ax
	mov	cdelta,ax
	jmp	cirlp		;loop for next pixel
gen1cq	endp
;
