;	SDLINE.CPY - DoLine Routine for SDOT macro drivers
;
;	Copyright (C) 1990 Evolution Computing
;	All rights reserved.
;	Written by Mike Riddle, 5/19/90
;
;	Uses macros: SDGPM,SDCBO,SDCSP,SDUPC,SDDNC,SDUPL,SDDNL
;
;	SDUPC, SDDNC, SDUPL, SDDNL:
;	  must not change SI, DI, BP, CX or assume any use of them
;	  ES:BX, AX are used for byte & mask scanners
;	  DX may be changed.
;
;	--------------------------------------------------------------
;	DOLINE - Draw line from P1 to P2
;	Direction is important for non-solid line styles
;	SI,DI = x1,y1  DX,AX = x2,y2
;	UPatrn = current line pattern register
;	--------------------------------------------------------------
;
DOLINE	proc	near
	mov	ES,dwseg		;select display segment
;
;	Calculate VPC, HPC, and CX = # pixels to draw
;	Set BX for desired scan routine:
;	  bit  2 = X2<=X1
;	  bit  4 = X2<X1
;	  bit  8 = Y2<=Y1
;	  bit 16 = Y2<Y1
;	  bit 32 = VPC>HPC (scan by Y)
;	  bit 64 = solid line style (optimize)
;
	xor	bx,bx			;bx = 0
	sub	dx,si			;DX = x2-x1
	jz	short x2ex1		;bx = bx + 2 if x2 <= x1
	jg	short x2gx1		;bx = bx + 0 if x2 > x1
	or	bx,4			;bx = bx + 4 if x2 < x1
	neg	dx			;DX = |dx|
x2ex1:	or	bx,2
x2gx1:	sub	ax,di			;AX = y2-y1
	jz	short y2ey1		;bx = bx + 8 if y2 <= y1
	jg	short y2gy1		;bx = bx + 0 if y2 > y1
	add	bx,16			;bx = bx + 16 if y2 < y1
	neg	ax			;AX = |ax|
y2ey1:	add	bx,8
y2gy1:	inc	ax			;AX = vert pixel count
	inc	dx			;DX = horizontal pixel count
	mov	cx,dx			;assume scan by X
	cmp	ax,dx			;vpc>hpc?
	jle	short scnbyx
	or	bx,32			;yes - scan by Y
	mov	cx,ax			;CX = # pixels counter
scnbyx:	cmp	Upatrn,-1		;solid line?
	jnz	short ntsldl
	or	bx,64			;yes - optimize
ntsldl:	push	bx			;push routine index
	push	dx			;push HPC
	push	ax			;push VPC
	SDGPM				;get pixel masks
	SDCBO				;calculate byte offsets
	pop	si			;SI = bump by
	pop	di			;DI = bump limit
	pop	bp			;go to draw routine
	jmp	DS:ldtbl[bp]
;
	align	2
ldtbl	dw	lxuyu			; 0 = scan x up, bump y up
	dw	0			; 2 = invalid
	dw	0			; 4 = invalid
	dw	lxdyu			; 6 = scan x down, bump y up
	dw	lhxu			; 8 = horiz line, scan x up
	dw	lsp			;10 = single point
	dw	0			;12 = invalid
	dw	lhxd			;14 = horiz line, scan x down
	dw	0			;16 = invalid
	dw	0			;18 = invalid
	dw	0			;20 = invalid
	dw	0			;22 = invalid
	dw	lxuyd			;24 = scan x up, bump y down
	dw	0			;26 = invalid
	dw	0			;28 = invalid
	dw	lxdyd			;30 = scan x down, bump y down
	dw	lyuxu			;32 = scan y up, bump x up
	dw	lvyu			;34 = vert line, scan y up
	dw	0			;36 = invalid
	dw	lyuxd			;38 = scan y up, bump x down
	dw	0			;40 = invalid
	dw	lsp			;42 = single point
	dw	0			;44 = invalid
	dw	0			;46 = invalid
	dw	0			;48 = invalid
	dw	0			;50 = invalid
	dw	0			;52 = invalid
	dw	0			;54 = invalid
	dw	lydxu			;56 = scan y down, bump x up
	dw	lvyd			;58 = vert line, scan y down
	dw	0			;60 = invalid
	dw	lydxd			;62 = scan y down, bump x down
; ----------------- solid line optimized versions --------------------
	dw	lsxuyu			; 0 = scan x up, bump y up
	dw	0			; 2 = invalid
	dw	0			; 4 = invalid
	dw	lsxdyu			; 6 = scan x down, bump y up
	dw	lshxu			; 8 = horiz line, scan x up
	dw	lsp			;10 = single point
	dw	0			;12 = invalid
	dw	lshxd			;14 = horiz line, scan x down
	dw	0			;16 = invalid
	dw	0			;18 = invalid
	dw	0			;20 = invalid
	dw	0			;22 = invalid
	dw	lsxuyd			;24 = scan x up, bump y down
	dw	0			;26 = invalid
	dw	0			;28 = invalid
	dw	lsxdyd			;30 = scan x down, bump y down
	dw	lsyuxu			;32 = scan y up, bump x up
	dw	lsvyu			;34 = vert line, scan y up
	dw	0			;36 = invalid
	dw	lsyuxd			;38 = scan y up, bump x down
	dw	0			;40 = invalid
	dw	lsp			;42 = single point
	dw	0			;44 = invalid
	dw	0			;46 = invalid
	dw	0			;48 = invalid
	dw	0			;50 = invalid
	dw	0			;52 = invalid
	dw	0			;54 = invalid
	dw	lsydxu			;56 = scan y down, bump x up
	dw	lsvyd			;58 = vert line, scan y down
	dw	0			;60 = invalid
	dw	lsydxd			;62 = scan y down, bump x down
;
LSP:	SDCSP				;single point - change it
	ret				;done
;
LHXU:	PBUMP				;horizontal non-solid line
	SDUPC				;scan x up
	loop	short lhxu		;scan for all pixels
	UNPBMP				;don't double-bump chained ends
	ret				;done
;
LHXD:	PBUMP				;horizontal non-solid line
	SDDNC				;scan x down
	loop	short lhxd		;scan for all pixels
	UNPBMP				;don't double-bump chained ends
	ret				;done
;
LVYU:	PBUMP				;vertical non-solid line
	SDUPL				;scan y up
	loop	short lvyu		;scan for all pixels
	UNPBMP				;don't double-bump chained ends
	ret				;done
;
LVYD:	PBUMP				;vertical non-solid line
	SDDNL				;scan y down
	loop	short lvyd		;scan for all pixels
	UNPBMP				;don't double-bump chained ends
	ret				;done
;
LSHXU:	SDCSP				;horizontal solid line
	SDUPC				;scan x up
	loop	short lshxu		;scan for all pixels
	ret				;done
;
LSHXD:	SDCSP				;horizontal solid line
	SDDNC				;scan x down
	loop	short lshxd		;scan for all pixels
	ret				;done
;
LSVYU:	SDCSP				;vertical solid line
	SDUPL				;scan y up
	loop	short lsvyu		;scan for all pixels
	ret				;done
;
LSVYD:	SDCSP				;vertical solid line
	SDDNL				;scan y down
	loop	short lsvyd		;scan for all pixels
	ret				;done
;
nsolid	equ	0
isolid	equ	1
byx	equ	0
byy	equ	1
;
DDL	macro	DSET,SBY,SCNBY,BMPBY
	local	slup,nobmp
	if	SBY EQ BYY
	xchg	si,di			;SI = bump by, DI = bump limit
	endif
	xor	bp,bp			;BP = bump counter
slup:
	if	DSET EQ ISolid
	SDCSP				;set solid line pixel
	else
	PBUMP				;set patterned line pixel
	endif
	SCNBY				;scan (SDUPC/SDDNC/SDUPL/SDDNL)
	add	bp,si			;bump = bump + bumpby
	cmp	bp,di			;bump >= bump limit?
	jb	short nobmp
	sub	bp,di			;yes: bump = bump - bump limit
	BMPBY				;bump (SDUPL/SDDNL/SDUPC/SDDNC)
nobmp:	loop	short slup		;loop for all pixels
	if	DSET EQ NSolid
	UNPBMP				;don't double-bump chained ends
	endif
	ret				;done
	endm
;
lxuyu:	DDL	NSolid,byx,SDUPC,SDUPL	;scan x up, bump y up
lxdyu:	DDL	NSolid,byx,SDDNC,SDUPL	;scan x down, bump y up
lxuyd:	DDL	NSolid,byx,SDUPC,SDDNL	;scan x up, bump y down
lxdyd:	DDL	NSolid,byx,SDDNC,SDDNL	;scan x down, bump y down
;
lyuxu:	DDL	NSolid,byy,SDUPL,SDUPC	;scan y up, bump x up
lydxu:	DDL	NSolid,byy,SDDNL,SDUPC	;scan y down, bump x up
lyuxd:	DDL	NSolid,byy,SDUPL,SDDNC	;scan y up, bump x down
lydxd:	DDL	NSolid,byy,SDDNL,SDDNC	;scan y down, bump x down
;
lsxuyu:	DDL	ISolid,byx,SDUPC,SDUPL	;scan x up, bump y up
lsxdyu:	DDL	ISolid,byx,SDDNC,SDUPL	;scan x down, bump y up
lsxuyd:	DDL	ISolid,byx,SDUPC,SDDNL	;scan x up, bump y down
lsxdyd:	DDL	ISolid,byx,SDDNC,SDDNL	;scan x down, bump y down
;
lsyuxu:	DDL	ISolid,byy,SDUPL,SDUPC	;scan y up, bump x up
lsydxu:	DDL	ISolid,byy,SDDNL,SDUPC	;scan y down, bump x up
lsyuxd:	DDL	ISolid,byy,SDUPL,SDDNC	;scan y up, bump x down
lsydxd:	DDL	ISolid,byy,SDDNL,SDDNC	;scan y down, bump x down
;
doline	endp
;
