;	SDMEGA - Set Dot Macros for EGA-like drivers
;
;	(c) 1986-1990 Evolution Computing
;	All rights reserved.
;
SDGPM	macro				;get pixel modification masks
	mov	bx,si			;x mod 8
	and	bx,7
	mov	AH,pmasks[bx]		;mask for selected pixel
	endm
;
SDCBO	macro				;calculate byte offset to change
	mov	bx,dsymax		;invert y value (0 at bottom)
	sub	bx,di
	shl	bx,1			;y*80=y*16+y*64 (16 clocks total)
	shl	bx,1
	shl	bx,1
	shl	bx,1
	mov	bp,bx
	shl	bx,1
	shl	bx,1
	add	bx,bp
	mov	bp,si			;work copy of X
	shr	bp,1			;+ x/8
	shr	bp,1
	shr	bp,1
	add	bx,bp			;ES:[bx] = byte to change
	endm
;
SDCSP	macro				;change selected pixel
	SetReg	3CEH,8,AH		;set pixel mask
	or	byte ptr ES:[bx],0FFH	;update pixel from S/R or invert it
	endm
;
SDUPL	macro				;move offset up one line
	sub	bx,50H			;bump line offset
	endm
;
SDDNL	macro				;move offset down one line
	add	bx,50H			;bump line offset
	endm
;
SDUPC	macro
	ror	ah,1			;shift pixel mask right
	adc	bx,0			;if wrap, bump byte offset
	endm
;
SDDNC	macro
	rol	ah,1			;shift pixel mask left
	sbb	bx,0			;if wrap, decrement byte offset
	endm
;
;	CCHRO - calculate character offset to BX; DX=line, AX=column
;
CCHRO	macro				;calculate char offset
	call	cchros
	endm
;
cchros	proc	near
	mov	bx,dx			;line number
	shl	bx,1			;word index
	add	bx,ltblptr		;lookup line offset
	mov	bx,[bx]
	add	bx,ax			;+cposn
	cmp	tccw,2			;*2?
	jnz	notx2
	add	bx,ax			;yes
notx2:	SetReg	3CEH,8,0FFH		;enable all pixels to be changed
	SetReg	3CEH,1,0H		;disable S/R (direct data write)
	SetReg	3CEH,3,0H		;write normal data
	ret
cchros	endp
;
;	GCHR - generate character from DS:[SI] cgen table, ES:BX=chr cell
;	BP= inversion mask 0 or -1; CX = NRAST value for loop; DX=tccw
;
GCHR	macro				;generate character
	local	gc1,gc2,gcxt
	cmp	dx,2			;word or byte?
	jz	short gc2
gc1:	lodsb				;get raster line
	xor	ax,bp			;invert if needed
	mov	ES:[bx],al		;set it into display
	sddnl				;move down to next raster
	loop	short gc1		;loop for entire character
	jmp	short gcxt
gc2:	lodsw				;get raster line
	xor	ax,bp			;invert if needed
	xchg	al,ah			;byte swap
	mov	ES:[bx],ax		;set it into display
	sddnl				;move down to next raster
	loop	short gc2		;loop for entire character
gcxt:
	endm
;
;	INVCR - invert character raster line ES:[BX], CX characters
;
INVCR	macro
	call	invcrs
	endm
;
invcrs	proc	near
invlup:	cmp	tccw,2			;word or byte?
	jz	inv2
	not	byte ptr ES:[bx]	;invert character raster
	jmp	short invc
inv2:	not	word ptr ES:[bx]	;invert character raster
invc:	add	bx,tccw			;next character
	loop	invlup			;for all char posns
	ret
invcrs	endp
;
;	CLRCR - clear character raster line ES:[BX],CX characters
;
CLRCR	macro
	call	clrcrs
	endm
;
clrcrs	proc	near
	sub	ax,ax			;clear to value
clrlup:	cmp	tccw,2			;word or byte?
	jz	clr2
	mov	ES:[bx],al		;clear chr raster
	jmp	short clrc
clr2:	mov	ES:[bx],ax		;clear chr raster
clrc:	add	bx,tccw			;bump chr cell
	loop	clrlup			;for all char positions
	ret
clrcrs	endp
;
;	-------------------------------------------------------------------
;
tccw	dw	1			;text character cell width in bytes
nrast	dw	12			;raster lines per character
;
CGENR	dw	402h			;character generator resource #
LTBLPTR	dw	offset ltbls		;line offset table
;
SDFHL	macro
	call	EGAFHL
	endm
;
EGAFHL	proc	near
	mov	si,scanyl		;select current lines' pattern
	and	si,07H
	mov	ah,FilPat[si]
	mov	si,bp			;x column position
	and	si,7			;mod 8 = left byte pixel mask index
	sub	cx,8			;remove left byte pixels from count
	add	cx,si
	jge	ntsngl			;more than one byte changes?
	mov	dx,3CEH			;select pixel mask register
	mov	al,8
	out	dx,al
	inc	dx
	mov	al,lbpmt[si]		;set left byte pixel mask
	and	al,ah			;(pattern mask)
	lea	si,rbpmt+8		;less unchanged right pixels
	add	si,cx	
	and	al,[si]
	out	dx,al			;send it
	or	byte ptr ES:[bx],0FFH	;update pixel from S/R or invert it
	ret
ntsngl:	mov	di,cx			;right byte pixel mask index
	and	di,7
	shr	cx,1			;remove right byte pixels from count
	shr	cx,1
	shr	cx,1
	mov	dx,3CEH			;select pixel mask register
	mov	al,8
	out	dx,al
	inc	dx
	mov	al,lbpmt[si]		;set left byte pixel mask
	and	al,ah			;(pattern mask)
	out	dx,al			;send it
	or	byte ptr ES:[bx],0FFH	;update pixel from S/R or invert it
	inc	bx			;bump byte to change offset
	jcxz	chgrb
	mov	al,ah			;set middle bytes pixel mask
	out	dx,al
chgmb:	or	byte ptr ES:[bx],0FFH	;update pixel from S/R or invert it
	inc	bx			;bump byte to change offset
	loop	chgmb			;loop for all middle bytes
chgrb:	mov	al,rbpmt[di]		;set right byte pixel mask
	and	al,ah			;(pattern mask)
	out	dx,al
	or	byte ptr ES:[bx],0FFH	;update pixel from S/R or invert it
	ret
egafhl	endp
;
lbpmt	db	0FFH,07FH,03FH,01FH,00FH,007H,003H,001H
rbpmt	db	000H,080H,0C0H,0E0H,0F0H,0F8H,0FCH,0FEH
;
LSScl	equ	2			;Line style scale factor
	include SDSUBS4.CPY
	include	DSBLN2.CPY
	include	VTEXT.CPY
;
