;	SDPOLY.CPY - Polygon fill routine for SDOT drivers
;
;	(c) Copyright 1986 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  11/21/86
;	Expanded to Multi-poly form 1/4/87 Mike Riddle
;
;	DSPOLY - draw screen filled polygons from list
;
;	ES:DI points to integer list of polys:
;
;	  Nr-of-Polys
;	    plist1
;	    plist2
;	    ...
;	    plistN
;
;	  Where each PLIST is:
;
;	    Count, X1,Y1, X2,Y2, ... Xn,Yn, X1,Y1
;
;	  Note that the first point MUST be repeated for closure.
;
DSPOLY:	entry
	mov	ax,ES			;save parmlist segment
	mov	PlSeg,ax
	mov	cx,ES:[di]		;get nr of polys
	mov	PlyCnt,cx
	add	di,2
	mov	PlBase,di		;first poly base offset
;
;	scan list and determine low and high Y values
;
	mov	ax,0FFFFH		;AX = scan Y low
	mov	dx,0			;DX = scan Y high
	mov	cx,PlyCnt		;# of polys to scan
NPHiLo:	push	cx			;save poly counter
	mov	cx,ES:[di]		;get node count
;	check this one poly for low/high Y values
	add	di,4			;point to Y1
yhilo:	mov	si,ES:[di]		;get next Y value
	cmp	si,ax			;Yn lower than YLO?
	jae	yhilo2
	mov	ax,si
yhilo2:	cmp	si,dx			;Yn higher than YHI?
	jbe	nxhilo
	mov	dx,si
nxhilo:	add	di,4			;try next Y value
	loop	yhilo
;	loop for any more polys for low/high Y test
	add	di,2			;at node 1 closing point
	pop	cx			;countdown # of polys
	loop	NPHiLo			;loop if more
	mov	scanyl,ax		;save Y scan limits
	mov	scanyh,dx
;
;	for each Y value, scan for X intersection values
;	from all polys and then draw horizontal lines.
;
	dec	ScanYL			;init scan loop
PSYLup:	inc	ScanYl			;advance Y scan line
	mov	ax,ScanYl		;new scan Y value
	cmp	ax,ScanYH		;past the limit?
	jbe	dopsy
	exit				;exit DSPOLY
dopsy:	lea	di,XIC			;x intersection values
	mov	NXIC,0			;no intersections yet
	mov	si,PlBase		;offset of 1st poly count
	mov	cx,PlyCnt		;# of polys in list
npxic:	push	cx			;save poly counter
	mov	cx,ES:[si]		;get count
	mov	NodeC,cx		;# of nodes = # lines
	add	si,2			;offset of X1
	call	BldXIC			;build table for this poly
	add	si,4			;skip closing point data
	pop	cx			;# polys left
	loop	npxic			;loop for all polys
	call	DrawXIC			;draw this scan line
	jmp	short PSYLup		;loop for next scan line
;
;	scan next poly for X intersection values
;
BldXIC	proc	near
	mov	firsts,si		;offset of first line seg
	mov	bx,NodeC		;offset of last line seg
	shl	bx,1
	shl	bx,1
	sub	bx,4
	add	bx,si
	mov	lasts,bx
nxpsy:	mov	ax,ES:2[si]		;test SCANYL within line segment
	mov	dx,ES:6[si]
	cmp	ax,dx			;order low,high
	jbe	ordy12
	xchg	ax,dx
ordy12:	cmp	ax,SCANYL		;lowest Y <= scanyl?
	ja	psyadv
	cmp	dx,SCANYL		;highest Y >= scanyl?
	jae	syok
psyadv:	add	si,4			;test next node line
	dec	cx
	jnz	nxpsy
	ret				;finished with this poly
syok:	mov	bx,ES:2[si]		;bx=Y1-Y2
	sub	bx,ES:6[si]
	jz	psyadv			;ignore horizontal line segments
	mov	ydiv,bx			;save Y1-Y2
	mov	ax,ES:[si]		;ax=X1-X2
	sub	ax,ES:4[si]
	jnz	psynv			;vertical?
isvert:	mov	ax,ES:[si]		;X1 is x intersection
savxis:	mov	[di],ax			;save intersection
	mov	ax,ES:2[si]		;Y1 = ScanYL?
	cmp	ax,ScanYL
	jnz	doais			;no - do add the intersection
	mov	bx,si			;point to prior line seg
bkuppp:	sub	bx,4
	cmp	bx,firsts		;need to wrap around?
	jae	prildr
	mov	bx,lasts		;yes - point to last line seg.
prildr:	mov	ax,ES:2[bx]		;ax = prior line direction
	sub	ax,ES:6[bx]
	jz	bkuppp			;if prior horizontal, backup
	mov	dx,ES:2[si]		;this line direction
	sub	dx,ES:6[si]
	xor	dx,ax			;same direction?
	jge	psyadv			;yes - don't add intersection
doais:	add	di,2
	inc	nxic			;and count it
	jmp	short psyadv		;go test next node
psynv:	mov	bx,ScanYL		;bx=YS-Y1
	sub	bx,ES:2[si]
	imul	bx			;ax=(Ys-Y1)*(X1-X2)
	idiv	ydiv			;ax=((Ys-Y1)*(X1-X2))/(Y1-Y2)
	add	ax,ES:[si]		;+X1 = x intersection value
	jmp	short savxis		;go save it
BldXIC	endp
;
;	For the current Y value, the table of X intersections,
;	XIC is complete. Draw the horizontal line segments.
;
DrawXIC	proc	near
	mov	ax,NXIC			;# intersections/2 = # line segs
	shr	ax,1
	mov	NLSEGS,ax
	mov	PSLX,0			;init low x scan
nxpdrw:	dec	NLSEGS			;countdown?
	jge	dopdrw
	ret				;end of this scan line
;	find the two lowest unused X values in XIC
dopdrw:	lea	bx,XIC			;find lowest X
	mov	cx,NXIC
	mov	ax,PSLX
	mov	dx,7FFFH
pdlow1:	cmp	ax,[bx]			;PSLX<=sample?
	jg	nths1
	cmp	dx,[bx]			;lowest above PSLX?
	jl	nths1
	mov	dx,[bx]			;this is best yet
	mov	si,bx
nths1:	add	bx,2			;advance to next point
	loop	pdlow1
	mov	PSLX,dx			;save new low X
	mov	word ptr [si],-1	;and mark it used
	lea	bx,XIC			;find lowest X
	mov	cx,NXIC
	mov	ax,PSLX
	mov	dx,7FFFH
pdlow2:	cmp	ax,[bx]			;PSLX<=sample?
	jg	nths2
	cmp	dx,[bx]			;lowest above PSLX?
	jl	nths2
	mov	dx,[bx]			;this is best yet
	mov	si,bx
nths2:	add	bx,2			;advance to next point
	loop	pdlow2
	mov	word ptr [si],-1	;and mark it used
;	set up and draw the fast horiz line from PSLX to DX at SCANYL.
	mov	si,PSLX			;from PSLX to dx
	mov	PSNX,si			;save start scanner
	mov	PSLX,dx			;(update next PSLX)
	mov	cx,dx			;pixel count
	sub	cx,si
	inc	cx
	mov	di,ScanYL		;at scanyl
	push	es			;draw segment switch
	mov	ES,dwseg
	push	cx			;fast horizontal line
	sdgpm
	sdcbo
	pop	cx
	mov	bp,PSNX			;x scanner for SDUPC/SDFHL
	SDFHL				;use fast horiz line routine
	pop	es			;restore poly node list seg
	jmp	NxPDrw			;go draw next segment
DrawXIC	endp
;
PlyCnt	dw	0			;# polys in mpoly list
PlBase	dw	0			;first poly in list starting offset
NodeC	dw	0			;node count of current poly
PlSeg	dw	0			;poly list segment
scanyl	dw	0			;scan Y lowest value
scanyh	dw	0			;scan Y highest value
ydiv	dw	0			;Y1-Y2 for intersection divisor
firsts	dw	0			;offset of first line seg
lasts	dw	0			;offset of last line seg
PSLX	dw	0			;lowest x for segment scan
PSNX	dw	0			;next lowest x for segment scan
NLSEGS	dw	0			;# line segs to draw (horiz)
NXIC	dw	0			;# of scan line intersections
XIC	dw	128 dup (0)		;X intersection values table
;
