;	VGAPAL.CPY - VGA driver color palette functions
;
;	Copyright 1990 Evolution Computing
;	All rights reserved.
;	Written by Mike Riddle 9-4-90
;
;	----------------------------------------------------------------
;	The palette data is 48 bytes: 16 3-byte sets of Red, Green,
;	then Blue values for each of the 16 user colors 0..15.
;	Each value is stored in a "standard" form: 0..255 with 255
;	representing the brightest level.
;	----------------------------------------------------------------
;
PAL	macro	pname
pname	db	000H, 000H, 000H	;black
	db	000H, 0F8H, 000H	;lime green
	db	0E8H, 000H, 038H	;red
	db	000H, 000H, 0F8H	;blue
	db	0F8H, 0E8H, 000H	;YELLOW
	db	000H, 0C8H, 0D8H	;CYAN
	db	0F8H, 000H, 0F8H	;MAGENTA
	db	0A8H, 000H, 0E8H	;PURPLE
	db	0F8H, 048H, 000H	;ORANGE
	db	068H, 088H, 0F8H	;LIGHT BLUE
	db	0F8H, 098H, 058H	;SAND
	db	0A8H, 048H, 000H	;BROWN
	db	088H, 088H, 038H	;OLIVE GREEN
	db	000H, 068H, 038H	;FOREST GREEN
	db	080H, 080H, 080H	;GREY
	db	0FFH, 0FFH, 0FFH	;WHITE
	endm
;
	PAL	PalDta			;user-changeable
	PAL	StdDta			;Standard restore copy
;
;	----------------------------------------------------------------
;
;	VGA/EGA palette values are:  00rgbRGB bits
;	where lc is dim, uc is bright
;
;	dk grn 010 med grn 002 brt grn 012 (green phosphor is dayglo)
;	dk red 020 med red 004 brt red 024 (red phosphor is orangish)
;	dk blu 008 med blu 001 brt blu 009
;
;	We will program the VGA DAC for color changes, so we
;	set up the EGA-compatible palette to a "pass-through"
;	state:
;
EGAPal	db	0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0
;
bootpal	db	0,1,2,3,4,5,6,7,38H,39H,3AH,3BH,3CH,3DH,3EH,3FH,0
;
;	----------------------------------------------------------------
;	DSRPAL - READ 48-byte color palette from driver
;	ES:DI -> palette data destination
;	Carry bit Set = standard set, clear = user set
;	----------------------------------------------------------------
;
DSRPAL:	ENTRY
	mov	si,offset PalDta	;copy changeable palette data
	jnc	short rdpal
	mov	si,offset StdDta	;copy standard palette data
rdpal:	cld
	mov	cx,24			;24 words
	rep	movsw			;do it
	EXIT
;
;	----------------------------------------------------------------
;	DSWPAL - WRITE 48-byte color palette to driver
;	ES:DI -> palette data source
;	----------------------------------------------------------------
;
DSWPAL:	ENTRY
	mov	si,di			;setup source
	mov	di,offset PalDta	;our data is destination
	push	DS			;save for our DS restore
	push	DS			;swap DS & ES
	push	ES
	pop	DS
	pop	ES
	cld				;upscan
	mov	cx,24			;24 words
	rep	movsw			;copy data
	pop	DS			;restore our DS
	call	SetPal
	EXIT
;
;	----------------------------------------------------------------
;	SETPAL - Setup local palette data to display
;	----------------------------------------------------------------
;
SETPAL	proc	near
;
;	send EGA palette to ROM BIOS for pass-through to DAC
;
	mov	ax,1002H		;initialize palette
	mov	dx,offset EGAPal
	push	DS
	push	ds			;ES:dx is palette data
	pop	es
	int	10H
	pop	DS			;restore our DS
;
;	range DAC data from 0..255 to 0..63
;
	mov	si,offset PalDta	;DAC data list
	mov	cx,48			;process 48 bytes
range1:	mov	al,[si]			;change 87654321 to 00876543
	shr	al,1
	shr	al,1
	mov	[si],al
	inc	si
	loop	short range1
;
;	send DAC data to VGA DAC registers
;
	mov	ax,DS			;ES:DX -> PalDta
	mov	ES,ax
	mov	dx,offset PalDta
	mov	cx,16			;change 16 colors
	mov	bx,0			;starting with color # 0
	mov	ax,1012h		;write to DAC
	int	10h			;do it
;
;	range DAC data back from 0..63 to 0..255
;
	mov	si,offset PalDta	;DAC data list
	mov	cx,48			;process 48 bytes
range2:	mov	al,[si]			;change 00876543 to 87654300
	shl	al,1
	shl	al,1
	mov	[si],al
	inc	si
	loop	short range2
;
	ret				;done
setpal	endp
;
;	----------------------------------------------------------------
;	DOSPAL - Restore normal DOS palette data to display
;	----------------------------------------------------------------
;
DOSPAL	proc	near
	mov	ax,1002H		;initialize palette
	mov	dx,offset bootpal
	push	ds			;ES:dx is palette data
	pop	es
	int	10H
	ret
dospal	endp
;
