;	CONTROLS.ASM - FastCAD(R) External Procedure Module
;	for CONTROLS entities (Includes Scott's work to 7/20/89)
;
;	(c) Copyright 1987 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle 10/21/87
;
	name	CTLS
;
	include	XPSTART.CPY
;
;	---------------------------------------------------------------
;	General-purpose macros and variables
;	---------------------------------------------------------------
;
DVALID	macro				;check for valid data from
	local	OK			;  XP input services
	cmp	dl,0
	jz	short OK
	EXIT
OK:
	endm
;
;	---------------------------------------------------------------
;	XCMD - External Process Command - command # BX
;
;	commands 0..10 are preassigned signals
;	user commands start with command #11
;	---------------------------------------------------------------
;
XCMD:	ENTRY
	shl	bx,1			;make word index
	add	bx,offset cmdtbl
	mov	bx,[bx]			;get command offset
	jmp	bx			;go to command handler
cmdtbl	dw	offset nulcmd		;0 = Startup ExtProc
	dw	offset nulcmd		;1 = Ending use of ExtProc
	dw	offset nulcmd		;2 = Drawing renamed
	dw	offset nulcmd		;3 = Drawing Just Loaded
	dw	offset nulcmd		;4 = Drawing about to be saved
	dw	offset nulcmd		;5 = Drawing REVERTed to prior save
	dw	offset nulcmd		;6 = before getting next command
	dw	offset nulcmd		;7 = regen completed - repath
	dw	offset nulcmd		;8 = part inserted
	dw	offset nulcmd		;9 = unused
	dw	offset nulcmd		;10 = unused
	dw	offset AqBtn		;11 = BUTTON command
	dw	offset AqRbtn		;12 = RBTN command
;	other commands offsets go here
nulcmd:	EXIT
;
;	---------------------------------------------------------------
;	XENTITY - External Process Entity Support
;	---------------------------------------------------------------
;
XENT:	ENTRY
	mov	bx,ax			;desired op # in AX
	shl	bx,1			;make word index
	add	bx,offset eftbl
	mov	bx,[bx]			;get entity function offset
	jmp	bx			;go to function  handler
eftbl	dw	offset xdraw		;0 = draw entity
	dw	offset xpick		;1 = pick test entity
	dw	offset xmove		;2 = move entity
	dw	offset xmove		;3 = stretch entity
	dw	offset xlist		;4 = list entity
	dw	offset xxchk		;5 = check entity extents
	dw	offset nullop		;6 = attach to entity
	dw	offset xact		;7 = actuate entity
nullop:	finit
	clc
	EXIT
;
;	---------------------------------------------------------------
;	XCSTG - Command Name String
;
;	Each command word is UPPERCASE, space delimited.
;	---------------------------------------------------------------
;
XPEXID	equ	0F000H			;EXID for this module
;
XCSTG	db	'BTN RBTN '
	db	0			;end of command word table
;
;	---------------------------------------------------------------
;	DOS Identification Message
;	---------------------------------------------------------------
;
ident	db	'CONTROLS - Control Panel Entities.',13,10
	db	'FastCAD(R) External Procedure Module - version '
	db	dvrlvl,'.00',13,10
	db	'BTN  - single button',13,10
	db	'RBTN - radio button',13,10
	db	'(C) Copyright 1987 Evolution Computing',13,10,'$'
;
;	---------------------------------------------------------------
;	AQBTN - Acquire Button Entities
;	---------------------------------------------------------------
;
button	struc				;18 bytes total
	dw	0,0,0			;standard stuff (see ESTRUC.CPY)
btnxid	dw	0			;controls EXID
betype	db	1			;button = control entity # 1
btnval	db	0			;button on/off status
btnx	dd	0.0			;button center X
btny	dd	0.0			;button center Y
button	ends
;
AQBTN:
;
;	initialize button entity record
;
	SVC	CINFO			;init common info
	les	di,ENTRP		;ES:DI points to ENTREC
	mov	ES:elayer[di],dl	;set layer
	mov	ES:ecolor[di],al	;set color
	mov	ES:estyle[di],bl	;set line style
	mov	ES:erlen[di],18		;length of button entity record
	mov	ES:etype[di],16		;etype=EXT entity
	mov	ES:exid[di],XPEXID	;our module ID
	mov	ES:betype[di],1		;local entity type 1 = button
	mov	ES:btnval[di],0		;initially off
;
;	get button position
;
	mov	ax,DS			;prompt for button position
	mov	ES,ax
	lea	si,bposn
	mov	ax,0			;snap enabled
	SVC	GPOINT			;get center position
	DVALID
	SVC	SUNADD			;set undo = unadd
	SVC	CINFO			;init common info as of now
	les	di,ENTRP		;ES:DI points to ENTREC
	mov	ES:elayer[di],dl	;set layer
	mov	ES:ecolor[di],al	;set color
	mov	ES:estyle[di],bl	;set line style
	fstp	ES:btny[di]		;store button center point
	fstp	ES:btnx[di]
	SVC	APND			;append entity
	SVC	DRAW			;then draw it on-screen
	EXIT				;done
;
bposn	db	'Button position: ',0
bsizex	dd	0.1875			;1/2 button size
bsizey	dd	0.125
onsx	dd	0.09375			;shrink to on status interior
onsy	dd	0.0625			;black box
;
;	---------------------------------------------------------------
;	AQRBTN - Acquire Radio Button Entities
;	---------------------------------------------------------------
;
rbtn	struc				;28 bytes total
	db	10 dup (0)		;common stuff with button (betype=2)
rbnrb	db	0			;total buttons 
rbx	db	0			;spare
rbx1	dd	0.0			;center of 1st button
rby1	dd	0.0
rbx2	dd	0.0			;center of last button
rby2	dd	0.0
rbtn	ends
;
rbp1	db	'first button: ',0
rbp2	db	'last button: ',0
Nbprmpt	db	'number of buttons: ',0
;
AQRBTN:	
;
;	get first button position
;
	mov	ax,DS			;prompt for 1st button center
	mov	ES,ax
	mov	si,offset rbp1
	mov	ax,0			;snap enabled
	SVC	GPOINT			;get center position
	DVALID				;quit if data invalid
	les	di,ENTRP		;access ENTREC
	fst	ES:rby1[di]		;store point (leave on 8087)
	fxch	st(1)
	fst	ES:rbx1[di]
	fxch	st(1)
;
;	get last button position
;
	mov	ax,DS			;prompt for last button center
	mov	ES,ax
	mov	si,offset rbp2
	SVC	GLENDP			;get center position
	DVALID				;quit if data invalid
	les	di,ENTRP		;access ENTREC
	fstp	ES:rby2[di]		;store point (leave on 8087)
	fstp	ES:rbx2[di]
;
;	get number of buttons
;
	mov	ax,DS			;ask how many buttons
	mov	ES,ax
	mov	si,offset Nbprmpt	;prompt
	SVC	GNUM
	DVALID
	les	di,ENTRP		;access ENTREC
	mov	ES:rbnrb[di],al		;save it
;
;	initialize button entity record
;
	SVC	SUNADD			;set undo = unadd
	SVC	CINFO			;init common info
	les	di,ENTRP		;ES:DI points to ENTREC
	mov	ES:elayer[di],dl	;set layer
	mov	ES:ecolor[di],al	;set color
	mov	ES:estyle[di],bl	;set line style
	mov	ES:erlen[di],28		;length of button entity record
	mov	ES:etype[di],16		;etype=EXT entity
	mov	ES:exid[di],XPEXID	;our module ID
	mov	ES:betype[di],2		;local entity type 2 = radio button
	mov	ES:btnval[di],1 	;initially 1st button is on
	SVC	APND
	SVC	DRAW
	EXIT
;
;	---------------------------------------------------------------
;	XDRAW - draw our entities
;	---------------------------------------------------------------
;
	align	4
CurX	dd	0.0
CurY	dd	0.0
BumpX	dd	0.0
BumpY	dd	0.0
Ngaps	dw	0
OnBtn	db	0			;which button is on
CBCtr	db	0			;current button counter
;
XDRAW:	les	di,ENTRP		;get control type #
	mov	bl,ES:betype[di]
	mov	bh,0
	shl	bx,1			;*2 for word table
	jmp	drwtbl[bx]		;lookup address
drwtbl	dw	0			;betype=0
	dw	offset drw1		;draw single button
	dw	offset drw2		;draw radio buttons

;	Drw1: draw a single button entity
;
Drw1:	mov	ax,0			;set solid fill
	CALLDP	DSFPAT
	les	di,ENTRP		;ES:DI points to ENTREC
	fld	ES:btnx[di]		;get button center
	fld	ES:btny[di]
	call	BTNPTS			;define button corners
	les	di,ENTRP		;button on or off?
	mov	al,ES:btnval[di]
	call	BDRAW			;draw the button
	EXIT
;
;	Drw2: draw a radio button set
;
Drw2:	mov	ax,0			;set solid fill
	CALLDP	DSFPAT
	les	di,ENTRP		;calculate button positions
	fmove	CurX,ES:rbx1[di]	;center of 1st button
	fmove	CurY,ES:rby1[di]
	mov	al,ES:rbnrb[di]
	dec	al			;# buttons - 1
	mov	byte ptr Ngaps,al
	mov	al,ES:btnval[di]	;which button is on
	mov	OnBtn,al
	fld	ES:rbx2[di]		;calc x dist between buttons
	fsub	CurX
	fidiv	NGaps
	fstp	BumpX
	fld	ES:rby2[di]		;calc y dist between buttons
	fsub	CurY
	fidiv	NGaps
	fstp	BumpY
	mov	CBCtr,0			;which button we're drawing
RbLup:	inc	CBCtr
	fld	CurX			;draw the current button
	fld	CurY
	call	BTNPTS
	mov	al,0			;assume off
	mov	ah,OnBtn
	cmp	ah,CBCtr
	jnz	short NotOn
	mov	al,1			;it's turned on	
NotOn:	call	BDRAW
	fld	CurX			;advance to next button
	fadd	BumpX
	fstp	CurX
	fld	CurY
	fadd	BumpY
	fstp	CurY
	mov	al,CBCtr		;loop for all buttons		
	cmp	al,byte ptr NGaps
	jle	short RbLup
	EXIT				;done
;
;
;	BTNPTS defines corners of button whose center is on 8087
;
BTNPTS	proc	near
	fld	st(1)			;lower-left corner
	fsub	bsizex
	fstp	bllx
	fld	st(0)
	fsub	bsizey
	fstp	blly
	fld	st(1)			;upper-left corner
	fsub	bsizex
	fstp	bulx
	fld	st(0)
	fadd	bsizey
	fstp	buly
	fld	st(1)			;upper-right corner
	fadd	bsizex
	fstp	burx
	fld	st(0)
	fadd	bsizey
	fstp	bury
	fsub	bsizey			;lower-right corner
	fstp	blry
	fadd	bsizex
	fstp	blrx
	ret
btnpts	endp
;
;	BDRAW draws button set up by btnpts
;	      button is off or on depending on al=0 or 1
;
BDRAW	proc	near			;al=0 for off, 1 for on
	push	ax			;save button status
	mov	cx,4			;fill 4-node polygon
	mov	ax,DS
	mov	ES,ax
	lea	di,bllx
	SVC	DFPOLY
	pop	ax
	cmp	al,1			;is this button on?
	jz	short IsOn		
	ret				;not on so we're done
IsOn:	fld	bllx			;shrink button:lower left
	fadd	onsx
	fstp	bllx
	fld	blly
	fadd	onsy
	fstp	blly
	fld	bulx			;shrink button:upper left
	fadd	onsx
	fstp	bulx
	fld	buly
	fsub	onsy
	fstp	buly
	fld	burx			;shrink button:upper right
	fsub	onsx
	fstp	burx
	fld	bury
	fsub	onsy
	fstp	bury
	fld	blrx			;shrink button:lower right
	fsub	onsx
	fstp	blrx
	fld	blry
	fadd	onsy
	fstp	blry
	SVC	CINFO			;now erasing?
	test	bp,4
	jnz	short nof		;yes - no on fill
	mov	ax,0			;in black:
	CALLDP	DSCOLOR
nof:	mov	cx,4			;fill shrunken 4-node polygon
	mov	ax,DS
	mov	ES,ax
	lea	di,bllx
	SVC	DFPOLY
	SVC	CINFO			;restore drawing color
	mov	ax,di
	CALLDP	DSCOLOR
	ret				;button is finished
bdraw	endp
;
	align	4
bllx	dd	0.0			;lower-left corner
blly	dd	0.0
bulx	dd	0.0			;upper-left corner
buly	dd	0.0
burx	dd	0.0			;upper-right corner
bury	dd	0.0
blrx	dd	0.0			;lower-right corner
blry	dd	0.0
;
;	---------------------------------------------------------------
;	XPICK - Pick test our entities
;	---------------------------------------------------------------
;
XPICK:	les	di,ENTRP		;get control type #
	mov	bl,ES:betype[di]
	mov	bh,0
	shl	bx,1			;*2 for word table
	jmp	picktbl[bx]		;lookup address
picktbl	dw	0			;betype=0
	dw	offset pick1		;pick single button
	dw	offset pick2		;pick radio buttons
;
;	pick test a button
;
PICK1:	les	di,ENTRP		;ES:DI points to ENTREC
	fld	ES:btnx[di]		;get button lower-left corner
	fsub	bsizex
	fld	ES:btny[di]
	fsub	bsizey
	fld	ES:btnx[di]		;get button upper-right corner
	fadd	bsizex
	fld	ES:btny[di]
	fadd	bsizey
	SVC	PKBOX			;pick test the box	
	EXIT				;done
;
PICK2:	les	di,ENTRP		;ES:DI points to ENTREC
	fmove	CurX,ES:rbx1[di]	;center of 1st button
	fmove	CurY,ES:rby1[di]
	mov	al,ES:rbnrb[di]
	dec	al			;# buttons - 1
	mov	byte ptr ngaps,al
	fld	ES:rbx2[di]		;calc x dist between buttons
	fsub	CurX
	fidiv	NGaps
	fstp	BumpX
	fld	ES:rby2[di]		;calc y dist between buttons
	fsub	CurY
	fidiv	NGaps
	fstp	BumpY
;
	mov	CBCtr,0			;which button we're picking
pkLup:	inc	CBCtr			;pick test the current button
	fld	CurX			;get button lower-left corner
	fsub	bsizex
	fld	CurY
	fsub	bsizey
	fld	CurX			;get button upper-right corner
	fadd	bsizex
	fld	CurY
	fadd	bsizey
	clc				;clear carry flag
	SVC	PKBOX			;pick test the box	
	jc	pkdun
;
	fld	CurX			;advance to next button
	fadd	BumpX
	fstp	CurX
	fld	CurY
	fadd	BumpY
	fstp	CurY
	mov	al,CBCtr		;loop for all buttons		
	cmp	al,byte ptr NGaps
	jle	short pkLup
pkdun:	EXIT				;done
;
;
;	---------------------------------------------------------------
;	XMOVE - translate our entities
;	---------------------------------------------------------------
;
XMOVE:	les	di,ENTRP		;get control type #
	mov	bl,ES:betype[di]
	mov	bh,0
	shl	bx,1			;*2 for word table
	jmp	trnstbl[bx]		;lookup address
trnstbl	dw	0			;betype=0
	dw	offset trns1		;translate single button
	dw	offset trns2		;translate radio buttons
;
;	move a button
;
Trns1:	les	di,ENTRP		;ES:DI points to ENTREC.btnx
	add	di,btnx
	SVC	XTRP2			;transform (allow stretch test)
	EXIT				;done
Trns2:	les	di,ENTRP		;ES:DI points to ENTREC.rbx1
	add	di,rbx1
	SVC	XTRP2			;transform (allow stretch test)
	les	di,ENTRP		;ES:DI points to ENTREC.rbx1
	add	di,rbx2
	SVC	XTRP2			;transform (allow stretch test)
	EXIT
;
;	---------------------------------------------------------------
;	XLIST - list our entities
;	---------------------------------------------------------------
;
XLIST:	les	di,ENTRP		;get control type #
	mov	bl,ES:betype[di]
	mov	bh,0
	shl	bx,1			;*2 for word table
	jmp	listtbl[bx]		;lookup address
listtbl	dw	0			;betype=0
	dw	offset list1		;list single button
	dw	offset list2		;list radio buttons
;
;	list a button
;
List1:	les	di,ENTRP		;ES:DI points to ENTREC
	fld	ES:btnx[di]		;button location
	fstp	lstbx
	fld	ES:btny[di]
	fstp	lstby
	lea	bx,offmsg		;on or off status
	cmp	ES:btnval[di],0
	jz	isoff
	lea	bx,onmsg
isoff:	mov	onoff,bx
	mov	ax,DS			;display entity data
	mov	ES,ax
	lea	si,btnlt
	lea	dx,btnvl
	SVC	FORMDS
	EXIT				;done
;
List2:	les	di,ENTRP		;ES:DI points to ENTREC
	fmove	lstbx1,ES:rbx1[di]	;button 1 location
	fmove	lstby1,ES:rby1[di]
	fmove	lstbx2,ES:rbx2[di]	;last button location
	fmove	lstby2,ES:rby2[di]
	mov	al,ES:rbnrb[di]
	mov	lstnb,al		;number of buttons
	mov	al,ES:btnval[di]
	mov	lstcb,al		;which button is on
	mov	ax,DS			;format and display
	mov	ES,ax
	mov	si,offset rblst
	mov	dx,offset rblvl
	SVC	FORMDS
	EXIT	
;
;	variables for single button
;
btnlt	db	'CONTROLS: BUTTON at !1,!1, status=!5',13,13,0
;
btnvl	dw	lstbx			;button variable list
	dw	lstby
onoff	dw	0			;pointer to on/off msg
;
onmsg	db	'ON',0			;button status messages
offmsg	db	'OFF',0
lstbx	dd	0.0			;copy of button location in
lstby	dd	0.0			;our DS segment
;
;	variables for radio buttons
;
	align	4
lstbx1	dd	0.0			;copy of 1st button in DS seg
lstby1	dd	0.0
lstbx2	dd	0.0			;2nd button
lstby2	dd	0.0
lstnb	db	0			;# of buttons
lstcb	db	4			;which button is on
;
rblst	db	'CONTROLS: RADIO BUTTON from !1,!1 to !1,!1',13
	db	'                       !3 buttons, set at #!3',13,13,0
;
rblvl	dw	offset lstbx1		;button variable list
	dw	offset lstby1
	dw	offset lstbx2
	dw	offset lstby2
	dw	offset lstnb
	dw	offset lstcb
;
;	---------------------------------------------------------------
;	XXCHK - check our entities' extents
;	---------------------------------------------------------------
;
XXCHK:	les	di,ENTRP		;get control type #
	mov	bl,ES:betype[di]
	mov	bh,0
	shl	bx,1			;*2 for word table
	jmp	xtchktbl[bx]		;lookup address
xtchktbl	dw	0			;betype=0
	dw	offset xtchk1		;extent check single button
	dw	offset xtchk2		;extent check radio buttons
;

;
;	check a button's extents
;
XtChk1:	les	di,ENTRP		;ES:DI points to ENTREC
	fld	ES:btnx[di]		;get button lower-left corner
	fsub	bsizex
	fld	ES:btny[di]
	fsub	bsizey
	fld	ES:btnx[di]		;get button upper-right corner
	fadd	bsizex
	fld	ES:btny[di]
	fadd	bsizey
	SVC	XCHKLN			;check extents
	EXIT				;done
;
XtChk2:	les	di,ENTRP		;ES:DI points to ENTREC
	fld	ES:rbx1[di]		;get radio button lower-left corner
	fsub	bsizex
	fld	ES:rby1[di]
	fsub	bsizey
	fld	ES:rbx2[di]		;get radio button upper-right corner
	fadd	bsizex
	fld	ES:rby2[di]
	fadd	bsizey
	SVC	XCHKLN			;check extents
	EXIT				;done
;
;	---------------------------------------------------------------
;	XACT - Actuate our entitie - 8087 holds X,Y pick point	
;	---------------------------------------------------------------
;
XACT:	les	di,ENTRP		;get control type #
	mov	bl,ES:betype[di]
	mov	bh,0
	shl	bx,1			;*2 for word table
	jmp	acttbl[bx]		;lookup address
acttbl	dw	0			;betype=0
	dw	offset act1		;activate single button
	dw	offset act2		;activate radio buttons
;
;	actuate a button
;
Act1:	finit				;no need for pick point
	les	di,ENTRP		;ES:DI points to ENTREC
	xor	ES:btnval[di],1		;toggle on/off status
	SVC	MODIFY			;update record in database
	SVC	DRAW			;redraw it to reflect change
	EXIT				;done
;
;
Act2:	
	fstp	CpkY			;this needs the pick point
	fstp	CpkX			;so save x,y from stack
	les	di,ENTRP		;ES:DI points to ENTREC
	mov	al,ES:rbnrb[di]
	dec	al			;# buttons - 1
	mov	byte ptr Ngaps,al
	fld	ES:rbx2[di]		;X center of 2nd button
	fsub	ES:rbx1[di]		;diff is deltaX
	fstp	DelX
	fld	ES:rby2[di]		;Y center of 2nd button
	fsub	ES:rby1[di]		;diff is deltaY
	fst	DelY			;leave it on stack
;
	mov	dl,0			;clear flag
	fcomp	DelX			;
	fjge	okmm			;if DelX>DelY then DL=1
	mov	dl,1			;set DL means DelX is bigger
okmm:
	test	dl,1			;
	jz	short useY
useX:
	fld	CpkX			;original Picked X
	fsub	ES:rbx1[di]		;short X on stack
	fld	ES:rbx2[di]
	fsub	ES:rbx1[di]		;long X on stack
	jmp	short actdun
useY:
	fld	CpkY			;original Picked Y
	fsub	ES:rby1[di]		;short Y on stack
	fld	ES:rby2[di]
	fsub	ES:rby1[di]		;long Y on stack
actdun:
	fdivp	st(1),st		; short/long
	fimul	Ngaps			; * ngaps
	fadd	kp25			; + 0.25
	fistp	tmpz			;
	mov	ax,tmpz			;
	inc	ax
	mov	ES:btnval[di],al	;set button on status
	SVC	MODIFY			;update record in database
	SVC	DRAW			;redraw it to reflect change
	EXIT				;done
;
	align	4
CpkX	dd	0.0
CpkY	dd	0.0
DelX	dd	0.0
DelY	dd	0.0
kp25	dd	0.25
Tmpr	dd	0.0
Tmpz	dw	0
;
;	---------------------------------------------------------------
;
	include	XPEND.CPY
