;	ESTRUC.CPY - entity record structure
;
;	(c) Copyright 1986,87,88,89 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  2/23/86
;	modified January, 1987 MR
;	LDIM, ADIM estyle changed 8/12/87 MR
;	3d adds starting 4/8/88 MR
;
;	notes:
;
;	1) bit 80H of ETYPE is set for deleted entities. All deleted
;	   entities are removed when the file is saved, so this should
;	   only be set while the file is in RAM.
;
;	common to all entities:
;
comer	STRUC
erlen	dw	0		;entity record length
etype	db	0		;entity type code
elayer	db	0		;entity layer
ecolor	db	0		;entity color
estyle	db	0		;entity style
comer	ENDS
;
;	----------------------------------------------------------------
;	etype 1: POINT entity data (13 bytes)
;	----------------------------------------------------------------
;
points	STRUC
	db	0,0,0,0,0	;common stuff (no estyle)
px	dd	0.0		;point coordinate
py	dd	0.0
points	ENDS
;
;	----------------------------------------------------------------
;	etype 2: LINE entity data (22 bytes)
;	----------------------------------------------------------------
;
lines	STRUC
	dw	0,0,0		;common stuff
lx1	dd	0.0		;line starting point
ly1	dd	0.0
lx2	dd	0.0		;line ending point
ly2	dd	0.0
lines	ENDS
;
;	----------------------------------------------------------------
;	etype 3: TEXT (27+nchrs+1 bytes)
;	----------------------------------------------------------------
;
txts	STRUC
	db	0,0,0,0,0	;common stuff
tjust	db	0		;text justification code
tfont	db	0		;text font id #
txx	dd	0.0		;text origin
txy	dd	0.0
thgt	dd	0.0		;text height
txar	dd	1.0		;text cell aspect ratio
tang	dd	0.0		;text angle
tdata	db	0		;text data ASCIIZ string
txts	ENDS
;
;	----------------------------------------------------------------
;	etype 4: CIRCLE entity data (18 bytes)
;	----------------------------------------------------------------
;
circles	STRUC
	dw	0,0,0		;common stuff
cenx	dd	0.0		;circle center
ceny	dd	0.0
crad	dd	0.0		;circle radius
circles	ENDS
;
;	----------------------------------------------------------------
;	etype 5: ARC entity data (26 bytes)
;	----------------------------------------------------------------
;
arcs	STRUC
	dw	0,0,0		;common stuff
acenx	dd	0.0		;arc center
aceny	dd	0.0
arad	dd	0.0		;arc radius
asang	dd	0.0		;arc starting angle (radians)
aangw	dd	0.0		;arc angular width (radians)
arcs	ENDS
;
;	----------------------------------------------------------------
;	etype 6: PATH (7+8*#nodes bytes)
;	etype 7: POLY (7+8*#nodes bytes)
;	etype 8: SPOLY (7+8*#nodes bytes)
;	etype 9: SPLINE (7+8*#nodes bytes)
;	etype 10: ARROW (7+8*#nodes bytes)
;	etype 11: DARROW (7+8*#nodes bytes)
;
;	Estyle bit usage:
;		bit 7=1 filled, 0=hollow
;		bit 6=1 no outline (fill), 0=normal
;		bit 5..0 = fill style, bits 3..0 = line style
;
;	[future]:
;	add flag byte, bits show arrows at either end.  No more
;	arrow, darrow entities.
;	----------------------------------------------------------------
;
POLYS	struc
	dw	0,0,0		;common stuff
pcount	db	0		;node count
;pflg	db	0		;flag bits
polyx	dd	0.0		;first node point
polyy	dd	0.0
polys	ENDS
;
POLYN	struc
polyx1	dd	0.0		;node point 1
polyy1	dd	0.0
polyx2	dd	0.0		;node point 2
polyy2	dd	0.0
polyx3	dd	0.0		;node point 3
polyy3	dd	0.0
polyx4	dd	0.0		;node point 4
polyy4	dd	0.0
polyn	ENDS
;
;	----------------------------------------------------------------
;	etype 12: LDIM - linear dimension entity (44/48 bytes)
;	----------------------------------------------------------------
;
LDIM	struc
	dw	0,0,0		;style is dim format,# dec. places
ldtxp	db	0		;0=inside ,1=over, 2=left, 3=right
;				;inside and over are std center arc text
ldndp	db	5		;# decimal places for numeric text format
dhgt	dd	0.0		;dimension text height
dimx1	dd	0.0		;dimension point 1
dimy1	dd	0.0
dimx2	dd	0.0		;dimension point 2
dimy2	dd	0.0
dimx3	dd	0.0		;dimension point 3
dimy3	dd	0.0
dimx4	dd	0.0		;dimension point 4
dimy4	dd	0.0
dvalue	dd	0.0		;dimension fixed value (optional)
ldim	ENDS
;
;	----------------------------------------------------------------
;	etype 13: WLINE (11+8*#nodes bytes)
;	----------------------------------------------------------------
;
WLINS	struc
	dw	0,0,0		;common stuff
wlcnt	db	0		;node count
wlwid	dd	0.0		;wide line width
wlinx	dd	0.0		;first node point
wliny	dd	0.0
wlins	ENDS
;
;	----------------------------------------------------------------
;	etype 14: GROUP Markers (6 bytes)
;	----------------------------------------------------------------
;
GRPS	struc
	db	0,0,0,0,0	;common stuff
gcode	db	0		;group marker type code
;				=1 for start group
;				=0 for end group
grps	ENDS
;
;	----------------------------------------------------------------
;	etype 15: ADIM - angular dimension entity (40/44 bytes)
;	----------------------------------------------------------------
;
ADIM	struc
	dw	0,0,0		;estyle is leader line style
adtxp	db	0		;0=inside ,1=over, 2=left, 3=right
;				;inside and over are std center arc text
adndp	db	5		;# decimal places for numeric text format
adhgt	dd	0.0		;dimension text height
adcx	dd	0.0		;dimension center
adcy	dd	0.0
adrad	dd	0.0		;arc radius
adsa	dd	0.0		;dim start angle
adaw	dd	0.0		;dim arc width
adsir	dd	0.0		;start leader inner radius
adeir	dd	0.0		;end leader inner radius
advalue	dd	0.0		;fixed display value (optional)
adim	ENDS
;
;	----------------------------------------------------------------
;	etype 16: EXT - extended functions entity (8 or more bytes)
;	----------------------------------------------------------------
;
EXTE	struc
	dw	0,0,0		;common stuff
exid	dw	0		;extended function mod ID # (asgnd. by E.C.)
xetype	db	0		;extended function mod entity type (1..255)
xvar	db	0		;other variable data may follow
exte	ENDS
;
;	Note that commercial packages MUST have their EXID value
;	assigned by Evolution Computing to prevent conflicts.
;	Use only one number per product.  You may make any use
;	of xetype that you desire.  EXID=values 0F000H..0FFFFH
;	are reserved for end user (non-product) use in site-specific
;	packages. Conflict with other users is practically guaranteed
;	if these values are used.
;
;	----------------------------------------------------------------
;	etype 17: SYMBOL DEFINITION GROUP Marker (22 bytes)
;
;	Symbol definitions consist of SYMDEF record,
;	followed by SymECnt defining entities.
;	----------------------------------------------------------------
;
SDEF	struc
	dw	0,0,0		;common stuff
SymECnt	dw	0		;# of entities in symbol (this ent. not cnt)
;				;CurRec+SymECnt=last included record
SDName	db	12 dup (0)	;symbol name text 1..12 chrs, pad NULLs
SDEOF	db	0		;eos byte
	db	0		;unused, = 0
sdef	ENDS
;
;	----------------------------------------------------------------
;	etype 18: SYMBOL REFERENCE entity (68 bytes)
;	----------------------------------------------------------------
;
SREF	struc
	dw	0,0,0		;common stuff
symtm	dd	12 dup (0.0)	;symbol partial transform matrix
symfb	db	0		;matrix flag bits
;				;bit 80h = 1 if symbol is mirrored
;				;bit 01h = 1 if scalex <> scaley
SRName	db	12 dup (' ')	;symbol name text 1..12 chrs, pad NULLs
SREOF	db	0		;eos byte
sref	ends
;
;	----------------------------------------------------------------
;	OLD etype 18: SYMBOL REFERENCE entity (40 bytes)
;	----------------------------------------------------------------
;
OldSREF	struc
	dw	0,0,0		;common stuff
symatx	dd	0.0		;symbol origin at x,y
symaty	dd	0.0
symrot	dd	0.0		;symbol rotated by
symscx	dd	0.0		;symbol X scaling
symscy	dd	0.0		;symbol Y scaling
OSRName	db	12 dup (' ')	;symbol name text 1..12 chrs, pad NULLs
OSREOF	db	0		;eos byte
	db	0		;unused, = 0
Oldsref	ends
;
;	----------------------------------------------------------------
;	etype 19: ELLIPSE entity (26 bytes)
;	----------------------------------------------------------------
;
ELIS	struc
	dw	0,0,0		;common stuff
ECenX	dd	0.0		;ellipse center
ECenY	dd	0.0
EMajR	dd	0.0		;major axis radius
Eccen	dd	0.0		;eccentricity = Minor/Major axis (0..1)
EIncl	dd	0.0		;inclination of maj axis (bearing ang 0..pi)
elis	ends
;
;	----------------------------------------------------------------
;	etype 20: ELLIPTICAL ARC (34 bytes)
;	----------------------------------------------------------------
;
ELAS	struc
	db	26 dup (0)	;common+ellipse data
ESAng	dd	0.0		;starting angle
EAngW	dd	0.0		;arc angular wid 0..2pi
elas	ends
;
;	----------------------------------------------------------------
;	etype 21: 3D LINE entity data (30 bytes)
;	----------------------------------------------------------------
;
line3	STRUC
	dw	0,0,0		;common stuff
l3x1	dd	0.0		;line starting point
l3y1	dd	0.0
l3z1	dd	0.0
l3x2	dd	0.0		;line ending point
l3y2	dd	0.0
l3z2	dd	0.0
line3	ENDS
;
;	----------------------------------------------------------------
;	etype 22: PATH3 (8+12*#nodes bytes)
;	etype 23: POLY3 (8+12*#nodes bytes)
;	etype 25: SPOLY3 (8+12*#nodes bytes)
;	etype 26: SPLINE3 (8+12*#nodes bytes)
;	----------------------------------------------------------------
;
POLY3	struc
	dw	0,0,0		;common stuff
p3cnt	db	0		;node count
p3flg	db	0		;flags (unused)
poly3x	dd	0.0		;first node point
poly3y	dd	0.0
poly3z	dd	0.0
poly3	ENDS
;
;	----------------------------------------------------------------
;	etype 24:SOLIDS (8+2*#sections+12*#nodes)
;	----------------------------------------------------------------
;
SOLID	struc
	dw	0,0,0		;common stuff
s3flg	db	0		;flags (unused)
;	Each section consists of:
s3ncnt	db	0		;section node count
s3sflg	db	0		;section flags (unused)
s3dta	dd	0.0		;start of node list x,y,z ...
;	S3NCNT=0 byte flags the end of the section list
solid	ends
;
;	----------------------------------------------------------------
;	etype 27: 3D POINT entity data (17 bytes)
;	----------------------------------------------------------------
;
point3	STRUC
	db	0,0,0,0,0	;common stuff (no estyle)
p3x	dd	0.0		;point coordinate
p3y	dd	0.0
p3z	dd	0.0
point3	ENDS
;
;	----------------------------------------------------------------
;	etype 28: 3D CIRCLE entity data (34 bytes)
;	----------------------------------------------------------------
;
circle3	STRUC
	dw	0,0,0		;common stuff
cenx3	dd	0.0		;circle center
ceny3	dd	0.0
cenz3	dd	0.0
crad3	dd	0.0		;circle radius
crxy	dd	0.0		;orientation angle in XY plane
cryz	dd	0.0		;orientation angle in YZ plane
crzx	dd	0.0		;orientation angle in ZX plane
circle3	ENDS
;
;	----------------------------------------------------------------
;	etype 29: 3D ARC entity data (38 bytes)
;	----------------------------------------------------------------
;
arc3	STRUC
	dw	0,0,0		;common stuff
acenx3	dd	0.0		;arc center
aceny3	dd	0.0
acenz3	dd	0.0
arad3	dd	0.0		;arc radius
arxy	dd	0.0		;orientation angle in XY plane
aryz	dd	0.0		;orientation angle in YZ plane
arzx	dd	0.0		;orientation angle in ZX plane
aangw3	dd	0.0		;arc angular width (radians)
arc3	ENDS
;
;	----------------------------------------------------------------
;	etype 30: TEXT (41+nchrs bytes)
;	----------------------------------------------------------------
;
texts	STRUC
	dw	0,0,0		;common stuff
tjust3	db	0		;text justification code
tfont3	db	0		;text font id #
txx3	dd	0.0		;text origin
txy3	dd	0.0
txz3	dd	0.0
thgt3	dd	0.0		;text height
txar3	dd	1.0		;text cell aspect ratio
trxy	dd	0.0		;text angle in XY plane
tryz	dd	0.0		;text angle in YZ plane
trzx	dd	0.0		;text angle in ZX plane
tdata3	db	0		;text data ASCIIZ string
texts	ENDS
;
;	----------------------------------------------------------------
;	etype 31: 3D ELLIPSE entity data (42 bytes)
;	----------------------------------------------------------------
;
ELIS3	STRUC
	db	6 dup (0)	;common stuff
Ecenx3	dd	0.0		;ellipse center
Eceny3	dd	0.0
Ecenz3	dd	0.0
Erad3	dd	0.0		;major radius
erxy	dd	0.0		;orientation angle in XY plane
eryz	dd	0.0		;orientation angle in YZ plane
erzx	dd	0.0		;orientation angle in ZX plane
Eec3	dd	0.0		;eccentricity
Ein3	dd	0.0		;inclination (XY rotation)
elis3	ENDS
;
;	----------------------------------------------------------------
;	etype 32: 3D ELLIPTICAL ARC entity data (46 bytes)
;	----------------------------------------------------------------
;
ELAS3	STRUC
	db	42 dup (0)	;ellipse common stuff
Eangw3	dd	0.0		;arc angular width (radians)
elas3	ENDS
;
;	----------------------------------------------------------------
;	etype 33: 3D Tabulated Profiles
;	size = 17 + val(TCSCNT)*24 + val(TC3ESZ)
;	----------------------------------------------------------------
;
TPROS	STRUC
	dw	0,0,0		;common stuff
tc3lrs	db	0		;linear axis resolution
tc3ars	db	0		;angular axis resolution
tc3flg	db	0		;bit1=1stEndCap,bit2=2ndEndCap
tcscnt	db	0		;section count (1=flat)
tc3thk	dd	0.0		;edge thickness (0=surf only)
tc3pth	dd	0.0		;path data (TCSCNT copies of TC3P below)
;				;variable length=24*val(TCSCNT)
;	db	0		;edge data (1 set of TC3E below)
;				starting at adr(TC3PTH)+24*val(TCSCNT)
tpros	ends
;
TP3P	STRUC
tc3x	dd	0.0		;path node point
tc3y	dd	0.0
tc3z	dd	0.0
tc3s	dd	1.0		;node scale factor (<1 = shrink)
tc3ryz	dd	0.0		;intersection plane tilt angles
tc3rzx	dd	0.0
tp3p	ends
;
;	TC3DTA is edge geometry (see EDGE.CPY)
;	TC3ESZ = # of geometry data bytes starting with TC3DTA
;	TC3TYP = geometry type code
;
TP3E	STRUC
tc3typ	db	0		;edge geometry type
tc3esz	dw	0		;# bytes of edge data
tc3dta	db	0		;edge-specific data
tp3e	ends
;
;	----------------------------------------------------------------
;	etype 34: 3D Spherical Section (62 bytes)
;	----------------------------------------------------------------
;
SPS3S	STRUC
	dw	0,0,0		;common stuff
sp3lrs	db	0		;longitude resolution (CLRes)
sp3mrs	db	0		;meridian resolution (CARes)
;				;0 = use current values (else fixed)
sp3flg	db	0		;flags:
;				;  1 = Dome/Sec has a top hole
sp3st	db	0		;subtype:
;				;  0 = sphere
;				;  1 = dome
;				;  2 = dome section
sp3cx	dd	0.0		;sphere center point
sp3cy	dd	0.0
sp3cz	dd	0.0
sp3rd	dd	1.0		;sphere radius (outer radius)
sp3thk	dd	0.0		;sphere thickness (0=surface only)
sp3rxy	dd	0.0		;orientation angles
sp3ryz	dd	0.0
sp3rzx	dd	0.0
sp3ta	dd	1.570796327	;dome top start angle (+90..-90)
sp3da	dd	3.141592654	;dome angle (360 max - +Z to -Z)
sp3sa	dd	3.141592654	;section angle (360 +X ccw to +X)
sp3pr1	dd	1.0		;0 polar ratio (Z to X)
sp3pr2	dd	1.0		;90 polar ratio (Z to Y)
SPS3S	ends
;
;	----------------------------------------------------------------
;	etype 35: 3D Surface of Revolution (SREV) 44+REVESZ # bytes
;	----------------------------------------------------------------
;
SURREV	STRUC
	dw	0,0,0		;common stuff
revlrs	db	0		;linear axis resolution
revars	db	0		;angular axis resolution
revpx	dd	0.0		;rev axis point
revpy	dd	0.0
revpz	dd	0.0
revdx	dd	0.0		;rev axis direction vector
revdy	dd	0.0
revdz	dd	0.0
revaaw	dd	0.0		;arc angle width (2pi = full rev)
revthk	dd	0.0		;edge thickness (0=surf only)
revflg	db	0		;revolved surface flags
;				;bit 1 = 1 if edge ends close
;				;bit 2 = 1 if rev ends close
;
;	REVDTA is edge geometry (see EDGE.CPY)
;	REVESZ = # of geometry data bytes starting with REVDTA
;	REVTYP = geometry type code
;
revtyp	db	0		;edge geometry type
revesz	dw	0		;# bytes of edge data
revdta	db	0		;edge-specific data
;
surrev	ends
;
;	----------------------------------------------------------------
;	etype 36: PADEF - Plot Assembly Definition
;	#bytes=22+PadqdValue+NameLength+1
;	----------------------------------------------------------------
;
padef	struc
	db	5 dup (0)	;common stuff
paflgs	db	0		;(estyle) global plot flags
;				;bits 03h = destination code
imgw	dw	0		;PTF image width
imgh	dw	0		;PTF image height
;				;1=portrait mode 0=landscape (normal)
padnum	dw	0		;plot assembly definition number
pasx	dd	0.0		;plot area size in real units
pasy	dd	0.0
padqd	dw	0		;# of qdata data bytes (padqd not incl.)
	db	?		;qdata data bytes (0..padqd)
	db	?		;ASCIIZ plot assembly name
padef	ends
;
;	----------------------------------------------------------------
;	etype 37: PAWIN - Plot Assembly Window
;	#bytes=32+PawvdlValue+PawfnLength+1+PawwnLength+1
;	----------------------------------------------------------------
;
pawin	struc
	db	6 dup (0)	;common stuff (estyle=plot flags)
pawnum	dw	0		;matches plot assembly definition number
pawx	dd	0.0		;window size in real units
pawy	dd	0.0
pawox	dd	0.0		;window position from lower-left corner
pawoy	dd	0.0
pawscl	dd	0.0		;scale data (0=not scaled)
pawvc	db	0		;view select code
;				;0=active window
;				;1-8=stored view #
;				;9=all (ZEXT)
;				;10=text file window
pawvf	db	0		;view flags; 1=frame the window in ecolor
pawvdl	dw	0		;view data length (standard=0)
;
;	text file "view" only has this data (pawvdl=6)
pawth	dd	0.0		;text height if text window
pawfnt	db	0		;font # if text window
pawjust	db	0		;justification code if text window
;
pawfn	db	?		;ASCIIZ .FCD file name (NULL=this file)
pawwn	db	?		;ASCIIZ plot assembly window name (for edit)
;
pawin	ends
;
;	----------------------------------------------------------------
;	etype 38: 3D Mesh Surface #bytes=I*J*12+12
;	Mesh(Row,column) column values change fastest
;	----------------------------------------------------------------
;
SURMSH	STRUC
	dw	0,0,0		;common stuff
	db	0		;reserved
MSFlgs	db	0		;flag bits
;				;1=close I direction
;				;2=close J direction
;				;4=spline smooth surface
MSurI	dw	0		;# I direction lines (columns)
MSurJ	dw	0		;# J direction lines (rows)
;
MSurD	dd	0.0		;3D point data array J*I (all I then J)
surmsh	ends
;
;	----------------------------------------------------------------
;	etype 39: CSURF - 3D Coons Surface
;	#bytes = 14+4*3+EdgSiz1+EdgSiz2+EdgSiz3+EdgSiz4
;	----------------------------------------------------------------
;
COONSF	STRUC
	dw	0,0,0		;common stuff
cslrs	db	0		;linear axis resolution
csars	db	0		;angular axis resolution
cse2o	dw	0		;offset of second edge in entrec
cse3o	dw	0		;offset of third edge in entrec
cse4o	dw	0		;offset of forth edge in entrec
;
;	the edge data structures for each edge follow in order
;
csedg1	db	0		;first edge data structure starts here
coonsf	ends
;
;	----------------------------------------------------------------
;	etype 40: RSURF - 3D Ruled Surface
;	#bytes = 8+2*3+EdgSiz1+EdgSiz2
;	----------------------------------------------------------------
;
RULSF	STRUC
	dw	0,0,0		;common stuff
rslrs	db	0		;linear axis resolution
rsars	db	0		;angular axis resolution
;
;	the edge data structures for each edge follow in order
;
rsedg1	db	0		;first edge data structure starts here
rulsf	ends
;
;	----------------------------------------------------------------
;	etype 41:Radius/Diameter dims: entity length 36 bytes
;	----------------------------------------------------------------
;
rdim	struc
	dw	0,0,0		;standard stuff
rdflgs	db	0		;flags (see comments below)
rddecp	db	0		;number of decimal places
rdhgt	dd	0.0		;dim text height
rdimx	dd	0.0		;center X
rdimy	dd	0.0		;center Y
rdrad	dd	0.0		;radius
rdang	dd	0.0		;arrow angle from center
rdlead	dd	0.0		;leader length (to bend pt)
rdtail	dd	0.0		;leader tail length
				;(0=no tail, <0 tail points to L)
rdim	ends
;
;rdflgs byte: 
;	bit 0:	01h: 0=inside, 1=over (ignored if global unidirectional)
;	bit 1:	02h: (left/right) ignored
;	bit 2:	04h: reserved
;	bit 3:	08h: reserved
;	bit 4:	10h: 0=diameter, 1=radius
;	bit 5:	20h: 0=no leader, 1=yes leader (text outside if 1)
;	bit 6:	40h: 0=no leader tail, 1=yes leader tail
;	bit 7:	80h: 0=no internal arrows (note), 1=yes int. arrows
;
;	----------------------------------------------------------------
;	etype 42:Ordinate dims: entity length 28+8*# of leaders
;	----------------------------------------------------------------
;
odim	struc
	dw	0,0,0		;standard stuff
odtxp	db	0		;0=inside ,1=over, 2=left, 3=right
;				;(ignored by this dimension)
;				;bit 80h 0=US, 1=Japan (with arrows)
oddecp	db	5		;# decimal places for numeric text format
odhgt	dd	0.0		;dim text height
odbsx	dd	0.0		;baseline origin x,y
odbsy	dd	0.0
odang	dd	0.0		;baseline angle
odorgl	dd	0.0		;distance to geom. origin
;
;	Then there are displacements for each leader
;
odispx	dd	0.0		;distance from odbsx,odbsy
odispy	dd	0.0		;leader length
;
;	The following code will calculate the # of leaders,
;	origin line not included:
;
;	mov	cx,ENTREC.erlen	;calc # leaders -> CX
;	sub	cx,28
;	shr	cx,1
;	shr	cx,1
;	shr	cx,1
;
odim	ends
;
;	----------------------------------------------------------------
;	etype 43: ACTION TEXT (24+nchrs+1 bytes)
;	----------------------------------------------------------------
;
atxt	STRUC
	db	0,0,0,0,0,0	;common stuff
AWBT	db	0		;action window box type (0=none)
AWFlgs	db	0		;action window flags (unused)
AWXL	dd	0.0		;action window lower right
AWYL	dd	0.0
AWXH	dd	0.0		;action window upper left
AWYH	dd	0.0
adata	db	0		;text data ASCIIZ string
atxt	ENDS
;
;	----------------------------------------------------------------
;	etype 44: 3D LIGHT entity data (36/48/60 bytes)
;	----------------------------------------------------------------
;
light3	STRUC
	db	0,0,0,0,0,0	;common stuff
lgttyp	db	0		;light type
;				;0 = ambient (36 bytes)
;				;1 = distant parallel (48)
;				;2 = point source (36)
;				;3 = spotlight (60)
lonoff	db	1		;light on 1, off 0
;
lgti	dd	1.0		;light intensity (0=off)
lgtr	dd	1.0		;light color RGB 0..1
lgtg	dd	1.0
lgtb	dd	1.0
lgtx1	dd	0.0		;light source point
lgty1	dd	0.0
lgtz1	dd	-1.0
;
lgtx2	dd	0.0		;light pointing at point
lgty2	dd	0.0
lgtz2	dd	-1.0
;
lgtca	dd	0.523598	;spotlight cone angle in radians
lgtpa	dd	0.087266	;spotlight cone delta (penumbra width)
lgtbd	dd	2.0		;beam distribution exponent
light3	ENDS
;
;	----------------------------------------------------------------
;	etype 45: 3D MPOLY entity data (8+sum of PLIST lengths bytes)
;	----------------------------------------------------------------
;
MPY3	STRUC
	db	0,0,0,0,0,0	;common stuff
mpflgs	db	0		;mpoly flags (unused)
mpcnt	db	1		;# of PLISTS
mpldta	db	0		;1st PLIST starts here
;				;MPCNT PLISTs follow in sequence
;				;the first 4 bytes of each PLIST
;				;are the same.
mpy3	ends
;
;	PLIST type 0: 3d node list
;
PLST0	STRUC
mpllen	dw	0		;PLIST length
mpltyp	db	0		;PLIST type
mplflg	db	0		;PLIST flags
;				; 0=POLY, 1=PATH, 2=SPLINE, 3=SPOLY
pl0cnt	dw	0		;# of nodes
pl0dta	dd	0.0		;PL0CNT* 3d points
plst0	ends
;
;	PLIST type 1: 3d Circle/Arc
;
PLST1	STRUC
	dw	32/36		;PLIST length
	db	1		;PLIST type
	db	0		;PLIST flags 0=CIRCLE, 1=ARC
;	3d arc geometry (circle does not have MPAAW3)
mpacx3	dd	0.0		;arc center
mpacy3	dd	0.0
mpacz3	dd	0.0
mpard3	dd	0.0		;arc radius
mparxy	dd	0.0		;orientation angle in XY plane
mparyz	dd	0.0		;orientation angle in YZ plane
mparzx	dd	0.0		;orientation angle in ZX plane
mpaaw3	dd	0.0		;arc angular width (radians)
plst1	ends
;
;	PLIST type 2: 3d Text
;
PLST2	STRUC
	dw	38		;PLIST length (38+nchrs+0eos)
	db	2		;PLIST type
	db	0		;PLIST flags (unused)
;	3d text geometry
mptj	db	0		;justification
mptf	db	0		;font #
mptx	dd	0.0		;location
mpty	dd	0.0
mptz	dd	0.0
mpth	dd	1.0		;text height
mptar	dd	1.0		;aspect ratio
mptrxy	dd	0.0		;X-orientation
mptryz	dd	0.0
mptrzx	dd	0.0
mptxt	db	0		;ASCIIZ text string
plst2	ends
;
;	----------------------------------------------------------------
;	etype 46: 3D Cylinder (58 bytes)
;	----------------------------------------------------------------
;
CYL	STRUC
	dw	0,0,0		;common stuff
cylrs	db	0		;linear axis resolution
cyars	db	0		;angular axis resolution
cyflg	db	0		;bit1=1=1stEndCap;bit2=1=2ndEndCap
cyflg2	db	0		;unused
cybx	dd	0.0		;base center point
cyby	dd	0.0
cybz	dd	0.0
cybrd	dd	0.0		;base outer radius
cyrxy	dd	0.0		;orientation angles
cyryz	dd	0.0
cyrzx	dd	0.0
cyaaw	dd	0.0		;arc angular width (circle = 0)
cyhgt	dd	1.0		;cylinder height
cytrd	dd	0.0		;top outer radius
cybtk	dd	0.0		;base thickness (0 = none)
cyttk	dd	0.0		;top thickness (0 = none)
cyl	ends
;
;	----------------------------------------------------------------
;	etype 47: 3d WorkPlane Definition (154+namechrs bytes)
;	----------------------------------------------------------------
;
WPLND	struc
	dw	0,0,0		;common stuff
wpdid	dw	0		;workplane ID number
wporg	dd	0.0		;workplane origin
	dd	0.0
	dd	0.0
wpxref	dd	0.0		;workplane +X reference
	dd	0.0
	dd	0.0
wpfref	dd	0.0		;workplane forward meridian
	dd	0.0
	dd	0.0
wprxy	dd	0.0		;workplane tilt angles
wpryz	dd	0.0
wprzx	dd	0.0
WP2WC	dd	12 dup (0.0)	;workplane to world tform
WC2WP	dd	12 dup (0.0)	;world to workplane tform
wpdflg	db	0		;workplane flags
wpdname	db	0		;ASCIIZ name string
wplnd	ends
;
;	----------------------------------------------------------------
;	etype: 48: 3d WorkPlane Linear Dimension (58 bytes)
;	----------------------------------------------------------------
;
WPLDM	struc
	dw	0,0,0		;common stuff
wpldid	dw	0		;workplane ID number
wpld1	dd	0.0		;workplane dim start ref point
	dd	0.0
	dd	0.0
wpld2	dd	0.0		;workplane dim end ref point
	dd	0.0
	dd	0.0
wpldb	dd	0.0		;workplane dim baseline ref point
	dd	0.0
wpldt	dd	0.0		;workplane dim text center point
	dd	0.0
wpldh	dd	0.0		;dim text height
wpldd	dd	0.0		;dimension distance
wpldf	db	0		;flags: 01h=1 if vert, 0 if horiz dim
;				;       02h=1 if centered text, 0=positioned
wpldnp	db	0		;# decimal places
wpldm	ends
;
;	----------------------------------------------------------------
;	etype 49: 2D MPOLY entity data (8+sum of PLIST lengths bytes)
;	----------------------------------------------------------------
;
MPY2	STRUC
	db	0,0,0,0,0,0	;common stuff
mp2flgs	db	0		;mpoly flags (unused)
mp2cnt	db	1		;# of PLISTS
mp2dta	db	0		;1st PLIST starts here
;				;MPCNT PLISTs follow in sequence
;				;the first 4 bytes of each PLIST
;				;are the same.
mpy2	ends
;
;	PLIST type 0: 2d node list
;
PLST20	STRUC
mp2len	dw	0		;PLIST length (6+p20cnt*8)
mp2typ	db	0		;PLIST type
mp2flg	db	0		;PLIST flags
;				; 0=POLY, 1=PATH, 2=SPLINE, 3=SPOLY
p20cnt	dw	0		;# of nodes
p20dta	dd	0.0		;PL0CNT * 2d points
plst20	ends
;
;	PLIST type 1: 2d Circle/Arc
;
PLST21	STRUC
	dw	16/24		;PLIST length
	db	1		;PLIST type
	db	0		;PLIST flags 0=CIRCLE, 1=ARC
;	2d arc geometry (circle does not have MP2ASA,MPAAW2)
mp2acx	dd	0.0		;arc center
mp2acy	dd	0.0
mp2ard	dd	0.0		;arc radius
mp2asa	dd	0.0		;arc starting angle (radians)
mp2aaw	dd	0.0		;arc angular width (radians)
plst21	ends
;
;	PLIST type 2: 2d Text
;
PLST22	STRUC
	dw	26		;PLIST length (26+nchrs+0eos)
	db	2		;PLIST type
	db	0		;PLIST flags (unused)
;	2d text geometry
mp2tj	db	0		;justification
mp2tf	db	0		;font #
mp2tx	dd	0.0		;location
mp2ty	dd	0.0
mp2th	dd	1.0		;text height
mp2tar	dd	1.0		;aspect ratio
mp2ta	dd	0.0		;2d text angle
mp2txt	db	0		;ASCIIZ text string
plst22	ends
;
