;	TFIO.CPY - Text File I/O Macros and Routines for XP & DL modules
;
;	Copyright 1990 Evolution Computing
;	All rights reserved
;	Written by Mike Riddle 1-8-90
;
;	---------------------------------------------------------------
;	TEXT FILE OUTPUT
;	---------------------------------------------------------------
;
;	Declare a buffer for any text file that you will write with:
;
;		TFBFR	MyFile
;
;	and initialize it with the open file handle:
;
;		mov	ax,FileHandle
;		TFINIT	MyFile
;
;	Then you can write ASCIIZ strings to that file with:
;
;		TFWSTG	MyFile,MyString
;
;	Or, by putting the string address in SI:
;
;		mov	si,offset MyString
;		TFWSTG	MyFile
;
;	You can write an end of line CR/LF with:
;
;		TFCRLF	MyFile
;
;	You can format and write  decimal or hex integer values from AX:
;
;		mov	ax,ivalue
;		TFWIV	MyFile		OR
;		TFWHIV	MyFile
;
;	You can format and write decimal or hex byte values from AL:
;
;		mov	al,bvalue
;		TFWBV	MyFile		OR
;		TFWHBV	MyFile
;
;	You can format and write a real value from the 8087 stack:
;
;		eld	rvalue
;		TFWRV	MyFile
;
;	You can format and write a 2d point from the 8087 stack:
;
;		eld	X
;		eld	Y
;		TFW2DP	MyFile
;
;	You can format and write a 3d point from the 8087 stack:
;
;		eld	X
;		eld	Y
;		eld	Z
;		TFW3DP	MyFile
;
;	You must write any partial buffer data before closing the
;	file:
;
;		FLUSH	MyFile
;
;	---------------------------------------------------------------
;	TEXT FILE INPUT
;	---------------------------------------------------------------
;
;	Declare a buffer for any text file that you will read with:
;
;		TFBFR	MyFile
;
;	and initialize it with the open file handle:
;
;		mov	ax,FileHandle
;		TFINIT	MyFile
;
;	Then you can read a line of text ended with CR or LF into
;	an ASCIIZ string (CR/LF characters are removed):
;
;		TFRLN	MyFile,MyString
;		jc	short EndOfFile
;
;	Or, by putting the string address in DI:
;
;		mov	di,offset MyString
;		TFRLN	MyFile
;		jc	short EndOfFile
;
;	Each call to TFRLN sets the pointer TFEDP to point to the start
;	of your line buffer.  The following routines scan your line
;	and edit values from it, advancing the TFEDP pointer to point
;	past the delimiting character.
;
;	You can get an integer value into AX with:
;
;		TFGIV
;
;	You can read a single real value to the 8087 stack with:
;
;		TFGRV
;
;	You can read a 2d point X,Y to the 8087 stack with:
;
;		TFG2DP
;
;	And a 3d point X,Y,Z with:
;
;		TFG3DP
;
;	---------------------------------------------------------------
;
	ifndef	Delim			;delimiter defaults to blank
Delim	equ	' '
	endif
;
TFBFR	macro	BfrName
BfrName	dw	0,0,0
	db	256 dup (0)
	endm
;
TFBFRWS	STRUC
FileH	dw	0			;file handle
FileP	dw	0			;next byte pointer
FileC	dw	0			;characters read into buffer
FileB	db	256 dup (0)		;file buffer
tfbfrws	ends
;
TFINIT	macro	Buffer
	mov	Buffer.FileH,ax
	mov	Buffer.FileP,0
	mov	Buffer.FileC,0
	endm
;
;	---------------------------------------------------------------
;	TFWSTG - Write ASCIIZ string [SI] to buffer [DI]
;	---------------------------------------------------------------
;
tfcrlfs	db	13,10,0
;
TFCRLF	macro	buffer
	mov	si,offset tfcrlfs
	mov	di,offset buffer
	call	TFWRS
	endm
;
TFWSTG	macro	buffer,string
	ifnb	<string>
	mov	si,offset string
	endif
	mov	di,offset buffer
	call	TFWRS
	endm
;
TFWEST	macro	buffer,delim
	mov	di,offset buffer
	ifnb	<delim>
	mov	ah,delim
	else
	mov	ah,0
	endif
	call	TFWESS
	endm
;
TFWRS	proc	near
	push	ES
	mov	ax,DS
	mov	ES,ax
	mov	ah,0			;NUL is delimiter
	jmp	short tfwsc
TFWESS:	push	ES
tfwsc:	lea	bx,FileB[di]		;get next store address
	mov	cx,FileP[di]
	add	bx,cx
	cld				;upscan
tfwnc:	mov	al,ES:[si]		;get byte
	inc	si
	cmp	al,ah			;end of string?
	jnz	short tfneos
	mov	FileP[di],cx		;save updated pointer
	pop	ES
	ret
tfneos:	mov	[bx],al			;no - write it to buffer
	inc	bx
	inc	cx			;and count it
	cmp	ch,0			;buffer full yet?
	jz	short tfwnc		;no - get next character
	push	si			;save write scanner
	push	di			;and buffer pointer	
	push	ax			;and delimiter
	mov	ah,40h			;write buffer to file
	mov	bx,FileH[di]
	lea	dx,FileB[di]
	int	21H
	pop	ax			;recover delimiter,
	pop	di			;buffer pointer,
	pop	si			;and write scanner
;;;;	jc	error_handler		;no error handler implemented
	lea	bx,FileB[di]		;next store address
	sub	cx,cx			;reset counter
	jmp	short tfwnc		;get next character
tfwrs	endp
;
;	---------------------------------------------------------------
;	TFWIV - Format an integer value and write it
;	---------------------------------------------------------------
;
tfedi	db	'!2',Delim,0		;single integer value format
tfedhi	db	'!9',Delim,0		;hex word format
tfedb	db	'!3',Delim,0		;single byte format
tfedhb	db	'!8',Delim,0		;hex byte format
tfiv	dw	0			;integer value for edit
tfivl	dw	tfiv			;var list
;
TFWIV	macro	buffer			;integer decimal
	mov	di,offset buffer
	call	TFWI
	endm
;
TFWBV	macro	buffer			;byte decimal
	mov	di,offset buffer
	call	TFWB
	endm
;
TFWHIV	macro	buffer			;hex word 0nnnn
	mov	di,offset buffer
	call	TFWHI
	endm
;
TFWHBV	macro	buffer			;hex byte 0nn
	mov	di,offset buffer
	call	TFWHB
	endm
;
TFWI	proc	near
	mov	si,offset tfedi		;format string for dec int
	jmp	short TFIVAR
TFWHI:	mov	si,offset tfedhi	;fmt stg for hex word
	jmp	short TFIVAR
TFWB:	mov	si,offset tfedb		;fmt stg for dec byte
	jmp	short TFBVAR
TFWHB:	mov	si,offset tfedhb	;fmt stg for hex byte
TFBVAR:	sub	ah,ah			;data is byte only
TFIVAR:	mov	tfiv,ax			;save data
	push	di
	mov	ax,DS			;format integer value
	mov	ES,ax
	mov	dx,offset tfivl		;variable list
	mov	di,offset tfedit	;result string
	SVC	FORMST			;format it
	mov	si,offset tfedit	;write string
	pop	di
	jmp	TFWRS
tfwi	endp
;
;	---------------------------------------------------------------
;	TFWRV - Format a real value on the 8087 stack and write it
;	---------------------------------------------------------------
;
tfedit	db	82 dup (0)		;edit line for formatting
tfed1	db	'!1',Delim,0		;single real value format
tfed2	db	'!1',Delim,'!1',Delim,0			;2d point format
tfed3	db	'!1',Delim,'!1',Delim,'!1',Delim,0	;3d point format
tfrx	dd	0.0			;real values for edit
tfry	dd	0.0
tfrz	dd	0.0
tfvl	dw	tfrx,tfry,tfrz		;var list
;
TFWRV	macro	buffer
	mov	di,offset buffer
	call	TFWR
	endm
;
TFW2DP	macro	buffer
	mov	di,offset buffer
	call	TFW2D
	endm
;
TFW3DP	macro	buffer
	mov	di,offset buffer
	call	TFW3D
	endm
;
TFWR	proc	near
	mov	si,offset tfed1		;format string
	jmp	short tfwcm1
TFW2D:	mov	si,offset tfed2		;format string
	jmp	short tfwcm2
TFW3D:	mov	si,offset tfed3		;format string
	estp	tfrz			;store value to edit
tfwcm2:	estp	tfry
tfwcm1:	estp	tfrx
	push	di
	mov	ax,DS			;format real value
	mov	ES,ax
	mov	dx,offset tfvl		;variable list
	mov	di,offset tfedit	;result string
	ewait
	SVC	FORMST			;format it
	mov	si,offset tfedit	;write string
	pop	di
	jmp	TFWRS
tfwr	endp
;
;	---------------------------------------------------------------
;	FLUSH - Flush Buffer [DI] (Write partial text file buffer)
;	Call this before closing any text write buffer
;	---------------------------------------------------------------
;
FLUSH	macro	buffer
	mov	di,offset buffer
	call	TFBF
	endm
;
TFBF	proc	near
	mov	ah,40h			;write buffer to file
	mov	bx,FileH[di]
	mov	cx,FileP[di]
	jcxz	short tffxt
	lea	dx,FileB[di]
	int	21H
;;;;	jc	error_handler		;no error handler implemented
	mov	FileP[di],0		;and reset pointer
tffxt:	ret
tfbf	endp
;
;	---------------------------------------------------------------
;	TFRLN - Read Text File Line ended by CR,LF, or CR/LF
;	into ASCIIZ string (EOL replaced with NUL)
;	Set carry if end of file
;	---------------------------------------------------------------
;
TFEDP	dw	0			;text line edit pointer
TrmFlg	db	0dh			;Termination flag byte <CR>
;
TFRLN	macro	buffer,string
	ifnb	<string>
	mov	di,offset string
	endif
	mov	si,offset buffer
	call	TFRLIN
	endm
;
TFRLIN	proc	near
	mov	TFEDP,di		;set edit line scanner to start
tfrnxc:	mov	cx,FileC[si]		;get # characters now in buffer
	jcxz	short tfrmty		;any characters now in buffer?
	lea	bx,FileB[si]		;get next read address
	mov	cx,FileP[si]
	add	bx,cx
	mov	al,[bx]			;get character from buffer
	inc	FileP[si]		;advance read pointer
	dec	FileC[si]		;countdown chr read

	cmp	al,13			;CR?
	je	short tfreol		;yes - count as end of line

	cmp	al,10			;LF?
	jne	short tfrRgC		;No, regular char
					;Else, check if LF is
	cmp	di,TFEDP		; first char in a line?
	jne	tfreol			;Else, end of line

	cmp	TrmFlg,0dh		;Else check if following a CR
	je	tfrnxc			;Then ignore and fetch another
	jmp	short tfreol		;Else, end of line

tfrRgC:
	mov	[di],al			;use the character
	inc	di
	jmp	short tfrnxc		;go for next character

tfreol:
	mov	TrmFlg,al		;Save as termination flag
	mov	byte ptr [di],0		;mark end of string
	clc				;line data available
	ret
tfrmty:	push	si			;save buffer pointer
	push	di			;and read scanner	
	mov	ah,3Fh			;read buffer from file
	mov	bx,FileH[si]
	lea	dx,FileB[si]
	mov	cx,256			;# characters desired
	int	21H
	pop	di			;recover read scanner
	pop	si			;and buffer pointer
	jc	short tfreof		;treat read error as eof
	mov	FileC[si],ax		;save actual read count
	mov	FileP[si],0		;reset file scanner
	cmp	ax,0			;read at eof?
	jnz	short tfrnxc
tfreof:	mov	FileC[si],0		;mark buffer empty
	mov	FileP[si],0		;and reset pointer
	stc				;set carry - at end of file
	ret
tfrlin	endp
;
;	---------------------------------------------------------------
;	Input Editing from DS:[TFEDP]
;
;	TFGIV - Integer to AX
;	TFGRV - Real value to 8087 tos
;	TFG2DP - 2d point to 8087 X,Y (tos)
;	TFG3DP - 3d point to 8087 X,Y,Z (tos)
;	---------------------------------------------------------------
;
TFGIV	macro
	mov	ax,DS			;point to string to edit
	mov	ES,ax
	mov	si,TFEDP
	SVC	I2GET			;get integer to AX
	mov	TFEDP,si
	endm
;
TFGRV	macro
	mov	ax,DS			;point to string to edit
	mov	ES,ax
	mov	si,TFEDP
	SVC	R4GET			;get real to 8087
	mov	TFEDP,si
	endm
;
TFG2DP	macro
	mov	ax,DS			;point to string to edit
	mov	ES,ax
	mov	si,TFEDP
	SVC	R4GET			;get real to 8087
	SVC	R4GET			;get real to 8087
	mov	TFEDP,si
	endm
;
TFG3DP	macro
	mov	ax,DS			;point to string to edit
	mov	ES,ax
	mov	si,TFEDP
	SVC	R4GET			;get real to 8087
	SVC	R4GET			;get real to 8087
	SVC	R4GET			;get real to 8087
	mov	TFEDP,si
	endm
