;	I32.CPY - 32-bit integer macros
;
;	(c) Copyright 1988 Evolution Computing
;	All rights reserved.
;	Written by Michael Riddle  8/3/88
;	Comments by Susan Montooth 8/8/91
;
;	Processor/Math chip specification
;
c86	equ	1	;8088/8086/80188/80186
c286	equ	2	;80286
c386	equ	3	;80386
;
e87	equ	0	;8087 emulator
c87	equ	1	;8087
c287	equ	2	;80287
c387	equ	3	;80387
;
cpu	equ	c86	;processor chip
cpu87	equ	c87	;math coprocessor
;
dmove	macro	dest,src		;copies 1 double-word in memory
	if	cpu EQ c386		;to another
	mov	EAX,dword ptr src
	mov	dword ptr dest,EAX
	else
	mov	ax,word ptr src
	mov	word ptr dest,ax
	mov	ax,word ptr src+2
	mov	word ptr dest+2,ax
	endif
	endm
;
dswap	macro	A,B			;swaps 2 double-words in memory
	if	cpu EQ c386
	mov	EAX,dword ptr A
	xchg	EAX,dword ptr B
	mov	dword ptr A,EAX
	else
	mov	AX,word ptr A
	xchg	AX,word ptr B
	mov	word ptr A,AX
	mov	AX,word ptr A+2
	xchg	AX,word ptr B+2
	mov	word ptr A+2,AX
	endif
	endm
;
;	for 16-bit cpu's, the DX:AX accumulator is used.  EAX is used
;	for 32-bit cpu's.
;
;	LA32 - move DX:AX 32-bit number and move to 32-bit accumulator
;
LA32	macro				;DX:AX->32-bit accumulator
	if	cpu EQ c386
	xchg	dx,ax
	shl	EAX,16
	mov	ax,dx
	endif
	endm
;
Z32	macro	var
	ifnb	<var>
	if	cpu EQ c386
	mov	dword ptr var,0
	else
	mov	word ptr var,0		;0->32-bit variable
	mov	word ptr var+2,0
	endif
	else
	if	cpu EQ c386
	sub	EAX,EAX
	else
	sub	ax,ax			;0->32-bit accumulator
	mov	dx,ax
	endif
	endif
	endm
;
LD32	macro	var			;32-bit variable load
	if	cpu EQ c386
	mov	EAX,dword ptr var
	else
	mov	ax,word ptr var
	mov	dx,word ptr var+2
	endif
	endm
;
ST32	macro	var			;32-bit variable store
	if	cpu EQ c386
	mov	dword ptr var,EAX
	else
	mov	word ptr var,ax
	mov	word ptr var+2,dx
	endif
	endm
;
INC32	macro	var			;32-bit variable/accumulator inc
	ifnb	<var>
	if	cpu EQ c386
	inc	dword ptr var
	else
	add	word ptr var,1
	adc	word ptr var+2,0
	endif
	else
	if	cpu EQ c386
	inc	EAX
	else
	add	ax,1
	adc	dx,0
	endif
	endif
	endm
;
ADD16	macro	var			;16-bit var add to 32-bit accum
	if	cpu EQ c386
	sub	EDX,EDX
	mov	dx,word ptr var
	add	EAX,EDX
	else
	add	ax,word ptr var
	adc	dx,0
	endif
	endm
;
ADD32	macro	var			;add 32-bit var to accumulator
	if	cpu EQ c386
	add	EAX,dword ptr var
	else
	add	ax,word ptr var
	adc	dx,word ptr var+2
	endif
	endm
;
SUB32	macro	var			;sub 32-bit var from accumulator
	if	cpu EQ c386
	sub	EAX,dword ptr var
	else
	sub	ax,word ptr var
	sbb	dx,word ptr var+2
	endif
	endm
;
JB32	macro	var,jto			;if acc << var then go jto
	local	nojump
	if	cpu EQ c386
	cmp	EAX,dword ptr var
	else
	cmp	dx,word ptr var+2
	ja	short nojump
	jb	short jto
	cmp	ax,word ptr var
	endif
	jb	short jto
nojump:
	endm
;
JBE32	macro	var,jto			;if acc << var then go jto
	local	nojump
	if	cpu EQ c386
	cmp	EAX,dword ptr var
	else
	cmp	dx,word ptr var+2
	ja	short nojump
	jb	short jto
	cmp	ax,word ptr var
	endif
	jbe	short jto
nojump:
	endm
;
JA32	macro	var,jto			;if acc >>= var then go jto
	local	nojump
	if	cpu EQ c386
	cmp	EAX,dword ptr var
	else
	cmp	dx,word ptr var+2
	ja	short jto
	jb	short nojump
	cmp	ax,word ptr var
	endif
	ja	short jto
nojump:
	endm
;
JAE32	macro	var,jto			;if acc >>= var then go jto
	local	nojump
	if	cpu EQ c386
	cmp	EAX,dword ptr var
	else
	cmp	dx,word ptr var+2
	ja	short jto
	jb	short nojump
	cmp	ax,word ptr var
	endif
	jae	short jto
nojump:
	endm
;
